@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * audiohandler.c
- * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
  *
  * audiohandler.c is free software.
  *
@@ -21,8 +21,10 @@
  *  Boston, MA  02110-1301, USA.
  */
 
-#include <glib/gi18n.h>
 #include "ghbcompat.h"
+
+#include <glib/gi18n.h>
+
 #include "handbrake/handbrake.h"
 #include "settings.h"
 #include "jobdict.h"
@@ -35,10 +37,10 @@
 #include "presets.h"
 
 static void audio_add_to_settings(GhbValue *settings, GhbValue *asettings);
-static void ghb_add_audio_to_ui(signal_user_data_t *ud, const GhbValue *settings);
+static void audio_add_to_ui(signal_user_data_t *ud, const GhbValue *settings);
 static GhbValue* audio_get_selected_settings(signal_user_data_t *ud, int *index);
-static void ghb_clear_audio_list_settings(GhbValue *settings);
-static void ghb_clear_audio_list_ui(GtkBuilder *builder);
+static void clear_audio_list_settings(GhbValue *settings);
+static void clear_audio_list_ui(GtkBuilder *builder);
 
 static gboolean block_updates = FALSE;
 
@@ -213,7 +215,7 @@ audio_deps(signal_user_data_t *ud, GhbValue *asettings, GtkWidget *widget)
     if (widget != NULL)
         ghb_check_dependency(ud, widget, NULL);
 
-    int track = -1, title_id, mix = 0, acodec = 0, sr = 0;
+    int title_id, mix = 0, acodec = 0, sr = 0;
     hb_audio_config_t *aconfig = NULL;
     const hb_title_t *title;
     gboolean qe;
@@ -223,7 +225,7 @@ audio_deps(signal_user_data_t *ud, GhbValue *asettings, GtkWidget *widget)
 
     if (asettings != NULL)
     {
-        track = ghb_dict_get_int(asettings, "Track");
+        int track = ghb_dict_get_int(asettings, "Track");
         acodec = ghb_settings_audio_encoder_codec(asettings, "Encoder");
         aconfig = ghb_get_audio_info(title, track);
         mix = ghb_settings_mixdown_mix(asettings, "Mixdown");
@@ -387,7 +389,8 @@ void ghb_sanitize_audio_track_settings(GhbValue *settings)
     }
 }
 
-void ghb_sanitize_audio_tracks(signal_user_data_t *ud)
+static void
+sanitize_audio_tracks (signal_user_data_t *ud)
 {
     int ii;
     GhbValue *alist = ghb_get_job_audio_list(ud->settings);
@@ -639,8 +642,8 @@ void ghb_audio_title_change(signal_user_data_t *ud, gboolean title_valid)
     gtk_widget_set_sensitive(w, title_valid);
 }
 
-void
-ghb_set_pref_audio_settings(GhbValue *settings)
+static void
+set_pref_audio_settings (GhbValue *settings)
 {
     int       title_id;
     GhbValue *copy_mask;
@@ -855,7 +858,7 @@ ghb_audio_list_refresh_selected(signal_user_data_t *ud)
     GhbValue *asettings = NULL;
     const GhbValue *audio_list;
 
-    g_debug("ghb_audio_list_refresh_selected ()");
+    ghb_log_func();
     tv = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "audio_list_view"));
     ts = gtk_tree_view_get_selection (tv);
     if (gtk_tree_selection_get_selected(ts, &tm, &ti))
@@ -895,7 +898,7 @@ audio_refresh_list_ui(signal_user_data_t *ud)
     count = ghb_array_len(audio_list);
     if (count != tm_count)
     {
-        ghb_clear_audio_list_ui(ud->builder);
+        clear_audio_list_ui(ud->builder);
         for (ii = 0; ii < count; ii++)
         {
             gtk_tree_store_append(GTK_TREE_STORE(tm), &ti, NULL);
@@ -912,12 +915,12 @@ audio_refresh_list_ui(signal_user_data_t *ud)
 void
 ghb_audio_list_refresh_all(signal_user_data_t *ud)
 {
-    ghb_sanitize_audio_tracks(ud);
+    sanitize_audio_tracks(ud);
     audio_refresh_list_ui(ud);
     ghb_update_summary_info(ud);
 }
 
-void
+static void
 audio_update_setting(
     GhbValue           *val,
     const char         *name,
@@ -1042,7 +1045,7 @@ audio_codec_changed_cb(GtkWidget *widget, signal_user_data_t *ud)
 G_MODULE_EXPORT void
 audio_track_changed_cb(GtkWidget *widget, signal_user_data_t *ud)
 {
-    g_debug("audio_track_changed_cb ()");
+    ghb_log_func();
     ghb_widget_to_setting(ud->settings, widget);
     GhbValue *val = ghb_widget_value(widget);
     audio_update_setting(ghb_value_xform(val, GHB_INT), "Track", ud);
@@ -1245,12 +1248,12 @@ gain_widget_changed_cb(GtkWidget *widget, gdouble gain, signal_user_data_t *ud)
     audio_update_setting(ghb_double_value_new(gain), "Gain", ud);
 }
 
-void
-ghb_clear_audio_list_settings(GhbValue *settings)
+static void
+clear_audio_list_settings (GhbValue *settings)
 {
     GhbValue *audio_list;
 
-    g_debug("clear_audio_list_settings ()");
+    ghb_log_func();
     audio_list = ghb_get_job_audio_list(settings);
     ghb_array_reset(audio_list);
 }
@@ -1261,7 +1264,7 @@ ghb_clear_audio_selection(GtkBuilder *builder)
     GtkTreeView *tv;
     GtkTreeSelection *tsel;
 
-    g_debug("clear_audio_list_ui ()");
+    ghb_log_func();
     tv = GTK_TREE_VIEW(GHB_WIDGET(builder, "audio_list_view"));
     // Clear tree selection so that updates are not triggered
     // that cause a recursive attempt to clear the tree selection (crasher)
@@ -1269,14 +1272,14 @@ ghb_clear_audio_selection(GtkBuilder *builder)
     gtk_tree_selection_unselect_all(tsel);
 }
 
-void
-ghb_clear_audio_list_ui(GtkBuilder *builder)
+static void
+clear_audio_list_ui (GtkBuilder *builder)
 {
     GtkTreeView *tv;
     GtkTreeStore *ts;
     GtkTreeSelection *tsel;
 
-    g_debug("clear_audio_list_ui ()");
+    ghb_log_func();
     tv = GTK_TREE_VIEW(GHB_WIDGET(builder, "audio_list_view"));
     ts = GTK_TREE_STORE(gtk_tree_view_get_model(tv));
     // Clear tree selection so that updates are not triggered
@@ -1287,7 +1290,7 @@ ghb_clear_audio_list_ui(GtkBuilder *builder)
 }
 
 static void
-ghb_add_audio_to_ui(signal_user_data_t *ud, const GhbValue *asettings)
+audio_add_to_ui (signal_user_data_t *ud, const GhbValue *asettings)
 {
     GtkTreeView *tv;
     GtkTreeIter ti;
@@ -1315,7 +1318,7 @@ audio_list_selection_changed_cb(GtkTreeSelection *ts, signal_user_data_t *ud)
     GhbValue *asettings = NULL;
     gint row;
 
-    g_debug("audio_list_selection_changed_cb ()");
+    ghb_log_func();
     if (gtk_tree_selection_get_selected(ts, &tm, &ti))
     {
         GtkTreeIter pti;
@@ -1378,7 +1381,7 @@ audio_add_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
     GhbValue *pref_audio = ghb_dict_get_value(ud->settings, "AudioList");
     asettings = audio_select_and_add_track(title, ud->settings, pref_audio,
                                            "und", 0, 0);
-    ghb_add_audio_to_ui(ud, asettings);
+    audio_add_to_ui(ud, asettings);
 
     if (asettings != NULL)
     {
@@ -1413,8 +1416,8 @@ audio_add_all_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
     const hb_title_t *title;
 
     // Add the current audio settings to the list.
-    ghb_clear_audio_list_settings(ud->settings);
-    ghb_clear_audio_list_ui(ud->builder);
+    clear_audio_list_settings(ud->settings);
+    clear_audio_list_ui(ud->builder);
 
     title_id = ghb_dict_get_int(ud->settings, "title");
     title = ghb_lookup_title(title_id, &titleindex);
@@ -1566,8 +1569,8 @@ audio_remove_clicked_cb(GtkWidget *widget, gchar *path, signal_user_data_t *ud)
 G_MODULE_EXPORT void
 audio_reset_clicked_cb(GtkWidget *widget, signal_user_data_t *ud)
 {
-    ghb_set_pref_audio_settings(ud->settings);
-    ghb_sanitize_audio_tracks(ud);
+    set_pref_audio_settings(ud->settings);
+    sanitize_audio_tracks(ud);
     audio_update_dialog_widgets(ud, audio_get_selected_settings(ud, NULL));
     audio_refresh_list_ui(ud);
     ghb_update_summary_info(ud);
@@ -1739,7 +1742,8 @@ audio_def_quality_changed_cb(GtkWidget *w, gdouble quality, signal_user_data_t *
 G_MODULE_EXPORT void
 audio_def_quality_enable_changed_cb(GtkWidget *w, signal_user_data_t *ud);
 
-GtkWidget * ghb_create_audio_settings_row(signal_user_data_t *ud)
+static GtkWidget *
+create_audio_settings_row (signal_user_data_t *ud)
 {
     GtkBox *box, *box2, *box3;
     GtkComboBox *combo;
@@ -2014,7 +2018,7 @@ audio_def_setting_update(signal_user_data_t *ud, GtkWidget *widget)
     }
 }
 
-void
+static void
 audio_add_lang_iter(GtkTreeModel *tm, GtkTreeIter *iter, signal_user_data_t *ud)
 {
     GtkTreeView         * selected;
@@ -2085,7 +2089,7 @@ audio_add_lang_clicked_cb(GtkWidget *widget, signal_user_data_t *ud)
     }
 }
 
-void
+static void
 audio_remove_lang_iter(GtkTreeModel *tm, GtkTreeIter *iter,
                        signal_user_data_t *ud)
 {
@@ -2191,7 +2195,8 @@ static void audio_quality_update_limits(
     gtk_adjustment_configure (adj, value, low, high, gran, gran, 0);
 }
 
-void audio_def_set_limits(signal_user_data_t *ud, GtkWidget *widget, gboolean set_default)
+static void
+audio_def_set_limits (signal_user_data_t *ud, GtkWidget *widget, gboolean set_default)
 {
     GtkListBoxRow *row = audio_settings_get_row(widget);
     gint index = gtk_list_box_row_get_index(row);
@@ -2247,13 +2252,15 @@ void audio_def_set_limits(signal_user_data_t *ud, GtkWidget *widget, gboolean se
     ghb_audio_samplerate_opts_filter(GTK_COMBO_BOX(w), enc);
 }
 
-void audio_def_set_all_limits_cb(GtkWidget *widget, gpointer data)
+static void
+audio_def_set_all_limits_cb (GtkWidget *widget, gpointer data)
 {
     signal_user_data_t *ud = (signal_user_data_t*)data;
     audio_def_set_limits(ud, widget, FALSE);
 }
 
-void audio_def_set_all_limits(signal_user_data_t *ud)
+static void
+audio_def_set_all_limits(signal_user_data_t *ud)
 {
     GtkListBox *list_box;
 
@@ -2368,7 +2375,7 @@ audio_def_setting_add_cb(GtkWidget *widget, signal_user_data_t *ud)
     audio_def_settings_show(GTK_WIDGET(row), TRUE);
 
     // Add new "Add" button
-    widget = ghb_create_audio_settings_row(ud);
+    widget = create_audio_settings_row(ud);
     audio_def_settings_show(widget, FALSE);
     GtkListBox *list_box;
     list_box = GTK_LIST_BOX(GHB_WIDGET(ud->builder, "audio_list_default"));
@@ -2545,12 +2552,12 @@ void ghb_audio_defaults_to_ui(signal_user_data_t *ud)
         GhbValue *adict;
 
         adict = ghb_array_get(alist, ii);
-        widget = ghb_create_audio_settings_row(ud);
+        widget = create_audio_settings_row(ud);
         gtk_list_box_insert(list_box, widget, -1);
         audio_def_update_widgets(widget, adict);
     }
     // Add row with "Add" button
-    widget = ghb_create_audio_settings_row(ud);
+    widget = create_audio_settings_row(ud);
     audio_def_settings_show(widget, FALSE);
     gtk_list_box_insert(list_box, widget, -1);
 }
