@@ -1,45 +1,27 @@
-/*
- * color-scheme.c - Change application theme based on D-Bus
- *
- * Copyright (C) 2022 Rob Hall
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- * MA 02110-1301, USA.
- */
+/* Copyright (C) 2022-2023 HandBrake Team
+ * SPDX-License-Identifier: GPL-2.0-or-later */
+
+#define G_LOG_DOMAIN "ghb"
 
-#include <stdbool.h>
-#include <gtk/gtk.h>
-#include <glib.h>
 #include "color-scheme.h"
 
 #define APP_DARK  APP_PREFERS_DARK /* 1 */
 #define APP_FORCE APP_FORCES_LIGHT /* 2 */
 #define DBUS_TIMEOUT 1000 /* Timeout for D-Bus to respond in ms */
 
-#if GLIB_CHECK_VERSION(2,72,0)
+#if GLIB_CHECK_VERSION(2, 72, 0)
 #define SETTING_CHANGED_SIGNAL "g-signal::SettingChanged"
 #else
 #define SETTING_CHANGED_SIGNAL "g-signal"
 #endif
 
 static AppColorScheme app_scheme = APP_FORCES_LIGHT;
-static GDBusProxy *portal;
+static GDBusProxy *desktop_portal;
 static GtkSettings *settings;
-static gboolean init_done = false;
+static gboolean init_done = FALSE;
 
-static void _set_gtk_theme (gboolean dark)
+static void
+set_gtk_theme (gboolean dark)
 {
     g_object_set(settings, "gtk-application-prefer-dark-theme", dark, NULL);
 }
@@ -47,15 +29,17 @@ static void _set_gtk_theme (gboolean dark)
 /*
  * Returns the currently selected app colour scheme.
  */
-AppColorScheme color_scheme_get_app_scheme (void)
+AppColorScheme
+color_scheme_get_app_scheme (void)
 {
     return app_scheme;
 }
 
 /*
- * Returns true if the app is currently using the dark theme.
+ * Returns TRUE if the app is currently using the dark theme.
  */
-gboolean color_scheme_is_dark_theme (void)
+gboolean
+color_scheme_is_dark_theme (void)
 {
     gboolean is_dark;
 
@@ -69,7 +53,8 @@ gboolean color_scheme_is_dark_theme (void)
  * changed to APP_FORCES_LIGHT or APP_FORCES_DARK, so changes
  * to the desktop color scheme no longer have any effect.
  */
-gboolean color_scheme_toggle (void)
+gboolean
+color_scheme_toggle (void)
 {
     gboolean set_dark = !color_scheme_is_dark_theme();
     g_object_set(settings, "gtk-application-prefer-dark-theme",
@@ -81,11 +66,9 @@ gboolean color_scheme_toggle (void)
     return set_dark;
 }
 
-static void _setting_changed (GDBusProxy *proxy,
-                              char       *sender_name,
-                              char       *signal_name,
-                              GVariant   *parameters,
-                              gpointer    user_data)
+static void
+setting_changed (GDBusProxy *proxy, char *sender_name, char *signal_name,
+                  GVariant *parameters, gpointer user_data)
 {
     const char *namespace_;
     const char *key;
@@ -102,14 +85,14 @@ static void _setting_changed (GDBusProxy *proxy,
           !g_strcmp0(key, "color-scheme"))
     {
         g_variant_get(var, "u", &portal_value);
-        set_dark = (portal_value == DESKTOP_PREFERS_DARK) ? true : false;
-
+        set_dark = (portal_value == DESKTOP_PREFERS_DARK) ? TRUE : FALSE;
         if (!(app_scheme & APP_FORCE))
-            _set_gtk_theme(set_dark);
+            set_gtk_theme(set_dark);
     }
 }
 
-static GDBusProxy *_portal_init (void)
+static GDBusProxy *
+portal_init (void)
 {
     GDBusProxy *portal;
     GError *error = NULL;
@@ -126,7 +109,64 @@ static GDBusProxy *_portal_init (void)
     return portal;
 }
 
-static gboolean _color_scheme_init (void)
+static void
+set_theme (void)
+{
+    DesktopColorScheme desktop_scheme;
+    gboolean set_dark;
+
+    desktop_scheme = color_scheme_get_desktop_scheme();
+    if ((app_scheme & APP_FORCE) || desktop_scheme == DESKTOP_NO_PREFERENCE)
+        set_dark = (app_scheme & APP_DARK);
+    else
+        set_dark = (desktop_scheme == DESKTOP_PREFERS_DARK);
+
+    set_gtk_theme(set_dark);
+}
+
+static void
+init_color_scheme_cb (GObject *object, GAsyncResult *res, gpointer user_data)
+{
+    GDBusProxy *portal;
+    GError *error = NULL;
+
+    portal = g_dbus_proxy_new_for_bus_finish(res, &error);
+
+    if (portal)
+    {
+        g_signal_connect(portal, SETTING_CHANGED_SIGNAL,
+                         G_CALLBACK(setting_changed), NULL);
+    }
+    else
+    {
+        g_debug("Could not access portal: %s", error->message);
+    }
+    init_done = TRUE;
+}
+
+static void
+init_color_scheme_async (void)
+{
+    static size_t init = 0;
+
+    if (g_once_init_enter(&init))
+    {
+        settings = gtk_settings_get_default();
+        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                                  G_DBUS_PROXY_FLAGS_NONE, NULL,
+                                  "org.freedesktop.portal.Desktop",
+                                  "/org/freedesktop/portal/desktop",
+                                  "org.freedesktop.portal.Settings",
+                                  NULL,
+                                  (GAsyncReadyCallback)init_color_scheme_cb,
+                                  NULL);
+        g_once_init_leave(&init, 1);
+    }
+    set_theme();
+}
+
+static gboolean
+init_color_scheme (void)
 {
     static GMutex init_mutex;
 
@@ -135,17 +175,17 @@ static gboolean _color_scheme_init (void)
         if (!init_done)
         {
             settings = gtk_settings_get_default();
-            portal = _portal_init();
-            if (portal)
+            desktop_portal = portal_init();
+            if (desktop_portal)
             {
-                g_signal_connect(portal, SETTING_CHANGED_SIGNAL,
-                                 G_CALLBACK(_setting_changed), NULL);
+                g_signal_connect(desktop_portal, SETTING_CHANGED_SIGNAL,
+                                 G_CALLBACK(setting_changed), NULL);
             }
         }
     }
-    init_done = true;
+    init_done = TRUE;
     g_mutex_unlock(&init_mutex);
-    return true;
+    return TRUE;
 }
 
 /*
@@ -153,7 +193,8 @@ static gboolean _color_scheme_init (void)
  * the org.freedesktop.portal.Settings portal. If the portal
  * is not available, DESKTOP_NO_PREFERENCE is returned.
  */
-DesktopColorScheme color_scheme_get_desktop_scheme (void)
+DesktopColorScheme
+color_scheme_get_desktop_scheme (void)
 {
     g_autoptr (GVariant) outer = NULL;
     g_autoptr (GVariant) inner = NULL;
@@ -161,16 +202,17 @@ DesktopColorScheme color_scheme_get_desktop_scheme (void)
     g_autoptr (GError) error = NULL;
     DesktopColorScheme scheme;
 
-    if (!init_done && !_color_scheme_init())
+    if (!init_done && !init_color_scheme())
         return DESKTOP_NO_PREFERENCE;
 
-    if (!portal)
+    if (!desktop_portal)
         return DESKTOP_NO_PREFERENCE;
 
     GVariant *call = g_variant_new("(ss)", "org.freedesktop.appearance",
                                    "color-scheme");
-    result = g_dbus_proxy_call_sync(portal, "Read", call, G_DBUS_CALL_FLAGS_NONE,
-                                    DBUS_TIMEOUT, NULL, &error);
+    result = g_dbus_proxy_call_sync(desktop_portal, "Read", call,
+                                    G_DBUS_CALL_FLAGS_NONE, DBUS_TIMEOUT,
+                                    NULL, &error);
     if (!result)
     {
         g_debug("%s", error->message);
@@ -189,15 +231,17 @@ DesktopColorScheme color_scheme_get_desktop_scheme (void)
 /*
  * Sets the color scheme to one of the four options.
  * Can be called once GTK has been initialised.
- * Returns false if the color scheme could not be set.
+ * Returns FALSE if the color scheme could not be set.
+ * If the return value is not needed, use color_scheme_set_async instead.
  */
-gboolean color_scheme_set (AppColorScheme scheme)
+gboolean
+color_scheme_set (AppColorScheme scheme)
 {
     gboolean set_dark;
     DesktopColorScheme desktop_scheme;
 
-    if (!init_done && !_color_scheme_init())
-        return false;
+    if (!init_done && !init_color_scheme())
+        return FALSE;
 
     app_scheme = scheme;
     desktop_scheme = color_scheme_get_desktop_scheme();
@@ -207,6 +251,21 @@ gboolean color_scheme_set (AppColorScheme scheme)
     else
         set_dark = (desktop_scheme == DESKTOP_PREFERS_DARK);
 
-    _set_gtk_theme(set_dark);
-    return true;
+    set_gtk_theme(set_dark);
+    return TRUE;
+}
+
+/*
+ * Sets the color scheme to one of the four options.
+ * Can be called once GTK has been initialised.
+ * Runs asynchronously, so preferred over color_scheme_set.
+ */
+void
+color_scheme_set_async (AppColorScheme scheme)
+{
+    app_scheme = scheme;
+    if (init_done)
+        set_theme();
+    else
+        init_color_scheme_async();
 }
