@@ -1,6 +1,6 @@
 /*
  * ghbcompat.h
- * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
  *
  * ghbcompat.h is free software.
  *
@@ -23,6 +23,12 @@
 #if !defined(_GHB_COMPAT_H_)
 #define _GHB_COMPAT_H_
 
+#define G_LOG_USE_STRUCTURED
+#define G_LOG_DOMAIN "ghb"
+
+#define ghb_log_func(x) g_debug("Function: %s", __func__)
+#define ghb_log_func_str(x) g_debug("Function: %s (%s)", __func__, (x))
+
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <string.h>
@@ -33,35 +39,23 @@
 #define GHB_UNSAFE_FILENAME_CHARS "/"
 #endif
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
 #define GHB_ICON_SIZE_BUTTON GTK_ICON_SIZE_NORMAL
 #else
 #define GHB_ICON_SIZE_BUTTON GTK_ICON_SIZE_BUTTON
 #endif
 
-#if !GTK_CHECK_VERSION(3, 10, 0)
-#define gtk_image_set_from_icon_name gtk_image_set_from_stock
-#define GHB_PLAY_ICON "gtk-media-play"
-#define GHB_PAUSE_ICON "gtk-media-pause"
-#else
 #define GHB_PLAY_ICON "media-playback-start"
 #define GHB_PAUSE_ICON "media-playback-pause"
-#endif
 
-#if !GTK_CHECK_VERSION(3, 10, 0)
-#define GHB_STOCK_OPEN      GTK_STOCK_OPEN
-#define GHB_STOCK_CANCEL    GTK_STOCK_CANCEL
-#define GHB_STOCK_SAVE      GTK_STOCK_SAVE
-#else
 #define GHB_STOCK_OPEN      _("_Open")
 #define GHB_STOCK_CANCEL    _("_Cancel")
 #define GHB_STOCK_SAVE      _("_Save")
-#endif
 
 static inline void ghb_widget_get_preferred_width(
     GtkWidget *widget, gint *min_width, gint * natural_width)
 {
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
     GtkRequisition min_req, nat_req;
 
     gtk_widget_get_preferred_size(widget, &min_req, &nat_req);
@@ -81,7 +75,7 @@ static inline void ghb_widget_get_preferred_width(
 static inline void ghb_widget_get_preferred_height(
     GtkWidget *widget, gint *min_height, gint * natural_height)
 {
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
     GtkRequisition min_req, nat_req;
 
     gtk_widget_get_preferred_size(widget, &min_req, &nat_req);
@@ -101,7 +95,7 @@ static inline void ghb_widget_get_preferred_height(
 static inline void ghb_button_set_icon_name(GtkButton *button,
                                             const char * name)
 {
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
     gtk_button_set_icon_name(button, name);
 #else
     GtkImage *image;
@@ -129,7 +123,7 @@ static inline void ghb_get_expand_fill(GtkBox * box, GtkWidget * child,
 
 static inline void ghb_box_append_child(GtkBox * box, GtkWidget * child)
 {
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
     GtkWidget * sibling = NULL;
 
     sibling = gtk_widget_get_last_child(GTK_WIDGET(box));
@@ -146,7 +140,7 @@ static inline void ghb_css_provider_load_from_data(GtkCssProvider *provider,
                                                    const gchar *data,
                                                    gssize length)
 {
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
     gtk_css_provider_load_from_data(provider, data, length);
 #else
     gtk_css_provider_load_from_data(provider, data, length, NULL);
@@ -155,56 +149,35 @@ static inline void ghb_css_provider_load_from_data(GtkCssProvider *provider,
 
 static inline GdkEventType ghb_event_get_event_type(const GdkEvent *event)
 {
-#if GTK_CHECK_VERSION(3, 10, 0)
     return gdk_event_get_event_type(event);
-#else
-    return event->type;
-#endif
 }
 
 static inline gboolean ghb_event_get_keyval(const GdkEvent *event,
                                             guint *keyval)
 {
-#if GTK_CHECK_VERSION(3, 2, 0)
     return gdk_event_get_keyval(event, keyval);
-#else
-    *keyval = ((GdkEventKey*)event)->keyval;
-    return TRUE;
-#endif
 }
 
 static inline gboolean ghb_event_get_button(const GdkEvent *event,
                                             guint *button)
 {
-#if GTK_CHECK_VERSION(3, 2, 0)
     return gdk_event_get_button(event, button);
-#else
-    *keyval = ((GdkEventButton*)event)->button;
-    return TRUE;
-#endif
 }
 
 static inline PangoFontDescription* ghb_widget_get_font(GtkWidget *widget)
 {
     PangoFontDescription *font = NULL;
 
-#if GTK_CHECK_VERSION(3, 0, 0)
     GtkStyleContext *style;
 
     style = gtk_widget_get_style_context(widget);
 
-#if GTK_CHECK_VERSION(3, 8, 0)
     gtk_style_context_get(style, GTK_STATE_FLAG_NORMAL,
                           "font", &font, NULL);
-#else
-    font = gtk_style_context_get_font(style, GTK_STATE_FLAG_NORMAL);
-#endif
-#endif
-
     return font;
 }
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
 typedef GdkSurface      GhbSurface;
 typedef GdkSurfaceHints GhbSurfaceHints;
 
@@ -254,7 +227,6 @@ static inline void ghb_monitor_get_size(GtkWidget *widget, gint *w, gint *h)
 {
     *w = *h = 0;
 
-#if GTK_CHECK_VERSION(3, 22, 0)
     GdkDisplay * display = gtk_widget_get_display(widget);
     GhbSurface * surface = ghb_widget_get_surface(widget);
     if (surface != NULL && display != NULL)
@@ -271,16 +243,6 @@ static inline void ghb_monitor_get_size(GtkWidget *widget, gint *w, gint *h)
             *h = rect.height;
         }
     }
-#else
-    GdkScreen *ss;
-
-    ss = gdk_screen_get_default();
-    if (ss != NULL)
-    {
-        *w = gdk_screen_get_width(ss);
-        *h = gdk_screen_get_height(ss);
-    }
-#endif
 }
 
 static inline gboolean ghb_strv_contains(const char ** strv, const char * str)
@@ -305,7 +267,7 @@ static inline gboolean ghb_strv_contains(const char ** strv, const char * str)
 #endif
 }
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
 
 #define ghb_editable_get_text(e) gtk_editable_get_text(GTK_EDITABLE(e))
 #define ghb_editable_set_text(e,t) gtk_editable_set_text(GTK_EDITABLE(e), (t))
@@ -317,7 +279,7 @@ static inline gboolean ghb_strv_contains(const char ** strv, const char * str)
 
 #endif
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
 static inline void
 ghb_image_set_from_icon_name(GtkImage * image, const gchar * name,
                              GtkIconSize size)
@@ -377,7 +339,7 @@ ghb_scale_button_new(gdouble min, gdouble max, gdouble step,
 }
 #endif
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
 static inline void ghb_drag_status(GdkDrop * ctx, GdkDragAction action,
                                    guint32 time)
 {
@@ -391,7 +353,7 @@ static inline void ghb_drag_status(GdkDragContext * ctx, GdkDragAction action,
 }
 #endif
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
 static inline void ghb_entry_set_width_chars(GtkEntry * entry, gint n_chars)
 {
     gtk_editable_set_width_chars(GTK_EDITABLE(entry), n_chars);
@@ -403,7 +365,7 @@ static inline void ghb_entry_set_width_chars(GtkEntry * entry, gint n_chars)
 }
 #endif
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
 static inline GdkAtom ghb_atom_string(const char * str)
 {
     return g_intern_static_string(str);
@@ -428,4 +390,44 @@ static inline GtkGesture *ghb_gesture_click_new (GtkWidget *widget)
     return gesture;
 }
 
+static inline void
+ghb_file_chooser_set_initial_file (GtkFileChooser *chooser, const char *file)
+{
+    if (!file || !file[0])
+    {
+        return;
+    }
+    else if (g_file_test(file, G_FILE_TEST_IS_DIR))
+    {
+#if GTK_CHECK_VERSION(4, 4, 0)
+        GFile *gfile = g_file_new_for_path(file);
+        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser), gfile, NULL);
+        g_object_unref(gfile);
+#else
+        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser), file);
+#endif
+    }
+    else
+    {
+        char *dir = g_path_get_dirname(file);
+        if (g_file_test(dir, G_FILE_TEST_IS_DIR))
+        {
+#if GTK_CHECK_VERSION(4, 4, 0)
+            GFile *gfile = g_file_new_for_path(dir);
+            gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser), gfile, NULL);
+            g_object_unref(gfile);
+#else
+            gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser), dir);
+#endif
+        }
+        g_free(dir);
+        if (gtk_file_chooser_get_action(chooser) == GTK_FILE_CHOOSER_ACTION_SAVE)
+        {
+            char *base = g_path_get_basename(file);
+            gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(chooser), base);
+            g_free(base);
+        }
+    }
+}
+
 #endif // _GHB_COMPAT_H_
