@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * main.c
- * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
  *
  * main.c is free software.
  *
@@ -28,8 +28,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
+#include <locale.h>
 
-#include <config.h>
+#ifndef _WIN32
+#include <sys/utsname.h>
+#endif
+
+#include "config.h"
 
 #include "ghbcompat.h"
 
@@ -62,6 +67,7 @@
 #include "preview.h"
 #include "ui_res.h"
 #include "color-scheme.h"
+#include "power-manager.h"
 
 
 /*
@@ -77,17 +83,16 @@
 #  define N_(String) (String)
 #endif
 
-
 #define BUILDER_NAME "ghb"
 
-GtkBuilder*
+static GtkBuilder*
 create_builder_or_die(const gchar * name)
 {
     GtkWidget *dialog;
     GtkBuilder *xml;
     GError *error = NULL;
 
-    g_debug("create_builder_or_die()\n");
+    ghb_log_func();
     xml = gtk_builder_new();
     gtk_builder_add_from_resource(xml, "/fr/handbrake/ghb/ui/ghb.ui", &error);
     if (!error)
@@ -146,12 +151,12 @@ MyConnect(
 
     //const gchar *name = ghb_get_setting_key((GtkWidget*)object);
     //g_message("\n\nname %s", name);
-    g_debug("handler_name %s", handler_name);
-    g_debug("signal_name %s", signal_name);
+    g_debug("Handler: %s", handler_name);
+    g_debug("Signal: %s", signal_name);
     callback = self_symbol_lookup(handler_name);
     if (!callback)
     {
-        g_message("Signal handler (%s) not found", handler_name);
+        g_warning("Signal handler (%s) not found", handler_name);
         return;
     }
     if (connect_object)
@@ -185,7 +190,7 @@ change_font(GtkWidget *widget, gpointer data)
     if (font_desc == NULL) exit(1);
     gtk_widget_modify_font(widget, font_desc);
     name = ghb_get_setting_key(widget);
-    g_debug("changing font for widget %s\n", name);
+    g_debug("changing font for widget %s", name);
     if (GTK_IS_CONTAINER(widget))
     {
         gtk_container_foreach((GtkContainer*)widget, change_font, data);
@@ -194,10 +199,6 @@ change_font(GtkWidget *widget, gpointer data)
     //gtk_container_foreach((GtkContainer*)window, change_font, "sans 20");
 #endif
 
-extern G_MODULE_EXPORT void audio_list_selection_changed_cb(void);
-extern G_MODULE_EXPORT void audio_edit_clicked_cb(void);
-extern G_MODULE_EXPORT void audio_remove_clicked_cb(void);
-
 // Create and bind the tree model to the tree view for the audio track list
 // Also, connect up the signal that lets us know the selection has changed
 static void
@@ -213,7 +214,7 @@ bind_audio_tree_model(signal_user_data_t *ud)
     GtkTreeView  *treeview;
     GtkTreeSelection *selection;
 
-    g_debug("bind_audio_tree_model()\n");
+    ghb_log_func();
     treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "audio_list_view"));
     selection = gtk_tree_view_get_selection(treeview);
     treestore = gtk_tree_store_new(6, G_TYPE_STRING, G_TYPE_STRING,
@@ -252,17 +253,13 @@ bind_audio_tree_model(signal_user_data_t *ud)
     //gtk_tree_view_column_set_min_width(column, 24);
     gtk_tree_view_append_column(treeview, GTK_TREE_VIEW_COLUMN(column));
 
-    g_signal_connect(selection, "changed", audio_list_selection_changed_cb, ud);
-    g_signal_connect(edit_cell, "clicked", audio_edit_clicked_cb, ud);
-    g_signal_connect(delete_cell, "clicked", audio_remove_clicked_cb, ud);
+    g_signal_connect(selection, "changed", G_CALLBACK(audio_list_selection_changed_cb), ud);
+    g_signal_connect(edit_cell, "clicked", G_CALLBACK(audio_edit_clicked_cb), ud);
+    g_signal_connect(delete_cell, "clicked", G_CALLBACK(audio_remove_clicked_cb), ud);
 
-    g_debug("Done\n");
+    g_debug("Done");
 }
 
-extern G_MODULE_EXPORT void subtitle_list_selection_changed_cb(void);
-extern G_MODULE_EXPORT void subtitle_edit_clicked_cb(void);
-extern G_MODULE_EXPORT void subtitle_remove_clicked_cb(void);
-
 // Create and bind the tree model to the tree view for the subtitle track list
 // Also, connect up the signal that lets us know the selection has changed
 static void
@@ -278,7 +275,7 @@ bind_subtitle_tree_model(signal_user_data_t *ud)
     GtkTreeView  *treeview;
     GtkTreeSelection *selection;
 
-    g_debug("bind_subtitle_tree_model()\n");
+    ghb_log_func();
     treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "subtitle_list_view"));
     selection = gtk_tree_view_get_selection(treeview);
     treestore = gtk_tree_store_new(6, G_TYPE_STRING, G_TYPE_STRING,
@@ -315,18 +312,12 @@ bind_subtitle_tree_model(signal_user_data_t *ud)
                                     _(" "), delete_cell, "icon-name", 4, NULL);
     gtk_tree_view_append_column(treeview, GTK_TREE_VIEW_COLUMN(column));
 
-    g_signal_connect(selection, "changed", subtitle_list_selection_changed_cb, ud);
-    g_signal_connect(edit_cell, "clicked", subtitle_edit_clicked_cb, ud);
-    g_signal_connect(delete_cell, "clicked", subtitle_remove_clicked_cb, ud);
+    g_signal_connect(selection, "changed", G_CALLBACK(subtitle_list_selection_changed_cb), ud);
+    g_signal_connect(edit_cell, "clicked", G_CALLBACK(subtitle_edit_clicked_cb), ud);
+    g_signal_connect(delete_cell, "clicked", G_CALLBACK(subtitle_remove_clicked_cb), ud);
 }
 
-extern G_MODULE_EXPORT void presets_list_selection_changed_cb(void);
-extern G_MODULE_EXPORT void presets_drag_data_received_cb(void);
-extern G_MODULE_EXPORT void presets_drag_motion_cb(void);
-extern G_MODULE_EXPORT void preset_edited_cb(void);
-extern void presets_row_expanded_cb(void);
-
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
 static const char * presets_drag_entries[] = {
     "widget/presets-list-row-drop"
 };
@@ -347,7 +338,7 @@ bind_presets_tree_model(signal_user_data_t *ud)
     GtkTreeView  *treeview;
     GtkTreeSelection *selection;
 
-    g_debug("bind_presets_tree_model()\n");
+    ghb_log_func();
     treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "presets_list"));
     selection = gtk_tree_view_get_selection(treeview);
     treestore = gtk_tree_store_new(5, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT,
@@ -358,13 +349,13 @@ bind_presets_tree_model(signal_user_data_t *ud)
     column = gtk_tree_view_column_new_with_attributes(_("Preset Name"), cell,
         "text", 0, "weight", 1, "style", 2, "editable", 4, NULL);
 
-    g_signal_connect(cell, "edited", preset_edited_cb, ud);
+    g_signal_connect(cell, "edited", G_CALLBACK(preset_edited_cb), ud);
 
     gtk_tree_view_append_column(treeview, GTK_TREE_VIEW_COLUMN(column));
     gtk_tree_view_column_set_expand(column, TRUE);
     gtk_tree_view_set_tooltip_column(treeview, 3);
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
     GdkContentFormats * targets;
 
     targets = gdk_content_formats_new(presets_drag_entries,
@@ -381,16 +372,16 @@ bind_presets_tree_model(signal_user_data_t *ud)
                                            GDK_ACTION_MOVE);
 #endif
 
-    g_signal_connect(treeview, "drag_data_received", presets_drag_data_received_cb, ud);
-    g_signal_connect(treeview, "drag_motion", presets_drag_motion_cb, ud);
-    g_signal_connect(treeview, "row_expanded", presets_row_expanded_cb, ud);
-    g_signal_connect(treeview, "row_collapsed", presets_row_expanded_cb, ud);
-    g_signal_connect(selection, "changed", presets_list_selection_changed_cb, ud);
-    g_debug("Done\n");
+    g_signal_connect(treeview, "drag_data_received", G_CALLBACK(presets_drag_data_received_cb), ud);
+    g_signal_connect(treeview, "drag_motion", G_CALLBACK(presets_drag_motion_cb), ud);
+    g_signal_connect(treeview, "row_expanded", G_CALLBACK(presets_row_expanded_cb), ud);
+    g_signal_connect(treeview, "row_collapsed", G_CALLBACK(presets_row_expanded_cb), ud);
+    g_signal_connect(selection, "changed", G_CALLBACK(presets_list_selection_changed_cb), ud);
+    g_debug("Done");
 }
 
 static void
-clean_old_logs()
+clean_old_logs (void)
 {
 #if !defined(_WIN32)
     const gchar *file;
@@ -518,7 +509,7 @@ IoRedirect(signal_user_data_t *ud)
     // Open activity log.
     config = ghb_get_user_config_dir(NULL);
     pid = getpid();
-    path = g_strdup_printf("%s/Activity.log.%"PRId64, config, (long)pid);
+    path = g_strdup_printf("%s/Activity.log.%"PRId64, config, (int64_t)pid);
     ud->activity_log = g_io_channel_new_file (path, "w", NULL);
     ud->job_activity_log = NULL;
     str = g_strdup_printf("<big><b>%s</b></big>", path);
@@ -552,13 +543,9 @@ IoRedirect(signal_user_data_t *ud)
     g_io_channel_set_encoding(channel, NULL, NULL);
     ud->stderr_src_id =
         g_io_add_watch(channel, G_IO_IN, ghb_log_cb, (gpointer)ud );
-}
 
-typedef struct
-{
-    gchar *filename;
-    gchar *iconname;
-} icon_map_t;
+    g_io_channel_unref(channel);
+}
 
 static gchar *dvd_device = NULL;
 static gchar *arg_preset = NULL;
@@ -597,7 +584,7 @@ win_message_cb(GdkXEvent *wmevent, GdkEvent *event, gpointer data)
 }
 #endif
 
-void
+static void
 watch_volumes(signal_user_data_t *ud)
 {
 #if !defined(_WIN32)
@@ -674,21 +661,17 @@ const gchar *MyCSS =
 "    padding: 0px 0px 0px 0px;"
 "}"
 
-#if GTK_CHECK_VERSION(3, 20, 0)
 "stackswitcher button.text-button"
 "{"
 "    min-width: 50px;"
 "}"
-#endif
 
-#if GTK_CHECK_VERSION(3, 16, 0)
 "#activity_view"
 "{"
 "    font-family: monospace;"
 "    font-size: 8pt;"
 "    font-weight: 300;"
 "}"
-#endif
 
 ".row:not(:first-child)"
 "{"
@@ -733,6 +716,12 @@ const gchar *MyCSS =
 "{"
 "    border-bottom: 1px solid #4e9a06; "
 "}"
+#if !GTK_CHECK_VERSION(3, 24, 0)
+"toolbar"
+"{"
+"    -gtk-icon-style: symbolic; "
+"}"
+#endif
 ;
 
 extern G_MODULE_EXPORT void status_icon_query_tooltip_cb(void);
@@ -747,7 +736,7 @@ dvd_source_activate_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
 source_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
-single_title_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+source_dir_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
 destination_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
@@ -755,9 +744,11 @@ preferences_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
 quit_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
-queue_add_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+title_add_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
-queue_add_all_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+title_add_multiple_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+title_add_all_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
 queue_start_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
@@ -837,18 +828,23 @@ G_MODULE_EXPORT void
 preset_select_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 G_MODULE_EXPORT void
 preset_reload_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+chapters_export_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+chapters_import_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 
 static void map_actions(GApplication * app, signal_user_data_t * ud)
 {
     const GActionEntry entries[] =
     {
         { "source",                source_action_cb                },
-        { "single",                single_title_action_cb          },
+        { "source-dir",            source_dir_action_cb            },
         { "destination",           destination_action_cb           },
         { "preferences",           preferences_action_cb           },
         { "quit",                  quit_action_cb                  },
-        { "queue-add",             queue_add_action_cb             },
-        { "queue-add-all",         queue_add_all_action_cb         },
+        { "add-current",           title_add_action_cb             },
+        { "add-multiple",          title_add_multiple_action_cb    },
+        { "add-all",               title_add_all_action_cb         },
         { "queue-start",           queue_start_action_cb           },
         { "queue-pause",           queue_pause_action_cb           },
         { "queue-play-file",       queue_play_file_action_cb       },
@@ -870,14 +866,10 @@ static void map_actions(GApplication * app, signal_user_data_t * ud)
         { "dvd-open",              dvd_source_activate_cb, "s"     },
         { "hbfd",                  NULL,
           NULL, "false",           hbfd_action_cb                  },
-        { "show-presets",          NULL,
-          NULL, "false",           show_presets_action_cb          },
-        { "show-queue",            NULL,
-          NULL, "false",           show_queue_action_cb            },
-        { "show-preview",          NULL,
-          NULL, "false",           show_preview_action_cb          },
-        { "show-activity",         NULL,
-          NULL, "false",           show_activity_action_cb         },
+        { "show-presets",          show_presets_action_cb          },
+        { "show-queue",            show_queue_action_cb            },
+        { "show-preview",          show_preview_action_cb          },
+        { "show-activity",         show_activity_action_cb         },
         { "preset-save",           preset_save_action_cb           },
         { "preset-save-as",        preset_save_as_action_cb        },
         { "preset-rename",         preset_rename_action_cb         },
@@ -892,6 +884,8 @@ static void map_actions(GApplication * app, signal_user_data_t * ud)
         { "preset-reload",         preset_reload_action_cb         },
         { "preview-fullscreen",    NULL,
           NULL, "false",           preview_fullscreen_action_cb    },
+        { "chapters-export",       chapters_export_action_cb       },
+        { "chapters-import",       chapters_import_action_cb       },
     };
     g_action_map_add_action_entries(G_ACTION_MAP(app), entries,
                                     G_N_ELEMENTS(entries), ud);
@@ -941,7 +935,8 @@ extern G_MODULE_EXPORT void on_presets_list_press_cb (GtkGesture *gest, gint n_p
                                                       gdouble y, signal_user_data_t *ud);
 extern G_MODULE_EXPORT void queue_button_press_cb (GtkGesture *gest, gint n_press, gdouble x,
                                                    gdouble y, signal_user_data_t *ud);
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
+extern G_MODULE_EXPORT void easter_egg_multi_cb(void);
 extern G_MODULE_EXPORT void preview_leave_cb(void);
 extern G_MODULE_EXPORT void preview_motion_cb(void);
 extern G_MODULE_EXPORT void preview_draw_cb(GtkDrawingArea*, cairo_t*, int, int,
@@ -950,15 +945,94 @@ extern G_MODULE_EXPORT void hud_enter_cb(void);
 extern G_MODULE_EXPORT void hud_leave_cb(void);
 #endif
 
+static void
+video_file_drop_received (GtkWidget *widget, GdkDragContext *context,
+                          int x, int y, GtkSelectionData *data, guint info,
+                          guint time, signal_user_data_t *ud)
+{
+    gchar **uris;
+    GFile *file = NULL;
+    gchar *filename = NULL;
+
+    uris = gtk_selection_data_get_uris(data);
+    if (uris != NULL)
+    {
+        // Only one file or folder at a time is supported
+        file = g_file_new_for_uri(uris[0]);
+        g_strfreev(uris);
+    }
+    if (file != NULL)
+    {
+        filename = g_file_get_path(file);
+        g_object_unref(file);
+    }
+    if (filename != NULL)
+    {
+        g_debug("File dropped on window: %s", filename);
+        ghb_dict_set_string(ud->prefs, "default_source", filename);
+        ghb_pref_save(ud->prefs, "default_source");
+        ghb_dvd_set_current(filename, ud);
+        ghb_do_scan(ud, filename, 0, TRUE);
+        g_free(filename);
+    }
+}
+
+static void
+video_file_drop_init (signal_user_data_t *ud)
+{
+    GtkWidget *summary = GHB_WIDGET (ud->builder, "hb_window");
+
+    gtk_drag_dest_set(summary, GTK_DEST_DEFAULT_ALL, NULL, 0, GDK_ACTION_COPY);
+    gtk_drag_dest_add_uri_targets(summary);
+    g_signal_connect(summary, "drag-data-received", G_CALLBACK(video_file_drop_received), ud);
+}
+
+static void
+print_system_information (void)
+{
+#if GLIB_CHECK_VERSION(2, 64, 0)
+    char *os_info = g_get_os_info(G_OS_INFO_KEY_PRETTY_NAME);
+    fprintf(stderr, "OS: %s\n", os_info);
+    g_free(os_info);
+#endif
+#ifndef _WIN32
+    char *exe_path;
+    ssize_t result;
+    struct utsname *host_info;
+
+    host_info = calloc(1, sizeof(struct utsname));
+    uname(host_info);
+
+    fprintf(stderr, "%s\n", HB_PROJECT_TITLE);
+    fprintf(stderr, "Kernel: %s %s (%s)\n", host_info->sysname,
+            host_info->release, host_info->machine);
+    fprintf(stderr, "CPU: %s x %d\n", hb_get_cpu_name(), hb_get_cpu_count());
+    free(host_info);
+    exe_path = calloc(1, PATH_MAX);
+    result = readlink( "/proc/self/exe", exe_path, PATH_MAX);
+    if (result > 0)
+    {
+        char *exe_dirname = g_path_get_dirname(exe_path);
+        fprintf(stderr, "Install Dir: %s\n", exe_dirname);
+        g_free(exe_dirname);
+    }
+    free(exe_path);
+#endif
+    char *config_dirname = ghb_get_user_config_dir(NULL);
+    fprintf(stderr, "Config Dir:  %s\n", config_dirname);
+    fprintf(stderr, "_______________________________\n\n");
+    g_free(config_dirname);
+}
+
 extern G_MODULE_EXPORT void
 ghb_activate_cb(GApplication * app, signal_user_data_t * ud)
 {
     GtkCssProvider     * provider = gtk_css_provider_new();
 
     ghb_css_provider_load_from_data(provider, MyCSS, -1);
-    color_scheme_set(APP_PREFERS_LIGHT);
+    color_scheme_set_async(APP_PREFERS_LIGHT);
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
     GdkDisplay *dd = gdk_display_get_default();
     gtk_style_context_add_provider_for_display(dd,
                                 GTK_STYLE_PROVIDER(provider),
@@ -975,8 +1049,6 @@ ghb_activate_cb(GApplication * app, signal_user_data_t * ud)
     ghb_resource_init();
     ghb_load_icons();
 
-    ghb_udev_init();
-
     // Override user config dir
     if (arg_config_dir != NULL)
     {
@@ -989,10 +1061,9 @@ ghb_activate_cb(GApplication * app, signal_user_data_t * ud)
     // connect shutdown signal for cleanup
     g_signal_connect(app, "shutdown", (GCallback)ghb_shutdown_cb, ud);
 
-    ud->debug = ghb_debug;
-    g_log_set_handler(NULL, G_LOG_LEVEL_DEBUG, debug_log_handler, ud);
-    g_log_set_handler("Gtk", G_LOG_LEVEL_WARNING, warn_log_handler, ud);
-
+#if GLIB_CHECK_VERSION(2, 72, 0)
+    g_log_set_debug_enabled(ghb_debug);
+#endif
     ud->globals = ghb_dict_new();
     ud->prefs = ghb_dict_new();
     ud->settings_array = ghb_array_new();
@@ -1001,19 +1072,18 @@ ghb_activate_cb(GApplication * app, signal_user_data_t * ud)
 
     ud->builder = create_builder_or_die(BUILDER_NAME);
 
+    // Initialize D-Bus connections to monitor power settings
+    ghb_power_manager_init(ud);
+
     // Enable drag & drop in queue list
     ghb_queue_drag_n_drop_init(ud);
+    video_file_drop_init(ud);
 
     // Enable events that alert us to media change events
     watch_volumes(ud);
 
     GtkWidget *widget;
 
-#if defined(_NO_UPDATE_CHECK)
-    widget = GHB_WIDGET(ud->builder, "check_updates_box");
-    gtk_widget_hide(widget);
-#endif
-
     // Get GtkTextBuffers for activity logs
     widget = GHB_WIDGET(ud->builder, "activity_view");
     ud->activity_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
@@ -1026,21 +1096,12 @@ ghb_activate_cb(GApplication * app, signal_user_data_t * ud)
     // Must set the names of the widgets that I want to modify
     // style for.
     gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_hud"), "preview_hud");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_frame"), "preview_frame");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "live_preview_play"), "live_preview_play");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "live_preview_fullscreen"), "live_preview_fullscreen");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "live_preview_progress"), "live_preview_progress");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "live_encode_progress"), "live_encode_progress");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "live_duration"), "live_duration");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_show_crop"), "preview_show_crop");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_reset"), "preview_reset");
     gtk_widget_set_name(GHB_WIDGET(ud->builder, "activity_view"), "activity_view");
 
     // Redirect stderr to the activity window
     ghb_preview_init(ud);
     IoRedirect(ud);
-    ghb_log( "%s - %s - %s",
-        HB_PROJECT_TITLE, HB_PROJECT_HOST_TITLE, HB_PROJECT_URL_WEBSITE );
+    print_system_information();
     ghb_init_dep_map();
 
     GtkTextView   * textview;
@@ -1060,8 +1121,8 @@ ghb_activate_cb(GApplication * app, signal_user_data_t * ud)
     // Set up UI combo boxes.  Some of these rely on HB global settings.
     ghb_combo_init(ud);
 
-    g_debug("ud %p\n", ud);
-    g_debug("ud->builder %p\n", ud->builder);
+    g_debug("ud %p", ud);
+    g_debug("ud->builder %p", ud->builder);
 
     bind_audio_tree_model(ud);
     bind_subtitle_tree_model(ud);
@@ -1138,36 +1199,6 @@ ghb_activate_cb(GApplication * app, signal_user_data_t * ud)
     window_width = ghb_dict_get_int(ud->prefs, "window_width");
     window_height = ghb_dict_get_int(ud->prefs, "window_height");
 
-    /*
-     * Filter objects in GtkBuilder xml
-     * Add filters to source chooser
-     */
-    GtkFileChooser *chooser;
-    chooser = GTK_FILE_CHOOSER(GHB_WIDGET(ud->builder, "source_dialog"));
-    ghb_add_file_filter(chooser, ud, _("All"), "FilterAll");
-    ghb_add_file_filter(chooser, ud, _("Video"), "SourceFilterVideo");
-    ghb_add_file_filter(chooser, ud, "TS", "SourceFilterTS");
-    ghb_add_file_filter(chooser, ud, "MPG", "SourceFilterMPG");
-    ghb_add_file_filter(chooser, ud, "EVO", "SourceFilterEVO");
-    ghb_add_file_filter(chooser, ud, "VOB", "SourceFilterVOB");
-    ghb_add_file_filter(chooser, ud, "MKV", "SourceFilterMKV");
-    ghb_add_file_filter(chooser, ud, "WebM", "SourceFilterWebM");
-    ghb_add_file_filter(chooser, ud, "MP4", "SourceFilterMP4");
-    ghb_add_file_filter(chooser, ud, "MOV", "SourceFilterMOV");
-    ghb_add_file_filter(chooser, ud, "AVI", "SourceFilterAVI");
-    ghb_add_file_filter(chooser, ud, "OGG", "SourceFilterOGG");
-    ghb_add_file_filter(chooser, ud, "FLV", "SourceFilterFLV");
-    ghb_add_file_filter(chooser, ud, "WMV", "SourceFilterWMV");
-
-
-#if !GTK_CHECK_VERSION(3, 16, 0)
-    PangoFontDescription *font_desc;
-    font_desc = pango_font_description_from_string("monospace 10");
-    textview = GTK_TEXT_VIEW(GHB_WIDGET(ud->builder, "activity_view"));
-    gtk_widget_override_font(GTK_WIDGET(textview), font_desc);
-    pango_font_description_free(font_desc);
-#endif
-
     // Grrrr!  Gtk developers !!!hard coded!!! the width of the
     // radio buttons in GtkStackSwitcher to 100!!!
     //
@@ -1233,7 +1264,7 @@ ghb_activate_cb(GApplication * app, signal_user_data_t * ud)
     gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(gest), 0);
     g_signal_connect(gest, "pressed", G_CALLBACK(queue_button_press_cb), ud);
 
-#if GTK_CHECK_VERSION(3, 90, 0)
+#if GTK_CHECK_VERSION(4, 4, 0)
     GtkEventController * econ;
 
     // Preview HUD popup management via mouse motion
@@ -1294,9 +1325,9 @@ main(int argc, char *argv[])
         // Enable console logging
         if(AttachConsole(ATTACH_PARENT_PROCESS) || AllocConsole()){
             close(STDOUT_FILENO);
-            freopen("CONOUT$", "w", stdout);
+            (void) freopen("CONOUT$", "w", stdout);
             close(STDERR_FILENO);
-            freopen("CONOUT$", "w", stderr);
+            (void) freopen("CONOUT$", "w", stderr);
         }
     }
     else
@@ -1347,10 +1378,13 @@ main(int argc, char *argv[])
     if (ud->builder != NULL)
         g_object_unref(ud->builder);
 
+    ghb_power_manager_dispose(ud);
+
     g_object_unref(ud->extra_activity_buffer);
     g_object_unref(ud->queue_activity_buffer);
     g_object_unref(ud->activity_buffer);
     g_free(ud->extra_activity_path);
+    ghb_preview_dispose(ud);
 
     g_free(ud->current_dvd_device);
     g_free(ud);
