@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * settings.c
- * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
  *
  * settings.c is free software.
  *
@@ -21,12 +21,13 @@
  *  Boston, MA  02110-1301, USA.
  */
 
+#include "ghbcompat.h"
+
 #include <fcntl.h>
 #include <unistd.h>
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <string.h>
-#include "ghbcompat.h"
 #include "settings.h"
 #include "hb-backend.h"
 #include "values.h"
@@ -37,7 +38,7 @@ void ghb_pref_audio_init(signal_user_data_t *ud);
 GObject*
 debug_get_object(GtkBuilder* b, const gchar *n)
 {
-    g_message("name %s\n", n);
+    g_debug("name %s\n", n);
     return gtk_builder_get_object(b, n);
 }
 
@@ -67,7 +68,7 @@ ghb_get_setting_key(GtkWidget *widget)
 {
     const gchar *name;
 
-    g_debug("get_setting_key ()\n");
+    ghb_log_func();
     if (widget == NULL) return NULL;
     g_return_val_if_fail(GTK_IS_WIDGET(widget), NULL);
     name = gtk_buildable_get_name(GTK_BUILDABLE(widget));
@@ -79,7 +80,7 @@ ghb_get_setting_key(GtkWidget *widget)
     if (name == NULL)
     {
         // Bad widget pointer?  Should never happen.
-        g_debug("Bad widget\n");
+        g_debug("Bad widget");
         return NULL;
     }
     return name;
@@ -94,7 +95,7 @@ ghb_widget_value(GtkWidget *widget)
 
     if (widget == NULL)
     {
-        g_debug("NULL widget\n");
+        g_debug("NULL widget");
         return NULL;
     }
 
@@ -108,7 +109,7 @@ ghb_widget_value(GtkWidget *widget)
     else if (type == GTK_TYPE_RADIO_BUTTON)
     {
         gboolean bval;
-#if !GTK_CHECK_VERSION(3, 90, 0)
+#if !GTK_CHECK_VERSION(4, 4, 0)
         bval = gtk_toggle_button_get_inconsistent(GTK_TOGGLE_BUTTON(widget));
         if (bval)
         {
@@ -305,7 +306,7 @@ ghb_widget_to_setting(GhbValue *settings, GtkWidget *widget)
     GhbValue *value;
 
     if (widget == NULL) return;
-    g_debug("ghb_widget_to_setting");
+    ghb_log_func();
     // Find corresponding setting
     key = ghb_get_setting_key(widget);
     if (key == NULL) return;
@@ -316,7 +317,7 @@ ghb_widget_to_setting(GhbValue *settings, GtkWidget *widget)
     }
     else
     {
-        g_debug("No value found for %s\n", key);
+        g_debug("No value found for %s", key);
     }
 }
 
@@ -455,44 +456,9 @@ ghb_update_widget(GtkWidget *widget, const GhbValue *value)
     }
     else if (type == GTK_TYPE_FILE_CHOOSER_BUTTON)
     {
-        GtkFileChooserAction act;
-        act = gtk_file_chooser_get_action(GTK_FILE_CHOOSER(widget));
-
-        if (str[0] == 0)
+        if (str[0] != 0)
         {
-            // Do nothing
-            ;
-        }
-        else if (act == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER ||
-                 act == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER)
-        {
-            gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(widget), str);
-        }
-        else if (act == GTK_FILE_CHOOSER_ACTION_SAVE)
-        {
-            gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(widget), str);
-        }
-        else
-        {
-            if (g_file_test(str, G_FILE_TEST_IS_DIR))
-            {
-                gtk_file_chooser_set_current_folder(
-                    GTK_FILE_CHOOSER(widget), str);
-            }
-            else if (g_file_test(str, G_FILE_TEST_EXISTS))
-            {
-                gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(widget), str);
-            }
-            else
-            {
-                gchar * dirname;
-
-                dirname = g_path_get_dirname(str);
-                gtk_file_chooser_set_current_folder(
-                    GTK_FILE_CHOOSER(widget), dirname);
-                gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(widget));
-                g_free(dirname);
-            }
+            ghb_file_chooser_set_initial_file(GTK_FILE_CHOOSER(widget), str);
         }
     }
     else
@@ -508,7 +474,7 @@ ghb_ui_update_from_settings(signal_user_data_t *ud, const gchar *name, const Ghb
     GObject *object;
     GhbValue * value;
 
-    g_debug("ghb_ui_update_from_settings() %s", name);
+    ghb_log_func_str(name);
     if (name == NULL)
         return 0;
     value = ghb_dict_get_value(settings, name);
@@ -517,7 +483,7 @@ ghb_ui_update_from_settings(signal_user_data_t *ud, const gchar *name, const Ghb
     object = GHB_OBJECT(ud->builder, name);
     if (object == NULL)
     {
-        g_debug("Failed to find widget for key: %s\n", name);
+        g_debug("Failed to find widget for key: %s", name);
         return -1;
     }
     ghb_update_widget((GtkWidget*)object, value);
@@ -532,13 +498,13 @@ ghb_ui_update(signal_user_data_t *ud, const gchar *name, const GhbValue *value)
 {
     GObject *object;
 
-    g_debug("ghb_ui_update() %s", name);
+    ghb_log_func_str(name);
     if (name == NULL || value == NULL)
         return 0;
     object = GHB_OBJECT(ud->builder, name);
     if (object == NULL)
     {
-        g_debug("Failed to find widget for key: %s\n", name);
+        g_debug("Failed to find widget for key: %s", name);
         return -1;
     }
     ghb_update_widget((GtkWidget*)object, value);
@@ -557,13 +523,13 @@ ghb_ui_settings_update(
 {
     GObject *object;
 
-    g_debug("ghb_ui_update() %s", name);
+    ghb_log_func_str(name);
     if (name == NULL || value == NULL)
         return 0;
     object = GHB_OBJECT(ud->builder, name);
     if (object == NULL)
     {
-        g_debug("Failed to find widget for key: %s\n", name);
+        g_debug("Failed to find widget for key: %s", name);
         return -1;
     }
     ghb_update_widget((GtkWidget*)object, value);
