@@ -1,6 +1,6 @@
 /*
  * settings.h
- * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
  *
  * settings.h is free software.
  *
@@ -58,7 +58,6 @@ typedef struct preview_s preview_t;
 typedef struct
 {
     gchar *current_dvd_device;
-    gboolean              debug;
     gboolean              dont_clear_presets;
     gboolean              scale_busy;
     gint                  cancel_encode;
@@ -77,10 +76,9 @@ typedef struct
     char                * extra_activity_path;
     gboolean              append_queue_activity;
     preview_t           * preview;
-    gchar               * appcast;
-    gint                  appcast_len;
     int                   stderr_src_id;
     GtkApplication      * app;
+    GtkFileChooserNative* source_dialog;
 } signal_user_data_t;
 
 enum
