@@ -1,7 +1,7 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * subtitlehandler.c
- * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
  *
  * subtitlehandler.c is free software.
  *
@@ -21,7 +21,6 @@
  *  Boston, MA  02110-1301, USA.
  */
 
-#include <glib/gi18n.h>
 #include "ghbcompat.h"
 #include "handbrake/handbrake.h"
 #include "settings.h"
@@ -35,6 +34,8 @@
 #include "audiohandler.h"
 #include "subtitlehandler.h"
 
+#include <glib/gi18n.h>
+
 static char * subtitle_get_track_description(GhbValue *settings,
                                              GhbValue *subsettings);
 static void subtitle_list_refresh_selected(signal_user_data_t *ud,
@@ -106,20 +107,20 @@ subtitle_refresh_list_row_ui(
         offset = ghb_dict_get_int(subsettings, "Offset");
         if (offset != 0)
         {
-            info_dst_2 = g_strdup_printf("Offset: %dms", offset);
+            info_dst_2 = g_strdup_printf(_("Offset: %dms"), offset);
         }
     }
 
     GString *str = g_string_new("<small>");
-    g_string_append_printf(str, "%s ", burned ? "Burned Into Video" :
-                                                "Passthrough");
+    g_string_append(str, burned ? _("Burned Into Video") :
+                                  _("Passthrough"));
     if (forced)
     {
-        g_string_append_printf(str, "(Forced Subtitles Only)");
+        g_string_append_printf(str, " (%s)", _("Forced Subtitles Only"));
     }
     if (def)
     {
-        g_string_append_printf(str, "(Default)");
+        g_string_append_printf(str, " (%s)", _("Default"));
     }
     g_string_append_printf(str, "</small>");
 
@@ -130,8 +131,8 @@ subtitle_refresh_list_row_ui(
         0, info_src,
         1, "-->",
         2, info_dst,
-        3, "hb-edit",
-        4, "hb-remove",
+        3, "document-edit-symbolic",
+        4, "edit-delete-symbolic",
         5, 0.5,
         -1);
 
@@ -520,8 +521,8 @@ ghb_subtitle_title_change(signal_user_data_t *ud, gboolean show)
     }
 }
 
-void
-ghb_set_pref_subtitle(signal_user_data_t *ud)
+static void
+set_pref_subtitle(signal_user_data_t *ud)
 {
     int               sub_count, title_id;
     GtkWidget        *widget;
@@ -555,30 +556,6 @@ ghb_set_pref_subtitle(signal_user_data_t *ud)
     subtitle_refresh_list_ui(ud);
 }
 
-gint
-ghb_selected_subtitle_row(signal_user_data_t *ud)
-{
-    GtkTreeView *tv;
-    GtkTreePath *tp;
-    GtkTreeSelection *ts;
-    GtkTreeModel *tm;
-    GtkTreeIter iter;
-    gint *indices;
-    gint row = -1;
-
-    tv = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "subtitle_list_view"));
-    ts = gtk_tree_view_get_selection(tv);
-    if (gtk_tree_selection_get_selected(ts, &tm, &iter))
-    {
-        // Get the row number
-        tp = gtk_tree_model_get_path(tm, &iter);
-        indices = gtk_tree_path_get_indices(tp);
-        row = indices[0];
-        gtk_tree_path_free(tp);
-    }
-    return row;
-}
-
 static GhbValue*
 subtitle_get_selected_settings(signal_user_data_t *ud, int *index)
 {
@@ -1187,6 +1164,7 @@ subtitle_add_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
         // Pop up the edit dialog
         GtkResponseType response;
         GtkWidget *dialog = GHB_WIDGET(ud->builder, "subtitle_dialog");
+        gtk_window_set_title(GTK_WINDOW(dialog), _("Add Subtitles"));
         response = gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_hide(dialog);
         if (response != GTK_RESPONSE_OK)
@@ -1240,6 +1218,7 @@ subtitle_add_fas_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
 
     GtkResponseType response;
     GtkWidget *dialog = GHB_WIDGET(ud->builder, "subtitle_dialog");
+    gtk_window_set_title(GTK_WINDOW(dialog), _("Foreign Audio Scan"));
     response = gtk_dialog_run(GTK_DIALOG(dialog));
     gtk_widget_hide(dialog);
     if (response != GTK_RESPONSE_OK)
@@ -1251,7 +1230,7 @@ subtitle_add_fas_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
     }
     else
     {
-        // Disabel FAS button
+        // Disable FAS button
         GtkWidget *w = GHB_WIDGET(ud->builder, "subtitle_add_fas");
         gtk_widget_set_sensitive(w, 0);
 
@@ -1299,7 +1278,7 @@ subtitle_add_all_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
 G_MODULE_EXPORT void
 subtitle_reset_clicked_cb(GtkWidget *xwidget, signal_user_data_t *ud)
 {
-    ghb_set_pref_subtitle(ud);
+    set_pref_subtitle(ud);
     ghb_update_summary_info(ud);
 }
 
@@ -1428,7 +1407,7 @@ ghb_subtitle_set_pref_lang(GhbValue *settings)
     }
 }
 
-void
+static void
 subtitle_add_lang_iter(GtkTreeModel *tm, GtkTreeIter *iter,
                        signal_user_data_t *ud)
 {
@@ -1557,10 +1536,10 @@ subtitle_remove_lang_iter(GtkTreeModel *tm, GtkTreeIter *iter,
     {
         if (ghb_array_len(slang_list) > 0)
         {
-            const iso639_lang_t *lang;
+            const iso639_lang_t *lang_id;
             GhbValue *entry = ghb_array_get(slang_list, 0);
-            lang = ghb_iso639_lookup_by_int(ghb_lookup_lang(entry));
-            subtitle_update_pref_lang(ud, lang);
+            lang_id = ghb_iso639_lookup_by_int(ghb_lookup_lang(entry));
+            subtitle_update_pref_lang(ud, lang_id);
         }
         else
         {
@@ -1737,6 +1716,7 @@ subtitle_edit(GtkTreeView *tv, GtkTreePath *tp, signal_user_data_t *ud)
         // Pop up the edit dialog
         GtkResponseType response;
         GtkWidget *dialog = GHB_WIDGET(ud->builder, "subtitle_dialog");
+        gtk_window_set_title(GTK_WINDOW(dialog), _("Edit Subtitles"));
         response = gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_hide(dialog);
         if (response != GTK_RESPONSE_OK)
