@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * subtitlehandler.h
- * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
  *
  * subtitlehandler.h is free software.
  *
@@ -40,4 +40,10 @@ GhbValue *ghb_get_subtitle_settings(GhbValue *settings);
 char * ghb_subtitle_short_description(const GhbValue *subsource,
                                       const GhbValue *subsettings);
 
+
+void subtitle_list_selection_changed_cb(GtkTreeSelection *ts, signal_user_data_t *ud);
+void subtitle_edit_clicked_cb(GtkWidget *widget, gchar *path, signal_user_data_t *ud);
+void subtitle_remove_clicked_cb(GtkWidget *widget, gchar *path, signal_user_data_t *ud);
+
+
 #endif // _SUBTITLEHANDLER_H_
