@@ -1,7 +1,7 @@
 /* chroma_smooth.c
 
    Copyright (c) 2002 Rémi Guyomarch <rguyom at pobox.com>
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -13,7 +13,7 @@
 #define CHROMA_SMOOTH_STRENGTH_DEFAULT 0.25
 #define CHROMA_SMOOTH_SIZE_DEFAULT 7
 #define CHROMA_SMOOTH_SIZE_MIN 3
-#define CHROMA_SMOOTH_SIZE_MAX 63
+#define CHROMA_SMOOTH_SIZE_MAX 15
 
 typedef struct
 {
@@ -88,7 +88,8 @@ static void name##_##nbits(const uint8_t *frame_src,
                                  uint8_t *frame_dst,                                                        \
                            const int width,                                                                 \
                            const int height,                                                                \
-                           int stride,                                                                      \
+                           int stride_src,                                                                  \
+                           int stride_dst,                                                                  \
                            chroma_smooth_plane_context_t * ctx,                                             \
                            chroma_smooth_thread_context_t * tctx)                                           \
 {                                                                                                           \
@@ -112,7 +113,20 @@ static void name##_##nbits(const uint8_t *frame_src,
     {                                                                                                       \
         if (src != dst)                                                                                     \
         {                                                                                                   \
-            memcpy(dst, src, stride*height);                                                                \
+            if (stride_src == stride_dst)                                                                   \
+            {                                                                                               \
+                memcpy(dst, src, stride_dst * height);                                                      \
+            }                                                                                               \
+            else                                                                                            \
+            {                                                                                               \
+                const int size = stride_src < stride_dst ? ABS(stride_src) : stride_dst;                    \
+                for (int yy = 0; yy < height; yy++)                                                         \
+                {                                                                                           \
+                    memcpy(dst, src, size);                                                                 \
+                    dst += stride_dst;                                                                      \
+                    src += stride_src;                                                                      \
+                }                                                                                           \
+            }                                                                                               \
         }                                                                                                   \
                                                                                                             \
         return;                                                                                             \
@@ -123,7 +137,8 @@ static void name##_##nbits(const uint8_t *frame_src,
         memset(SC[y], 0, sizeof(SC[y][0]) * (width + 2 * steps));                                           \
     }                                                                                                       \
                                                                                                             \
-    stride /= ctx->bps;                                                                                     \
+    stride_src /= ctx->bps;                                                                                 \
+    stride_dst /= ctx->bps;                                                                                 \
                                                                                                             \
     for (y = -steps; y < height + steps; y++)                                                               \
     {                                                                                                       \
@@ -132,7 +147,7 @@ static void name##_##nbits(const uint8_t *frame_src,
             src2 = src;                                                                                     \
         }                                                                                                   \
                                                                                                             \
-        memset(SR, 0, sizeof(SR[0]) * (2 * steps - 1));                                                     \
+        memset(SR, 0, sizeof(SR[0]) * (2 * steps));                                                         \
                                                                                                             \
         for (x = -steps; x < width + steps; x++)                                                            \
         {                                                                                                   \
@@ -152,8 +167,8 @@ static void name##_##nbits(const uint8_t *frame_src,
                                                                                                             \
             if (x >= steps && y >= steps)                                                                   \
             {                                                                                               \
-                const uint##nbits##_t *srx = src - steps * stride + x - steps;                              \
-                uint##nbits##_t       *dsx = dst - steps * stride + x - steps;                              \
+                const uint##nbits##_t *srx = src - steps * stride_src + x - steps;                          \
+                uint##nbits##_t       *dsx = dst - steps * stride_dst + x - steps;                          \
                                                                                                             \
                 res = (int32_t)*srx - ((((int32_t)*srx -                                                    \
                       (int32_t)((Tmp1 + halfscale) >> scalebits)) * amount) >> 16);                         \
@@ -163,8 +178,8 @@ static void name##_##nbits(const uint8_t *frame_src,
                                                                                                             \
         if (y >= 0)                                                                                         \
         {                                                                                                   \
-            dst += stride;                                                                                  \
-            src += stride;                                                                                  \
+            dst += stride_dst;                                                                              \
+            src += stride_src;                                                                              \
         }                                                                                                   \
     }                                                                                                       \
 }                                                                                                           \
@@ -310,6 +325,11 @@ static int chroma_smooth_init_thread(hb_filter_object_t *filter, int threads)
 
     chroma_smooth_thread_close(pv);
     pv->thread_ctx = calloc(threads, sizeof(chroma_smooth_thread_context3_t));
+    if (pv->thread_ctx == NULL)
+    {
+        hb_error("Chroma Smooth calloc failed");
+        return -1;
+    }
     pv->threads = threads;
     for (int c = 0; c < 3; c++)
     {
@@ -330,7 +350,6 @@ static int chroma_smooth_init_thread(hb_filter_object_t *filter, int threads)
                     if (tctx->SC[z] == NULL)
                     {
                         hb_error("Chroma Smooth calloc failed");
-                        chroma_smooth_close(filter);
                         return -1;
                     }
                 }
@@ -386,10 +405,11 @@ static int chroma_smooth_work_thread(hb_filter_object_t *filter,
                       in->plane[c].width,
                       in->plane[c].height,
                       in->plane[c].stride,
+                      out->plane[c].stride,
                       ctx, tctx);
     }
 
-    out->s = in->s;
+    hb_buffer_copy_props(out, in);
     *buf_out = out;
 
     return HB_FILTER_OK;
