@@ -1,6 +1,6 @@
 /* comb_detect.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -592,6 +592,11 @@ static int comb_detect_init(hb_filter_object_t *filter,
                             hb_filter_init_t   *init)
 {
     filter->private_data = calloc(1, sizeof(struct hb_filter_private_s));
+    if (filter->private_data == NULL)
+    {
+        hb_error("comb_detect: calloc failed");
+        return -1;
+    }
     hb_filter_private_t *pv = filter->private_data;
 
     hb_buffer_list_clear(&pv->out_list);
@@ -603,6 +608,11 @@ static int comb_detect_init(hb_filter_object_t *filter,
     pv->half_value = (1 << pv->depth) / 2;
 
     pv->gamma_lut = malloc(sizeof(float) * (pv->max_value + 1));
+    if (pv->gamma_lut == NULL)
+    {
+        hb_error("comb_detect: malloc failed");
+        return -1;
+    }
     build_gamma_lut(pv);
 
     pv->frames = 0;
@@ -637,6 +647,15 @@ static int comb_detect_init(hb_filter_object_t *filter,
         hb_dict_extract_int(&pv->block_height, dict, "block-height");
     }
 
+    if (pv->block_width > init->geometry.width)
+    {
+        pv->block_width = init->geometry.width;
+    }
+    if (pv->block_height > init->geometry.height)
+    {
+        pv->block_height = init->geometry.height;
+    }
+
     // Scale the thresholds for the current depth
     pv->motion_threshold  <<= (pv->depth - 8);
     pv->spatial_threshold <<= (pv->depth - 8);
@@ -693,6 +712,7 @@ static int comb_detect_init(hb_filter_object_t *filter,
                      sizeof(comb_detect_thread_arg_t), comb_detect_filter_work) == 0)
     {
         hb_error("comb_detect could not initialize taskset");
+        return -1;
     }
 
     comb_detect_thread_arg_t *comb_detect_prev_thread_args = NULL;
@@ -745,6 +765,7 @@ static int comb_detect_init(hb_filter_object_t *filter,
                      sizeof(comb_detect_thread_arg_t), comb_detect_check_work) == 0)
     {
         hb_error("comb_detect check could not initialize taskset");
+        return -1;
     }
 
     comb_detect_prev_thread_args = NULL;
@@ -755,7 +776,7 @@ static int comb_detect_init(hb_filter_object_t *filter,
         thread_args = taskset_thread_args(&pv->comb_detect_check_taskset, ii);
         thread_args->pv = pv;
         thread_args->arg.segment = ii;
-        thread_args->arg.taskset =  &pv->comb_detect_check_taskset;
+        thread_args->arg.taskset = &pv->comb_detect_check_taskset;
 
         for (int pp = 0; pp < 3; pp++)
         {
@@ -794,6 +815,7 @@ static int comb_detect_init(hb_filter_object_t *filter,
                          sizeof(comb_detect_thread_arg_t), mask_filter_work) == 0)
         {
             hb_error( "mask filter could not initialize taskset" );
+            return -1;
         }
 
         comb_detect_prev_thread_args = NULL;
@@ -803,8 +825,8 @@ static int comb_detect_init(hb_filter_object_t *filter,
 
             thread_args = taskset_thread_args(&pv->mask_filter_taskset, ii);
             thread_args->pv = pv;
-            thread_args->arg.taskset = &pv->mask_filter_taskset;
             thread_args->arg.segment = ii;
+            thread_args->arg.taskset = &pv->mask_filter_taskset;
 
             for (int pp = 0; pp < 3; pp++)
             {
@@ -837,6 +859,7 @@ static int comb_detect_init(hb_filter_object_t *filter,
                              sizeof(comb_detect_thread_arg_t), mask_erode_work) == 0)
             {
                 hb_error("mask erode could not initialize taskset");
+                return -1;
             }
 
             comb_detect_prev_thread_args = NULL;
@@ -846,8 +869,8 @@ static int comb_detect_init(hb_filter_object_t *filter,
 
                 thread_args = taskset_thread_args( &pv->mask_erode_taskset, ii );
                 thread_args->pv = pv;
-                thread_args->arg.taskset = &pv->mask_erode_taskset;
                 thread_args->arg.segment = ii;
+                thread_args->arg.taskset = &pv->mask_erode_taskset;
 
                 for (int pp = 0; pp < 3; pp++)
                 {
@@ -878,6 +901,7 @@ static int comb_detect_init(hb_filter_object_t *filter,
                              sizeof(comb_detect_thread_arg_t), mask_dilate_work) == 0)
             {
                 hb_error("mask dilate could not initialize taskset");
+                return -1;
             }
 
             comb_detect_prev_thread_args = NULL;
@@ -945,6 +969,8 @@ static void comb_detect_close(hb_filter_object_t *filter)
         }
     }
 
+    hb_buffer_list_close(&pv->out_list);
+
     /* Cleanup reference buffers. */
     for (int ii = 0; ii < 3; ii++)
     {
