@@ -1,6 +1,6 @@
 /* decavsub.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -315,7 +315,7 @@ int decavsubWork( hb_avsub_context_t * ctx,
     ctx->pkt->data = in->data;
     ctx->pkt->size = in->size;
     ctx->pkt->pts  = in_s.start;
-    if (in_s.duration > 0 || ctx->subtitle->source != PGSSUB)
+    if (in_s.duration > 0 || ctx->subtitle->source == SSASUB || ctx->subtitle->source == IMPORTSSA)
     {
         duration = in_s.duration;
     }
