@@ -1,6 +1,6 @@
 /* decomb.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -211,6 +211,11 @@ static int hb_decomb_init(hb_filter_object_t *filter,
                           hb_filter_init_t *init)
 {
     filter->private_data = calloc(1, sizeof(struct hb_filter_private_s));
+    if (filter->private_data == NULL)
+    {
+        hb_error("decomb: calloc failed");
+        return -1;
+    }
     hb_filter_private_t *pv = filter->private_data;
     pv->input                = *init;
     hb_buffer_list_clear(&pv->out_list);
@@ -329,6 +334,7 @@ static int hb_decomb_init(hb_filter_object_t *filter,
                      sizeof(yadif_thread_arg_t), yadif_decomb_filter_work) == 0)
     {
         hb_error("decomb yadif could not initialize taskset");
+        return -1;
     }
 
     yadif_thread_arg_t *yadif_prev_thread_args = NULL;
@@ -373,6 +379,7 @@ static int hb_decomb_init(hb_filter_object_t *filter,
                          sizeof(eedi2_thread_arg_t), eedi2_filter_work) == 0)
         {
             hb_error("decomb eedi2 could not initialize taskset");
+            return -1;
         }
 
         if (pv->post_processing > 1)
@@ -387,6 +394,7 @@ static int hb_decomb_init(hb_filter_object_t *filter,
             if (!pv->cx2 || !pv->cy2 || !pv->cxy || !pv->tmpc)
             {
                 hb_error("EEDI2: failed to malloc derivative arrays");
+                return -1;
             }
             else
             {
@@ -433,6 +441,8 @@ static void hb_decomb_close(hb_filter_object_t *filter)
         taskset_fini(&pv->eedi2_taskset);
     }
 
+    hb_buffer_list_close(&pv->out_list);
+
     // Cleanup reference buffers
     for (int ii = 0; ii < 3; ii++)
     {
@@ -545,7 +555,7 @@ static void process_frame(hb_filter_private_t *pv)
             pv->filter(pv, buf, parity, tff);
 
             // Copy buffered settings to output buffer settings
-            buf->s = pv->ref[1]->s;
+            hb_buffer_copy_props(buf, pv->ref[1]);
 
             hb_buffer_list_append(&pv->out_list, buf);
         }
@@ -570,6 +580,15 @@ static int hb_decomb_work(hb_filter_object_t *filter,
     hb_filter_private_t *pv = filter->private_data;
     hb_buffer_t *in = *buf_in;
 
+    // TODO: eliminate extra buffer copies in decomb
+    if (in->plane[0].height_stride - in->plane[0].height < 3)
+    {
+        // Decomb requires some additional rows
+        // to work on.
+        in = hb_buffer_dup(in);
+        hb_buffer_close(buf_in);
+    }
+
     // Input buffer is always consumed.
     *buf_in = NULL;
     if (in->s.flags & HB_BUF_FLAG_EOF)
