@@ -1,6 +1,6 @@
 /* decsrtsub.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -56,7 +56,7 @@ struct hb_work_private_s
     unsigned long last_entry_number;
     unsigned long current_state;
     srt_entry_t current_entry;
-    iconv_t *iconv_context;
+    iconv_t iconv_context;
     hb_subtitle_t *subtitle;
     uint64_t start_time;              // In HB time
     uint64_t stop_time;               // In HB time
