@@ -1,6 +1,6 @@
 /* detelecine.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -927,6 +927,11 @@ static int hb_detelecine_init( hb_filter_object_t * filter,
                                hb_filter_init_t * init )
 {
     filter->private_data = calloc(1, sizeof(struct hb_filter_private_s));
+    if (filter->private_data == NULL)
+    {
+        hb_error("detelecine: calloc failed");
+        return -1;
+    }
 
     hb_filter_private_t   * pv = filter->private_data;
     struct pullup_context * ctx;
@@ -1044,9 +1049,30 @@ static int hb_detelecine_work( hb_filter_object_t * filter,
     }
 
     /* Copy input buffer into pullup buffer */
-    memcpy( buf->planes[0], in->plane[0].data, buf->size[0] );
-    memcpy( buf->planes[1], in->plane[1].data, buf->size[1] );
-    memcpy( buf->planes[2], in->plane[2].data, buf->size[2] );
+    /* Copy pullup frame buffer into output buffer */
+    for (int pp = 0; pp < 3; pp++)
+    {
+        if (in->plane[pp].stride == ctx->stride[pp])
+        {
+            memcpy(buf->planes[pp], in->plane[pp].data, buf->size[pp]);
+        }
+        else
+        {
+            const int stride_src = in->plane[pp].stride;
+            const int stride_dst = ctx->stride[pp];
+            const int height = in->plane[pp].height;
+            const int size = stride_src < stride_dst ? ABS(stride_src) : stride_dst;
+            uint8_t *dst = buf->planes[pp];
+            uint8_t *src = in->plane[pp].data;
+
+            for (int yy = 0; yy < height; yy++)
+            {
+                memcpy(dst, src, size);
+                dst += stride_dst;
+                src += stride_src;
+            }
+        }
+    }
 
     /* Submit buffer fields based on buffer flags.
        Detelecine assumes BFF when the TFF flag isn't present. */
@@ -1147,7 +1173,7 @@ static int hb_detelecine_work( hb_filter_object_t * filter,
 
     pullup_release_frame( frame );
 
-    out->s = in->s;
+    hb_buffer_copy_props(out, in);
     *buf_out = out;
 
 output_frame:
