@@ -101,9 +101,6 @@ struct hb_work_private_s
     int                  is_sys_mem;
     mfxSession           mfx_session;
 
-    // whether to expect input from VPP or from QSV decode
-    int                  is_vpp_present;
-
     // whether the encoder is initialized
     int                  init_done;
 
@@ -201,15 +198,17 @@ static int log_encoder_params(const hb_work_private_t *pv, const mfxVideoParam *
         else if (option->Header.BufferId != MFX_EXTBUFF_VIDEO_SIGNAL_INFO &&
                  option->Header.BufferId != MFX_EXTBUFF_CHROMA_LOC_INFO &&
                  option->Header.BufferId != MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME &&
-                 option->Header.BufferId != MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO)
+                 option->Header.BufferId != MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO &&
+                 option->Header.BufferId != MFX_EXTBUFF_AV1_BITSTREAM_PARAM)
         {
             hb_log("Unknown Header.BufferId=%d", option->Header.BufferId);
         }
     }
 
     // log code path and main output settings
-    hb_log("encqsvInit: using%s%s%s path",
-           pv->is_sys_mem ? " encode-only" : " full QSV",
+    hb_log("encqsvInit: using%s%s%s%s path",
+           hb_qsv_full_path_is_enabled(pv->job) ? " full QSV" : " encode-only",
+           hb_qsv_get_memory_type(pv->job) == MFX_IOPATTERN_OUT_VIDEO_MEMORY ? " via video memory" : " via system memory",
            videoParam->mfx.LowPower == MFX_CODINGOPTION_ON ? " (LowPower)" : "",
            extHyperModeOption != NULL ? hyper_encode_name(extHyperModeOption->Mode) : "");
     hb_log("encqsvInit: %s %s profile @ level %s",
@@ -1019,7 +1018,7 @@ int qsv_enc_init(hb_work_private_t *pv)
                 .Free   = hb_qsv_frame_free,
             };
 
-            if (hb_qsv_hw_filters_are_enabled(pv->job))
+            if (hb_qsv_hw_filters_via_video_memory_are_enabled(pv->job) || hb_qsv_hw_filters_via_system_memory_are_enabled(pv->job))
             {
                 frame_allocator.pthis = pv->job->qsv.ctx->hb_vpp_qsv_frames_ctx;
             }
@@ -1036,37 +1035,6 @@ int qsv_enc_init(hb_work_private_t *pv)
 
     if (!pv->is_sys_mem)
     {
-        if (!pv->is_vpp_present && job->list_filter != NULL)
-        {
-            for (i = 0; i < hb_list_count(job->list_filter); i++)
-            {
-                hb_filter_object_t *filter = hb_list_item(job->list_filter, i);
-                if (filter->id == HB_FILTER_QSV_PRE  ||
-                    filter->id == HB_FILTER_QSV_POST ||
-                    filter->id == HB_FILTER_QSV)
-                {
-                    pv->is_vpp_present = 1;
-                    break;
-                }
-            }
-        }
-
-        if (pv->is_vpp_present)
-        {
-            if (qsv->vpp_space == NULL)
-            {
-                return 2;
-            }
-            for (i = 0; i < hb_qsv_list_count(qsv->vpp_space); i++)
-            {
-                hb_qsv_space *vpp = hb_qsv_list_item(qsv->vpp_space, i);
-                if (!vpp->is_init_done)
-                {
-                    return 2;
-                }
-            }
-        }
-
         hb_qsv_space *dec_space = qsv->dec_space;
         if (dec_space == NULL || !dec_space->is_init_done)
         {
@@ -1088,28 +1056,6 @@ int qsv_enc_init(hb_work_private_t *pv)
         hb_qsv_list_add(qsv_encode->tasks, task);
     }
 
-#if !HB_QSV_ONEVPL
-    // plugins should be loaded before querying for surface allocation
-    if (pv->loaded_plugins == NULL)
-    {
-        if (MFXQueryVersion(qsv->mfx_session, &version) != MFX_ERR_NONE)
-        {
-            hb_error("qsv_enc_init: MFXQueryVersion failed");
-            *job->done_error = HB_ERROR_INIT;
-            *job->die = 1;
-            return -1;
-        }
-        pv->loaded_plugins = hb_qsv_load_plugins(hb_qsv_get_adapter_index(), pv->qsv_info, qsv->mfx_session, version);
-        if (pv->loaded_plugins == NULL)
-        {
-            hb_error("qsv_enc_init: hb_qsv_load_plugins failed");
-            *job->done_error = HB_ERROR_INIT;
-            *job->die = 1;
-            return -1;
-        }
-    }
-#endif
-
     // setup surface allocation
     pv->param.videoParam->IOPattern = (pv->is_sys_mem                 ?
                                        MFX_IOPATTERN_IN_SYSTEM_MEMORY :
@@ -1157,14 +1103,6 @@ int qsv_enc_init(hb_work_private_t *pv)
         {
             qsv_encode->surface_num = HB_QSV_SURFACE_NUM;
         }
-        // Use only when VPP will be fixed
-        // hb_qsv_space *in_space = qsv->dec_space;
-        // if (pv->is_vpp_present)
-        // {
-        //     // we get our input from VPP instead
-        //     in_space = hb_qsv_list_item(qsv->vpp_space,
-        //                                 hb_qsv_list_count(qsv->vpp_space) - 1);
-        // }
     }
 
     // allocate sync points
@@ -1214,11 +1152,17 @@ int qsv_enc_init(hb_work_private_t *pv)
  **********************************************************************/
 int encqsvInit(hb_work_object_t *w, hb_job_t *job)
 {
+    if (!hb_qsv_available())
+    {
+       hb_error("encqsvInit: qsv is not available on the system");
+       return -1;
+    }
+
     int brc_param_multiplier;
     hb_work_private_t *pv = calloc(1, sizeof(hb_work_private_t));
     w->private_data       = pv;
 
-    pv->is_sys_mem         = hb_qsv_full_path_is_enabled(job) ? 0 : 1; // TODO: re-implement QSV VPP filtering support
+    pv->is_sys_mem         = (hb_qsv_get_memory_type(job) == MFX_IOPATTERN_OUT_SYSTEM_MEMORY);
     pv->job                = job;
     pv->qsv_info           = hb_qsv_encoder_info_get(hb_qsv_get_adapter_index(), job->vcodec);
     pv->delayed_processing = hb_list_init();
@@ -1227,6 +1171,12 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
 
     pv->chapter_queue    = hb_chapter_queue_init();
 
+    if (!pv->qsv_info)
+    {
+        hb_error("encqsvInit: %s codec is not supported by this GPU adapter", hb_video_encoder_get_long_name(job->vcodec));
+        return -1;
+    }
+
     // default encoding parameters
     if (hb_qsv_param_default_preset(&pv->param, &pv->enc_space.m_mfxVideoParam,
                                      pv->qsv_info, job->encoder_preset))
@@ -1239,6 +1189,8 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     pv->param.videoParam->AsyncDepth = job->qsv.async_depth;
 
     // set and enable colorimetry (video signal information)
+    pv->param.videoSignalInfo.VideoFullRange = (pv->job->color_range == AVCOL_RANGE_JPEG);
+
     pv->param.videoSignalInfo.ColourPrimaries          = hb_output_color_prim(job);
     pv->param.videoSignalInfo.TransferCharacteristics  = hb_output_color_transfer(job);
     pv->param.videoSignalInfo.MatrixCoefficients       = hb_output_color_matrix(job);
@@ -1254,31 +1206,65 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     /* HDR10 Static metadata */
     if (job->color_transfer == HB_COLR_TRA_SMPTEST2084)
     {
-        const int masteringChromaDen = 50000;
-        const int masteringLumaDen = 10000;
-
-        /* Mastering display metadata */
-        if (job->mastering.has_primaries && job->mastering.has_luminance)
+        if (pv->qsv_info->codec_id == MFX_CODEC_HEVC)
         {
-            pv->param.masteringDisplayColourVolume.InsertPayloadToggle = MFX_PAYLOAD_IDR;
-            pv->param.masteringDisplayColourVolume.DisplayPrimariesX[0] = rescale(job->mastering.display_primaries[0][0], masteringChromaDen);
-            pv->param.masteringDisplayColourVolume.DisplayPrimariesY[0] = rescale(job->mastering.display_primaries[0][1], masteringChromaDen);
-            pv->param.masteringDisplayColourVolume.DisplayPrimariesX[1] = rescale(job->mastering.display_primaries[1][0], masteringChromaDen);
-            pv->param.masteringDisplayColourVolume.DisplayPrimariesY[1] = rescale(job->mastering.display_primaries[1][1], masteringChromaDen);
-            pv->param.masteringDisplayColourVolume.DisplayPrimariesX[2] = rescale(job->mastering.display_primaries[2][0], masteringChromaDen);
-            pv->param.masteringDisplayColourVolume.DisplayPrimariesY[2] = rescale(job->mastering.display_primaries[2][1], masteringChromaDen);
-            pv->param.masteringDisplayColourVolume.WhitePointX = rescale(job->mastering.white_point[0], masteringChromaDen);
-            pv->param.masteringDisplayColourVolume.WhitePointY = rescale(job->mastering.white_point[1], masteringChromaDen);
-            pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance = rescale(job->mastering.max_luminance, masteringLumaDen);
-            pv->param.masteringDisplayColourVolume.MinDisplayMasteringLuminance = rescale(job->mastering.min_luminance, masteringLumaDen);
+            const int masteringChromaDen = 50000;
+            const int masteringLumaDen = 10000;
+
+            /* Mastering display metadata */
+            if (job->mastering.has_primaries && job->mastering.has_luminance)
+            {
+                pv->param.masteringDisplayColourVolume.InsertPayloadToggle = MFX_PAYLOAD_IDR;
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[0] = FFMIN(rescale(job->mastering.display_primaries[1][0], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[0] = FFMIN(rescale(job->mastering.display_primaries[1][1], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[1] = FFMIN(rescale(job->mastering.display_primaries[2][0], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[1] = FFMIN(rescale(job->mastering.display_primaries[2][1], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[2] = FFMIN(rescale(job->mastering.display_primaries[0][0], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[2] = FFMIN(rescale(job->mastering.display_primaries[0][1], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.WhitePointX = FFMIN(rescale(job->mastering.white_point[0], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.WhitePointY = FFMIN(rescale(job->mastering.white_point[1], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance = rescale(job->mastering.max_luminance, masteringLumaDen);
+                pv->param.masteringDisplayColourVolume.MinDisplayMasteringLuminance = FFMIN(rescale(job->mastering.min_luminance, masteringLumaDen),
+                    pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance);
+            }
+
+            /*  Content light level */
+            if (job->coll.max_cll && job->coll.max_fall)
+            {
+                pv->param.contentLightLevelInfo.InsertPayloadToggle = MFX_PAYLOAD_IDR;
+                pv->param.contentLightLevelInfo.MaxContentLightLevel  = FFMIN(job->coll.max_cll, 65535);
+                pv->param.contentLightLevelInfo.MaxPicAverageLightLevel = FFMIN(job->coll.max_fall, 65535);
+            }
         }
-
-        /*  Content light level */
-        if (job->coll.max_cll && job->coll.max_fall)
+        else if (pv->qsv_info->codec_id == MFX_CODEC_AV1)
         {
-            pv->param.contentLightLevelInfo.InsertPayloadToggle = MFX_PAYLOAD_IDR;
-            pv->param.contentLightLevelInfo.MaxContentLightLevel  = job->coll.max_cll;
-            pv->param.contentLightLevelInfo.MaxPicAverageLightLevel = job->coll.max_fall;
+            const int masteringChromaDen = 1 << 16;
+            const int max_luma_den = 1 << 8;
+            const int min_luma_den = 1 << 14;
+
+            /* Mastering display metadata */
+            if (job->mastering.has_primaries && job->mastering.has_luminance)
+            {
+                pv->param.masteringDisplayColourVolume.InsertPayloadToggle = MFX_PAYLOAD_IDR;
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[0] = rescale(job->mastering.display_primaries[0][0], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[0] = rescale(job->mastering.display_primaries[0][1], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[1] = rescale(job->mastering.display_primaries[1][0], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[1] = rescale(job->mastering.display_primaries[1][1], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[2] = rescale(job->mastering.display_primaries[2][0], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[2] = rescale(job->mastering.display_primaries[2][1], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.WhitePointX = rescale(job->mastering.white_point[0], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.WhitePointY = rescale(job->mastering.white_point[1], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance = rescale(job->mastering.max_luminance, max_luma_den);
+                pv->param.masteringDisplayColourVolume.MinDisplayMasteringLuminance = rescale(job->mastering.min_luminance, min_luma_den);
+            }
+
+            /*  Content light level */
+            if (job->coll.max_cll && job->coll.max_fall)
+            {
+                pv->param.contentLightLevelInfo.InsertPayloadToggle = MFX_PAYLOAD_IDR;
+                pv->param.contentLightLevelInfo.MaxContentLightLevel  = job->coll.max_cll;
+                pv->param.contentLightLevelInfo.MaxPicAverageLightLevel = job->coll.max_fall;
+            }
         }
     }
 
@@ -1326,9 +1312,6 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
 #if defined(_WIN32) || defined(__MINGW32__)
     if (pv->is_sys_mem && hb_qsv_implementation_is_hardware(pv->qsv_info->implementation))
     {
-        // select the right hardware implementation based on dx index
-        if (!job->qsv.ctx->qsv_device)
-            hb_qsv_param_parse_dx_index(pv->job, hb_qsv_get_adapter_index());
         mfxIMPL hw_preference = MFX_IMPL_VIA_D3D11;
         pv->qsv_info->implementation = hb_qsv_dx_index_to_impl(job->qsv.ctx->dx_index) | hw_preference;
     }
@@ -1347,18 +1330,20 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
                       job->par.num, job->par.den, UINT16_MAX);
 
     // some encoding parameters are used by filters to configure their output
+    int align_width = 0, align_height = 0;
+
     switch (pv->qsv_info->codec_id)
     {
         case MFX_CODEC_HEVC:
         case MFX_CODEC_AV1:
-            job->qsv.enc_info.align_width  = HB_QSV_ALIGN32(job->width);
-            job->qsv.enc_info.align_height = HB_QSV_ALIGN32(job->height);
+            align_width  = HB_QSV_ALIGN32(job->width);
+            align_height = HB_QSV_ALIGN32(job->height);
             break;
 
         case MFX_CODEC_AVC:
         default:
-            job->qsv.enc_info.align_width  = HB_QSV_ALIGN16(job->width);
-            job->qsv.enc_info.align_height = HB_QSV_ALIGN16(job->height);
+            align_width  = HB_QSV_ALIGN16(job->width);
+            align_height = HB_QSV_ALIGN16(job->height);
             break;
     }
     if (pv->param.videoParam->mfx.FrameInfo.PicStruct != MFX_PICSTRUCT_PROGRESSIVE)
@@ -1367,15 +1352,13 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         switch (pv->qsv_info->codec_id)
         {
             case MFX_CODEC_AVC:
-                job->qsv.enc_info.align_height = HB_QSV_ALIGN32(job->qsv.enc_info.align_height);
+                align_height = HB_QSV_ALIGN32(align_height);
                 break;
 
             default:
                 break;
         }
     }
-    job->qsv.enc_info.pic_struct   = pv->param.videoParam->mfx.FrameInfo.PicStruct;
-    job->qsv.enc_info.is_init_done = 1;
 
     // set codec, profile/level and FrameInfo
     pv->param.videoParam->mfx.CodecId                 = pv->qsv_info->codec_id;
@@ -1391,9 +1374,8 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     pv->param.videoParam->mfx.FrameInfo.CropY         = 0;
     pv->param.videoParam->mfx.FrameInfo.CropW         = job->width;
     pv->param.videoParam->mfx.FrameInfo.CropH         = job->height;
-    pv->param.videoParam->mfx.FrameInfo.PicStruct     = job->qsv.enc_info.pic_struct;
-    pv->param.videoParam->mfx.FrameInfo.Width         = job->qsv.enc_info.align_width;
-    pv->param.videoParam->mfx.FrameInfo.Height        = job->qsv.enc_info.align_height;
+    pv->param.videoParam->mfx.FrameInfo.Width         = align_width;
+    pv->param.videoParam->mfx.FrameInfo.Height        = align_height;
 
     // parse user-specified codec profile and level
     if (hb_qsv_profile_parse(&pv->param, pv->qsv_info, job->encoder_profile, job->vcodec))
@@ -1440,8 +1422,11 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         }
     }
 
+    int hw_generation = hb_qsv_hardware_generation(hb_qsv_get_platform(hb_qsv_get_adapter_index()));
     // sanitize ICQ
-    if (!(pv->qsv_info->capabilities & HB_QSV_CAP_RATECONTROL_ICQ))
+    // workaround for MediaSDK platforms below TGL to disable ICQ if incorrectly detected
+    if (!(pv->qsv_info->capabilities & HB_QSV_CAP_RATECONTROL_ICQ) ||
+        ((pv->param.videoParam->mfx.LowPower == MFX_CODINGOPTION_ON) && (hw_generation < QSV_G8)))
     {
         // ICQ not supported
         pv->param.rc.icq = 0;
@@ -1469,15 +1454,16 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     {
         pv->param.rc.lookahead = pv->param.rc.lookahead && (pv->param.rc.icq || job->vquality <= HB_INVALID_VIDEO_QUALITY);
     }
-#if HB_PROJECT_FEATURE_QSV
+
     if (pv->job->qsv.ctx != NULL)
     {
         job->qsv.ctx->la_is_enabled = pv->param.rc.lookahead ? 1 : 0;
     }
-#endif
-    //libmfx BRC parameters are 16 bits thus maybe overflow, then BRCParamMultiplier is needed
-    brc_param_multiplier = (FFMAX(FFMAX3(job->vbitrate, pv->param.rc.vbv_max_bitrate, pv->param.rc.vbv_buffer_size / 8),
-                            pv->param.rc.vbv_buffer_init / 8) + 0x10000) / 0x10000;
+
+    // libmfx BRC parameters are 16 bits thus maybe overflow, then BRCParamMultiplier is needed
+    // Comparison vbitrate in Kbps (kilobit) with vbv_max_bitrate, vbv_buffer_size, vbv_buffer_init in KB (kilobyte)
+    brc_param_multiplier = (FFMAX(FFMAX3(job->vbitrate, pv->param.rc.vbv_max_bitrate, pv->param.rc.vbv_buffer_size),
+                            pv->param.rc.vbv_buffer_init) + 0x10000) / 0x10000;
     // set VBV here (this will be overridden for CQP and ignored for LA)
     // only set BufferSizeInKB, InitialDelayInKB and MaxKbps if we have
     // them - otherwise Media SDK will pick values for us automatically
@@ -1595,7 +1581,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     // set the GOP structure
     if (pv->param.gop.gop_ref_dist < 0)
     {
-        if ((hb_qsv_hardware_generation(hb_qsv_get_platform(hb_qsv_get_adapter_index())) >= QSV_G8) &&
+        if ((hw_generation >= QSV_G8) &&
             (pv->param.videoParam->mfx.CodecId == MFX_CODEC_HEVC ||
             pv->param.videoParam->mfx.CodecId == MFX_CODEC_AV1))
         {
@@ -1611,9 +1597,9 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     // set the keyframe interval
     if (pv->param.gop.gop_pic_size < 0)
     {
-        int rate = (double)job->orig_vrate.num / job->orig_vrate.den + 0.5;
+        double rate = (double)job->orig_vrate.num / job->orig_vrate.den + 0.5;
         // set the keyframe interval based on the framerate
-        pv->param.gop.gop_pic_size = rate;
+        pv->param.gop.gop_pic_size = (int)(FFMIN(rate * 2, 120));
     }
     pv->param.videoParam->mfx.GopPicSize = pv->param.gop.gop_pic_size;
 
@@ -1628,7 +1614,9 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         {
             pv->param.videoParam->mfx.IdrInterval = 0;
         }
-        pv->param.videoParam->AsyncDepth = 60;
+        // sanitize some of the encoding parameters
+        pv->param.videoParam->mfx.GopPicSize = (int)(FFMIN(pv->param.gop.gop_pic_size, 60));
+        pv->param.videoParam->AsyncDepth = (int)(FFMAX(pv->param.videoParam->AsyncDepth, 30));
     }
     // sanitize some settings that affect memory consumption
     if (!pv->is_sys_mem)
@@ -1667,18 +1655,21 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     int err;
     mfxStatus sts;
     mfxVersion version;
-    mfxLoader loader;
     mfxVideoParam videoParam;
-    mfxExtBuffer *extParamArray[5];
+    mfxExtBuffer *extParamArray[8];
     mfxSession session = (mfxSession)0;
     mfxExtCodingOption  option1_buf, *option1 = &option1_buf;
     mfxExtCodingOption2 option2_buf, *option2 = &option2_buf;
     mfxExtCodingOptionSPSPPS sps_pps_buf, *sps_pps = &sps_pps_buf;
     mfxExtAV1BitstreamParam av1_bitstream_buf, *av1_bitstream = &av1_bitstream_buf;
+    mfxExtChromaLocInfo chroma_loc_info_buf, *chroma_loc_info = &chroma_loc_info_buf;
+    mfxExtMasteringDisplayColourVolume mastering_display_color_volume_buf, *mastering_display_color_volume = &mastering_display_color_volume_buf;
+    mfxExtContentLightLevelInfo content_light_level_info_buf, *content_light_level_info = &content_light_level_info_buf;
     mfxExtHyperModeParam hyper_encode_buf, *hyper_encode = &hyper_encode_buf;
     version.Major = HB_QSV_MINVERSION_MAJOR;
     version.Minor = HB_QSV_MINVERSION_MINOR;
-    sts = hb_qsv_create_mfx_session(pv->qsv_info->implementation, job->qsv.ctx->dx_index, &version, &session, &loader);
+    uint32_t render_node = hb_qsv_get_adapter_render_node(hb_qsv_get_adapter_index());
+    sts = hb_qsv_create_mfx_session(pv->qsv_info->implementation, render_node, &version, &session);
     if (sts != MFX_ERR_NONE)
     {
         hb_error("encqsvInit: MFXInit failed (%d) with implementation %d", sts, pv->qsv_info->implementation);
@@ -1689,7 +1680,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     {
         // On linux, the handle to the VA display must be set.
         // This code is essentially a NOP other platforms.
-        job->qsv.ctx->display = hb_qsv_display_init();
+        job->qsv.ctx->display = hb_qsv_display_init(render_node);
         if (job->qsv.ctx->display != NULL)
         {
             MFXVideoCORE_SetHandle(session, job->qsv.ctx->display->mfxType,
@@ -1706,30 +1697,8 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         return -1;
     }
 
-#if !HB_QSV_ONEVPL
-    /* Load required MFX plug-ins */
-    pv->loaded_plugins = hb_qsv_load_plugins(hb_qsv_get_adapter_index(), pv->qsv_info, session, version);
-    if (pv->loaded_plugins == NULL)
-    {
-        hb_error("encqsvInit: hb_qsv_load_plugins failed");
-        MFXClose(session);
-        return -1;
-    }
-#endif
     /* MFXVideoENCODE_Init with desired encoding parameters */
     sts = MFXVideoENCODE_Init(session, pv->param.videoParam);
-// workaround for the early 15.33.x driver, should be removed later
-#define HB_DRIVER_FIX_33
-#ifdef  HB_DRIVER_FIX_33
-    int la_workaround = 0;
-    if (sts < MFX_ERR_NONE &&
-        pv->param.videoParam->mfx.RateControlMethod == MFX_RATECONTROL_LA)
-    {
-        pv->param.videoParam->mfx.RateControlMethod = MFX_RATECONTROL_CBR;
-        sts = MFXVideoENCODE_Init(session, pv->param.videoParam);
-        la_workaround = 1;
-    }
-#endif
     if (sts < MFX_ERR_NONE) // ignore warnings
     {
         hb_error("encqsvInit: MFXVideoENCODE_Init failed (%d)", sts);
@@ -1738,9 +1707,6 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         {
             hb_error("encqsvInit: log_encoder_params failed (%d)", err);
         }
-#if !HB_QSV_ONEVPL
-        hb_qsv_unload_plugins(&pv->loaded_plugins, session, version);
-#endif
         MFXClose(session);
         return -1;
     }
@@ -1786,6 +1752,27 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     {
         videoParam.ExtParam[videoParam.NumExtParam++] = (mfxExtBuffer*)av1_bitstream;
     }
+    memset(chroma_loc_info, 0, sizeof(mfxExtChromaLocInfo));
+    chroma_loc_info->Header.BufferId = MFX_EXTBUFF_CHROMA_LOC_INFO;
+    chroma_loc_info->Header.BufferSz = sizeof(mfxExtChromaLocInfo);
+    if (pv->qsv_info->capabilities & HB_QSV_CAP_VUI_CHROMALOCINFO)
+    {
+        videoParam.ExtParam[videoParam.NumExtParam++] = (mfxExtBuffer*)chroma_loc_info;
+    }
+    memset(mastering_display_color_volume, 0, sizeof(mfxExtMasteringDisplayColourVolume));
+    mastering_display_color_volume->Header.BufferId = MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME;
+    mastering_display_color_volume->Header.BufferSz = sizeof(mfxExtMasteringDisplayColourVolume);
+    if (pv->qsv_info->capabilities & HB_QSV_CAP_VUI_MASTERINGINFO)
+    {
+        videoParam.ExtParam[videoParam.NumExtParam++] = (mfxExtBuffer*)mastering_display_color_volume;
+    }
+    memset(content_light_level_info, 0, sizeof(mfxExtContentLightLevelInfo));
+    content_light_level_info->Header.BufferId = MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO;
+    content_light_level_info->Header.BufferSz = sizeof(mfxExtContentLightLevelInfo);
+    if (pv->qsv_info->capabilities & HB_QSV_CAP_VUI_CLLINFO)
+    {
+        videoParam.ExtParam[videoParam.NumExtParam++] = (mfxExtBuffer*)content_light_level_info;
+    }
     memset(hyper_encode, 0, sizeof(mfxExtHyperModeParam));
     hyper_encode->Header.BufferId = MFX_EXTBUFF_HYPER_MODE_PARAM;
     hyper_encode->Header.BufferSz = sizeof(mfxExtHyperModeParam);
@@ -1798,9 +1785,6 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     if (sts != MFX_ERR_NONE)
     {
         hb_error("encqsvInit: MFXVideoENCODE_GetVideoParam failed (%d)", sts);
-#if !HB_QSV_ONEVPL
-        hb_qsv_unload_plugins(&pv->loaded_plugins, session, version);
-#endif
         MFXClose(session);
         return -1;
     }
@@ -1821,9 +1805,6 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         if (qsv_hevc_make_header(w, session, &videoParam) < 0)
         {
             hb_error("encqsvInit: qsv_hevc_make_header failed");
-#if !HB_QSV_ONEVPL
-            hb_qsv_unload_plugins(&pv->loaded_plugins, session, version);
-#endif
             MFXVideoENCODE_Close(session);
             MFXClose(session);
             return -1;
@@ -1833,16 +1814,6 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     /* We don't need this encode session once we have the header */
     MFXVideoENCODE_Close(session);
 
-#ifdef HB_DRIVER_FIX_33
-    if (la_workaround)
-    {
-        videoParam.mfx.RateControlMethod =
-        pv->param.videoParam->mfx.RateControlMethod = MFX_RATECONTROL_LA;
-        option2->LookAheadDepth = pv->param.codingOption2.LookAheadDepth;
-        hb_log("encqsvInit: using LookAhead workaround (\"early 33 fix\")");
-    }
-#endif
-
     // when using system memory, we re-use this same session
     if (pv->is_sys_mem)
     {
@@ -1850,9 +1821,6 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     }
     else
     {
-#if !HB_QSV_ONEVPL
-        hb_qsv_unload_plugins(&pv->loaded_plugins, session, version);
-#endif
         MFXClose(session);
     }
 
@@ -1864,10 +1832,9 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         {
             case MFX_CODEC_AVC:
             case MFX_CODEC_HEVC:
-            case MFX_CODEC_AV1:
                 pv->init_delay = &w->config->init_delay;
                 break;
-            default: // unreachable
+            default:
                 break;
         }
 
@@ -1896,7 +1863,6 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
 void encqsvClose(hb_work_object_t *w)
 {
     hb_work_private_t *pv = w->private_data;
-    mfxVersion version;
     int i;
 
     if (pv != NULL && pv->job != NULL && pv->job->qsv.ctx != NULL &&
@@ -1908,13 +1874,6 @@ void encqsvClose(hb_work_object_t *w)
 
         if (qsv_ctx != NULL)
         {
-#if !HB_QSV_ONEVPL
-            /* Unload MFX plug-ins */
-            if (MFXQueryVersion(qsv_ctx->mfx_session, &version) == MFX_ERR_NONE)
-            {
-                hb_qsv_unload_plugins(&pv->loaded_plugins, qsv_ctx->mfx_session, version);
-            }
-#endif
             hb_qsv_uninit_enc(pv->job);
             if (qsv_enc_space != NULL)
             {
@@ -2025,7 +1984,7 @@ static void compute_init_delay(hb_work_private_t *pv, mfxBitstream *bs)
         if (pv->bfrm_delay < 1 || pv->bfrm_delay > BFRM_DELAY_MAX)
         {
             hb_log("compute_init_delay: "
-                   "invalid delay %d (PTS: %"PRIu64", DTS: %"PRId64")",
+                   "invalid delay %d (PTS: %llu, DTS: %lld)",
                    pv->bfrm_delay, bs->TimeStamp, bs->DecodeTimeStamp);
 
             /* we have B-frames, the frame delay should be at least 1 */
@@ -2388,11 +2347,11 @@ int encqsvWork(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
     }
     else
     {
-#if HB_PROJECT_FEATURE_QSV
         QSVMid *mid = NULL;
-        if (in->qsv_details.frame && in->qsv_details.frame->data[3])
+        AVFrame *frame = (AVFrame *)in->storage;
+        if (frame && frame->data[3])
         {
-            surface = ((mfxFrameSurface1*)in->qsv_details.frame->data[3]);
+            surface = ((mfxFrameSurface1*)frame->data[3]);
             frames_ctx = in->qsv_details.qsv_frames_ctx;
             hb_qsv_get_mid_by_surface_from_pool(frames_ctx, surface, &mid);
             hb_qsv_replace_surface_mid(frames_ctx, mid, surface);
@@ -2402,7 +2361,7 @@ int encqsvWork(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
             hb_error("encqsv: in->qsv_details no surface available");
             goto fail;
         }
-#endif
+
         // At this point, enc_qsv takes ownership of the QSV resources
         // in the 'in' buffer.
         in->qsv_details.qsv_atom = NULL;
@@ -2503,12 +2462,14 @@ int encqsvWork(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
         hb_error("encqsvWork: qsv_enc_work failed %d", err);
         goto fail;
     }
-#if HB_PROJECT_FEATURE_QSV
-    if (in->qsv_details.frame)
+
+    if (in->storage)
     {
-        in->qsv_details.frame->data[3] = 0;
+        // FIXME: This looks weird
+        AVFrame *frame = (AVFrame *)in->storage;
+        frame->data[3] = 0;
     }
-#endif
+
     *buf_out = hb_buffer_list_clear(&pv->encoded_frames);
     return HB_WORK_OK;
 
