@@ -1,6 +1,6 @@
 /* fifo.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    Copyright 2022 NVIDIA Corporation
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
@@ -16,9 +16,13 @@
 #include "handbrake/qsv_common.h"
 #endif
 
+#ifdef __APPLE__
+#include <CoreMedia/CoreMedia.h>
+#include "platform/macosx/vt_common.h"
+#endif
+
 #ifndef SYS_DARWIN
-#if defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined(SYS_NETBSD) || \
-    defined( SYS_OPENBSD ) || defined ( __OpenBSD__ )
+#if defined( SYS_FREEBSD ) || defined( SYS_NETBSD ) || defined( SYS_OPENBSD )
 #include <stdlib.h>
 #else
 #include <malloc.h>
@@ -111,6 +115,10 @@ void hb_buffer_pool_init( void )
      * 2^25 will get passed through to malloc. */
     int i;
 
+    // Create a queue with empty buffers for non native storage types
+    buffers.pool[0] = hb_fifo_init(BUFFER_POOL_MAX_ELEMENTS*10, 1);
+    buffers.pool[0]->buffer_size = 0;
+
     // Create larger queue for 2^10 bucket since all allocations smaller than
     // 2^10 come from here.
     buffers.pool[BUFFER_POOL_FIRST] = hb_fifo_init(BUFFER_POOL_MAX_ELEMENTS*10, 1);
@@ -333,6 +341,11 @@ void hb_buffer_pool_free( void )
 static hb_fifo_t *size_to_pool( int size )
 {
 #if !defined(HB_NO_BUFFER_POOL)
+    if (size == 0)
+    {
+        return buffers.pool[0];
+    }
+
     int i;
     for ( i = BUFFER_POOL_FIRST; i <= BUFFER_POOL_LAST; ++i )
     {
@@ -349,11 +362,11 @@ hb_buffer_t * hb_buffer_init_internal( int size )
 {
     hb_buffer_t * b;
     // Certain libraries (hrm ffmpeg) expect buffers passed to them to
-    // end on certain alignments (ffmpeg is 8). So allocate some extra bytes.
+    // end on certain alignments. So allocate some extra bytes.
     // Note that we can't simply align the end of our buffer because
     // sometimes we feed data to these libraries starting from arbitrary
     // points within the buffer.
-    int alloc = size + AV_INPUT_BUFFER_PADDING_SIZE;
+    int alloc = size ? size + AV_INPUT_BUFFER_PADDING_SIZE : 0;
     hb_fifo_t *buffer_pool = size_to_pool( alloc );
 
     if( buffer_pool )
@@ -371,7 +384,11 @@ hb_buffer_t * hb_buffer_init_internal( int size )
             memset( b, 0, sizeof(hb_buffer_t) );
             b->alloc          = buffer_pool->buffer_size;
             b->size           = size;
-            b->data           = data;
+            if (size)
+            {
+                b->data           = data;
+            }
+            b->storage        = NULL;
             b->s.start        = AV_NOPTS_VALUE;
             b->s.stop         = AV_NOPTS_VALUE;
             b->s.renderOffset = AV_NOPTS_VALUE;
@@ -426,6 +443,11 @@ hb_buffer_t * hb_buffer_init_internal( int size )
     return b;
 }
 
+hb_buffer_t * hb_buffer_wrapper_init()
+{
+    return hb_buffer_init_internal(0);
+}
+
 hb_buffer_t * hb_buffer_init( int size )
 {
     return hb_buffer_init_internal(size);
@@ -471,38 +493,255 @@ void hb_buffer_realloc( hb_buffer_t * b, int size )
 
 void hb_buffer_reduce( hb_buffer_t * b, int size )
 {
-
-    if ( size < b->alloc / 8 || b->data == NULL )
+    if (b->storage_type == STANDARD && (size < b->alloc / 8 || b->data == NULL))
     {
-        hb_buffer_t * tmp = hb_buffer_init( size );
-
-        hb_buffer_swap_copy( b, tmp );
-        memcpy( b->data, tmp->data, size );
-        tmp->next = NULL;
-        hb_buffer_close( &tmp );
+        hb_buffer_t *tmp = hb_buffer_init(size);
+        if (tmp)
+        {
+            hb_buffer_swap_copy(b, tmp);
+            if (tmp->data)
+            {
+                memcpy(b->data, tmp->data, size);
+            }
+            tmp->next = NULL;
+        }
+        hb_buffer_close(&tmp);
     }
 }
 
-hb_buffer_t * hb_buffer_dup( const hb_buffer_t * src )
+AVFrameSideData *hb_buffer_new_side_data_from_buf(hb_buffer_t *buf,
+                                                  enum AVFrameSideDataType type,
+                                                  AVBufferRef *side_data_buf)
 {
-
-    hb_buffer_t * buf;
-
-    if ( src == NULL )
-        return NULL;
-
-    buf = hb_buffer_init( src->size );
-    if ( buf )
+    AVFrameSideData *ret;
+    if (buf->storage_type == AVFRAME)
     {
-        memcpy( buf->data, src->data, src->size );
-        buf->s = src->s;
-        buf->f = src->f;
-        if ( buf->s.type == FRAME_BUF )
-            hb_buffer_init_planes( buf );
+        AVFrame *frame = (AVFrame *)buf->storage;
+        ret = av_frame_new_side_data_from_buf(frame, type, side_data_buf);
+        buf->side_data = (void **)frame->side_data;
+        buf->nb_side_data = frame->nb_side_data;
+        return ret;
+    }
+    else
+    {
+        AVFrameSideData **tmp;
+
+        if (!buf)
+        {
+            return NULL;
+        }
+
+        if (buf->nb_side_data > INT_MAX / sizeof(*buf->side_data) - 1)
+        {
+            return NULL;
+        }
+
+        tmp = av_realloc(buf->side_data, (buf->nb_side_data + 1) * sizeof(*buf->side_data));
+        if (!tmp)
+        {
+            return NULL;
+        }
+        buf->side_data = (void **)tmp;
+
+        ret = av_mallocz(sizeof(*ret));
+        if (!ret)
+        {
+            return NULL;
+        }
+
+        ret->buf = side_data_buf;
+        ret->data = ret->buf->data;
+        ret->size = side_data_buf->size;
+        ret->type = type;
+
+        buf->side_data[buf->nb_side_data++] = ret;
+
+        return ret;
+    }
+}
+
+static void free_side_data(AVFrameSideData **ptr_sd)
+{
+    AVFrameSideData *sd = *ptr_sd;
+
+    av_buffer_unref(&sd->buf);
+    av_dict_free(&sd->metadata);
+    av_freep(ptr_sd);
+}
+
+void hb_buffer_remove_side_data(hb_buffer_t *buf, enum AVFrameSideDataType type)
+{
+    if (buf->storage_type == AVFRAME)
+    {
+        AVFrame *frame = (AVFrame *)buf->storage;
+        av_frame_remove_side_data(frame, type);
+        buf->nb_side_data = frame->nb_side_data;
+    }
+    else
+    {
+        for (int i = buf->nb_side_data - 1; i >= 0; i--)
+        {
+            AVFrameSideData *sd = buf->side_data[i];
+            if (sd->type == type)
+            {
+                free_side_data((AVFrameSideData **)&buf->side_data[i]);
+                buf->side_data[i] = buf->side_data[buf->nb_side_data - 1];
+                buf->nb_side_data--;
+            }
+        }
+    }
+}
+
+void hb_buffer_wipe_side_data(hb_buffer_t *buf)
+{
+    for (int i = 0; i < buf->nb_side_data; i++)
+    {
+        free_side_data((AVFrameSideData **)&buf->side_data[i]);
+    }
+    buf->nb_side_data = 0;
+
+    av_freep(&buf->side_data);
+}
+
+void hb_buffer_copy_side_data(hb_buffer_t *dst, const hb_buffer_t *src)
+{
+    for (int i = 0; i < src->nb_side_data; i++)
+    {
+        const AVFrameSideData *sd_src = src->side_data[i];
+        AVBufferRef *ref = av_buffer_ref(sd_src->buf);
+        AVFrameSideData *sd_dst = hb_buffer_new_side_data_from_buf(dst, sd_src->type, ref);
+        if (!sd_dst)
+        {
+            av_buffer_unref(&ref);
+            hb_buffer_wipe_side_data(dst);
+        }
+    }
+}
+
+void hb_buffer_copy_props(hb_buffer_t *dst, const hb_buffer_t *src)
+{
+    dst->s = src->s;
+    hb_buffer_copy_side_data(dst, src);
+}
+
+static int copy_hwframe_to_video_buffer(const AVFrame *frame, hb_buffer_t *buf)
+{
+    int ret;
+    AVFrame *hw_frame = av_frame_alloc();
+
+    ret = av_frame_copy_props(hw_frame, frame);
+    if (ret < 0)
+    {
+        hb_log("fifo: av_frame_copy_props");
+    }
+    ret = av_hwframe_get_buffer(frame->hw_frames_ctx, hw_frame, 0);
+    if (ret < 0)
+    {
+        hb_log("fifo: av_hwframe_get_buffer failed");
+    }
+    ret = av_hwframe_transfer_data(hw_frame, frame, 0);
+    if (ret < 0)
+    {
+        hb_log("fifo: av_hwframe_transfer_data failed");
     }
 
+    buf->storage = hw_frame;
+    buf->storage_type = AVFRAME;
+
+    return ret;
+}
+
+static void copy_avframe_to_video_buffer(const AVFrame *frame, hb_buffer_t *buf)
+{
+    for (int pp = 0; pp <= buf->f.max_plane; pp++)
+    {
+        if (buf->plane[pp].stride == frame->linesize[pp])
+        {
+            memcpy(buf->plane[pp].data, frame->data[pp], frame->linesize[pp] * buf->plane[pp].height);
+        }
+        else
+        {
+            const int stride    = buf->plane[pp].stride;
+            const int height    = buf->plane[pp].height;
+            const int linesize  = frame->linesize[pp];
+            const int size = linesize < stride ? ABS(linesize) : stride;
+            uint8_t *dst = buf->plane[pp].data;
+            uint8_t *src = frame->data[pp];
+            for (int yy = 0; yy < height; yy++)
+            {
+                memcpy(dst, src, size);
+                dst += stride;
+                src += linesize;
+            }
+        }
+    }
+}
+
+hb_buffer_t * hb_buffer_dup(const hb_buffer_t *src)
+{
+    hb_buffer_t *buf = NULL;
+
+    if (src == NULL)
+    {
+        return NULL;
+    }
+
+    if (src->storage_type == STANDARD)
+    {
+        buf = hb_buffer_init(src->size);
+        if (buf)
+        {
+            buf->f = src->f;
+            hb_buffer_copy_props(buf, src);
+
+            if (buf->s.type == FRAME_BUF)
+            {
+                hb_buffer_init_planes(buf);
+            }
+
+            memcpy(buf->data, src->data, src->size);
+        }
+    }
+    else if (src->storage_type == AVFRAME)
+    {
+        const AVFrame *frame = (AVFrame *)src->storage;
+
+        // If it's an hardware frame, make a copy
+        // into another hardware AVFrame.
+        if (frame->hw_frames_ctx)
+        {
+            buf = hb_buffer_wrapper_init();
+            if (buf)
+            {
+                buf->f = src->f;
+                hb_buffer_copy_props(buf, src);
+                copy_hwframe_to_video_buffer(frame, buf);
+            }
+        }
+        // If not, copy the content to a standard hb_buffer
+        else
+        {
+            buf = hb_frame_buffer_init(src->f.fmt, src->f.width, src->f.height);
+            if (buf)
+            {
+                buf->f = src->f;
+                hb_buffer_copy_props(buf, src);
+                copy_avframe_to_video_buffer(frame, buf);
+            }
+        }
+    }
+#ifdef __APPLE__
+    else if (src->storage_type == COREMEDIA)
+    {
+        buf = hb_vt_buffer_dup(src);
+    }
+#endif
+
 #if HB_PROJECT_FEATURE_QSV
-    memcpy(&buf->qsv_details, &src->qsv_details, sizeof(src->qsv_details));
+    if (buf)
+    {
+        memcpy(&buf->qsv_details, &src->qsv_details, sizeof(src->qsv_details));
+    }
 #endif
 
     return buf;
@@ -517,8 +756,8 @@ int hb_buffer_copy(hb_buffer_t * dst, const hb_buffer_t * src)
         return -1;
 
     memcpy( dst->data, src->data, src->size );
-    dst->s = src->s;
     dst->f = src->f;
+    hb_buffer_copy_props(dst, src);
     if (dst->s.type == FRAME_BUF)
         hb_buffer_init_planes(dst);
 
@@ -587,11 +826,6 @@ hb_buffer_t * hb_frame_buffer_init( int pix_fmt, int width, int height )
 
     hb_buffer_init_planes(buf);
 
-#if HB_PROJECT_FEATURE_NVENC
-    if (AV_PIX_FMT_CUDA == pix_fmt)
-        buf->hw_ctx.frame = av_frame_alloc();
-#endif
-
     return buf;
 }
 
@@ -737,15 +971,15 @@ void hb_buffer_close( hb_buffer_t ** _b )
 #if HB_PROJECT_FEATURE_QSV
         // Reclaim QSV resources before dropping the buffer.
         // when decoding without QSV, the QSV atom will be NULL.
-        if(b->qsv_details.frame && b->qsv_details.ctx != NULL)
+        if (b->storage != NULL && b->qsv_details.ctx != NULL)
         {
-            mfxFrameSurface1 *surface = (mfxFrameSurface1*)b->qsv_details.frame->data[3];
-            if(surface)
+            AVFrame *frame = (AVFrame *)b->storage;
+            mfxFrameSurface1 *surface = (mfxFrameSurface1 *)frame->data[3];
+            if (surface)
             {
                 hb_qsv_release_surface_from_pool_by_surface_pointer(b->qsv_details.qsv_frames_ctx, surface);
-                b->qsv_details.frame->data[3] = 0;
+                frame->data[3] = 0;
             }
-            av_frame_unref(b->qsv_details.frame);
         }
         if (b->qsv_details.qsv_atom != NULL && b->qsv_details.ctx != NULL)
         {
@@ -763,15 +997,10 @@ void hb_buffer_close( hb_buffer_t ** _b )
                 }
             }
             hb_qsv_flush_stages(b->qsv_details.ctx->pipes,
-                                (hb_qsv_list**)&b->qsv_details.qsv_atom);
+                                (hb_qsv_list**)&b->qsv_details.qsv_atom, 1);
         }
 #endif
 
-#if HB_PROJECT_FEATURE_NVENC
-        if (b->hw_ctx.frame)
-            av_frame_free((AVFrame**)&b->hw_ctx.frame);
-#endif
-
         hb_buffer_t * next = b->next;
         hb_fifo_t *buffer_pool = size_to_pool( b->alloc );
 
@@ -782,7 +1011,24 @@ void hb_buffer_close( hb_buffer_t ** _b )
         hb_list_rem(buffers.alloc_list, b);
         hb_unlock(buffers.lock);
 #endif
-        if( buffer_pool && b->data && !hb_fifo_is_full( buffer_pool ) )
+        if (b->storage_type == AVFRAME)
+        {
+            av_frame_unref((AVFrame *)b->storage);
+            av_frame_free((AVFrame **)&b->storage);
+        }
+#ifdef __APPLE__
+        else if (b->storage_type == COREMEDIA)
+        {
+            CFRelease((CMSampleBufferRef)b->storage);
+        }
+#endif
+        if (b->storage_type != AVFRAME)
+        {
+            hb_buffer_wipe_side_data(b);
+            av_freep(&b->side_data);
+        }
+
+        if (buffer_pool && !hb_fifo_is_full(buffer_pool))
         {
 #if defined(HB_BUFFER_DEBUG)
             if (hb_fifo_contains(buffer_pool, b))
@@ -797,7 +1043,7 @@ void hb_buffer_close( hb_buffer_t ** _b )
         }
         // either the pool is full or this size doesn't use a pool
         // free the buf
-        if( b->data )
+        if (b->data && b->storage_type == STANDARD)
         {
             av_free(b->data);
             hb_lock(buffers.lock);
@@ -890,7 +1136,6 @@ hb_image_t * hb_buffer_to_image(hb_buffer_t *buf)
     image->color_prim     = buf->f.color_prim;
     image->color_transfer = buf->f.color_transfer;
     image->color_matrix   = buf->f.color_matrix;
-    memcpy(image->data, buf->data, buf->size);
 
     int p;
     uint8_t *data = image->data;
@@ -902,6 +1147,9 @@ hb_image_t * hb_buffer_to_image(hb_buffer_t *buf)
         image->plane[p].stride = buf->plane[p].stride;
         image->plane[p].height_stride = buf->plane[p].height_stride;
         image->plane[p].size = buf->plane[p].size;
+
+        memcpy(image->plane[p].data, buf->plane[p].data, buf->plane[p].size);
+
         data += image->plane[p].size;
     }
     return image;
