@@ -1,6 +1,6 @@
 /* common.h
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -138,6 +138,8 @@ void        hb_list_rem( hb_list_t *, void * );
 void      * hb_list_item( const hb_list_t *, int );
 void        hb_list_close( hb_list_t ** );
 
+hb_list_t * hb_string_list_copy(const hb_list_t *src);
+
 void hb_reduce( int *x, int *y, int num, int den );
 void hb_limit_rational( int *x, int *y, int64_t num, int64_t den, int limit );
 void hb_reduce64( int64_t *x, int64_t *y, int64_t num, int64_t den );
@@ -333,7 +335,8 @@ struct hb_subtitle_config_s
     int64_t      offset;
 };
 
-struct hb_mastering_display_metadata_s {
+struct hb_mastering_display_metadata_s
+{
     hb_rational_t display_primaries[3][2];
     hb_rational_t white_point[2];
     hb_rational_t min_luminance;
@@ -342,11 +345,33 @@ struct hb_mastering_display_metadata_s {
     int has_luminance;
 };
 
-struct hb_content_light_metadata_s {
+struct hb_content_light_metadata_s
+{
     unsigned max_cll;
     unsigned max_fall;
 };
 
+struct hb_ambient_viewing_environment_metadata_s
+{
+    hb_rational_t ambient_illuminance;
+    hb_rational_t ambient_light_x;
+    hb_rational_t ambient_light_y;
+};
+
+struct hb_dovi_conf_s
+{
+    unsigned dv_version_major;
+    unsigned dv_version_minor;
+    unsigned dv_profile;
+    unsigned dv_level;
+    unsigned rpu_present_flag;
+    unsigned el_present_flag;
+    unsigned bl_present_flag;
+    unsigned dv_bl_signal_compatibility_id;
+};
+
+int hb_str_ends_with(const char *base, const char *str);
+
 /*******************************************************************************
  * Lists of rates, mixdowns, encoders etc.
  *******************************************************************************
@@ -408,9 +433,10 @@ void        hb_video_quality_get_limits(uint32_t codec, float *low, float *high,
 const char* hb_video_quality_get_name(uint32_t codec);
 int         hb_video_quality_is_supported(uint32_t codec);
 
-int         hb_video_twopass_is_supported(uint32_t codec);
+int         hb_video_multipass_is_supported(uint32_t codec);
 
 int                hb_video_encoder_is_supported(int encoder);
+int                hb_video_encoder_get_count_of_analysis_passes(int encoder);
 int                hb_video_encoder_pix_fmt_is_supported(int encoder, int pix_fmt, const char *profile);
 int                hb_video_encoder_get_depth   (int encoder);
 const char* const* hb_video_encoder_get_presets (int encoder);
@@ -554,15 +580,17 @@ struct hb_job_s
          cfr:               0 (vfr), 1 (cfr), 2 (pfr) [see render.c]
          pass:              0, 1 or 2 (or -1 for scan)
          areBframes:        boolean to note if b-frames are used */
+
+    // Changing the constant values requires changes in win/CS/HandBrake.Interop/Interop/HbLib/NativeConstants.cs for Windows GUI
 #define HB_VCODEC_INVALID            0x00000000
 
 #define HB_VCODEC_AV1_MASK           0x40000000
 #define HB_VCODEC_H264_MASK          0x20000000
 #define HB_VCODEC_H265_MASK          0x10000000
 
-#define HB_VCODEC_SVT_AV1_MASK      (0x00800000 | HB_VCODEC_AV1_MASK | HB_VCODEC_FFMPEG_MASK)
-#define HB_VCODEC_X264_MASK         (0x00400000 | HB_VCODEC_H264_MASK)
-#define HB_VCODEC_X265_MASK         (0x00200000 | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_SVT_AV1_MASK       0x00800000
+#define HB_VCODEC_X264_MASK          0x00400000
+#define HB_VCODEC_X265_MASK          0x00200000
 
 #define HB_VCODEC_VT_MASK            0x00080000
 #define HB_VCODEC_QSV_MASK           0x00040000
@@ -570,15 +598,15 @@ struct hb_job_s
 
 #define HB_VCODEC_THEORA             0x00000001
 
-#define HB_VCODEC_X264_8BIT         (0x00000002 | HB_VCODEC_X264_MASK)
+#define HB_VCODEC_X264_8BIT         (0x00000002 | HB_VCODEC_X264_MASK | HB_VCODEC_H264_MASK)
 #define HB_VCODEC_X264              HB_VCODEC_X264_8BIT
-#define HB_VCODEC_X264_10BIT        (0x00000003 | HB_VCODEC_X264_MASK)
+#define HB_VCODEC_X264_10BIT        (0x00000003 | HB_VCODEC_X264_MASK | HB_VCODEC_H264_MASK)
 
-#define HB_VCODEC_X265_8BIT         (0x00000004 | HB_VCODEC_X265_MASK)
+#define HB_VCODEC_X265_8BIT         (0x00000004 | HB_VCODEC_X265_MASK | HB_VCODEC_H265_MASK)
 #define HB_VCODEC_X265              HB_VCODEC_X265_8BIT
-#define HB_VCODEC_X265_10BIT        (0x00000005 | HB_VCODEC_X265_MASK)
-#define HB_VCODEC_X265_12BIT        (0x00000006 | HB_VCODEC_X265_MASK)
-#define HB_VCODEC_X265_16BIT        (0x00000007 | HB_VCODEC_X265_MASK)
+#define HB_VCODEC_X265_10BIT        (0x00000005 | HB_VCODEC_X265_MASK | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_X265_12BIT        (0x00000006 | HB_VCODEC_X265_MASK | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_X265_16BIT        (0x00000007 | HB_VCODEC_X265_MASK | HB_VCODEC_H265_MASK)
 
 #define HB_VCODEC_FFMPEG_MPEG4      (0x00000008 | HB_VCODEC_FFMPEG_MASK)
 #define HB_VCODEC_FFMPEG_MPEG2      (0x00000009 | HB_VCODEC_FFMPEG_MASK)
@@ -590,6 +618,7 @@ struct hb_job_s
 #define HB_VCODEC_FFMPEG_VCE_H264           (0x0000000D | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H264_MASK)
 #define HB_VCODEC_FFMPEG_VCE_H265           (0x0000000E | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
 #define HB_VCODEC_FFMPEG_VCE_H265_10BIT     (0x0000000F | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_FFMPEG_VCE_AV1            (0x00000010 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_AV1_MASK)
 
 #define HB_VCODEC_FFMPEG_MF_H264    (0x00000020 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H264_MASK)
 #define HB_VCODEC_FFMPEG_MF_H265    (0x00000021 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
@@ -597,10 +626,12 @@ struct hb_job_s
 #define HB_VCODEC_FFMPEG_NVENC_H264         (0x00000030 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H264_MASK)
 #define HB_VCODEC_FFMPEG_NVENC_H265         (0x00000031 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
 #define HB_VCODEC_FFMPEG_NVENC_H265_10BIT   (0x00000032 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_FFMPEG_NVENC_AV1          (0x00000033 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_AV1_MASK)
+#define HB_VCODEC_FFMPEG_NVENC_AV1_10BIT    (0x00000034 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_AV1_MASK)
 
-#define HB_VCODEC_FFMPEG_SVT_AV1_8BIT       (0x00000041 | HB_VCODEC_SVT_AV1_MASK)
-#define HB_VCODEC_FFMPEG_SVT_AV1            HB_VCODEC_FFMPEG_SVT_AV1_8BIT
-#define HB_VCODEC_FFMPEG_SVT_AV1_10BIT      (0x00000042 | HB_VCODEC_SVT_AV1_MASK)
+#define HB_VCODEC_SVT_AV1_8BIT       (0x00000041 | HB_VCODEC_SVT_AV1_MASK | HB_VCODEC_AV1_MASK)
+#define HB_VCODEC_SVT_AV1            HB_VCODEC_SVT_AV1_8BIT
+#define HB_VCODEC_SVT_AV1_10BIT      (0x00000042 | HB_VCODEC_SVT_AV1_MASK | HB_VCODEC_AV1_MASK)
 
 #define HB_VCODEC_VT_H264           (0x00000050 | HB_VCODEC_VT_MASK | HB_VCODEC_H264_MASK)
 #define HB_VCODEC_VT_H265           (0x00000051 | HB_VCODEC_VT_MASK | HB_VCODEC_H265_MASK)
@@ -628,8 +659,8 @@ struct hb_job_s
     hb_rational_t   orig_vrate;
     int             cfr;
     PRIVATE int     pass_id;
-    int             twopass;        // Enable 2-pass encode. Boolean
-    int             fastfirstpass;
+    int             multipass;        // Enable multi-pass encode. Boolean
+    int             fastanalysispass;
     char           *encoder_preset;
     char           *encoder_tune;
     char           *encoder_options;
@@ -653,6 +684,7 @@ struct hb_job_s
 // see https://developer.apple.com/library/content/technotes/tn2162/_index.html
 //     https://developer.apple.com/library/content/documentation/QuickTime/QTFF/QTFFChap3/qtff3.html#//apple_ref/doc/uid/TP40000939-CH205-125526
 //     libav pixfmt.h
+#define HB_COLR_PRI_UNSET       -1
 #define HB_COLR_PRI_BT709        1
 #define HB_COLR_PRI_UNDEF        2
 #define HB_COLR_PRI_BT470M       4
@@ -666,6 +698,7 @@ struct hb_job_s
 #define HB_COLR_PRI_SMPTE432     12
 #define HB_COLR_PRI_JEDEC_P22    22
 // 0, 3-4, 7-8, 10-65535: reserved/not implemented
+#define HB_COLR_TRA_UNSET       -1
 #define HB_COLR_TRA_BT709        1 // also use for bt470m, bt470bg, smpte170m, bt2020_10 and bt2020_12
 #define HB_COLR_TRA_UNDEF        2
 #define HB_COLR_TRA_GAMMA22      4
@@ -684,6 +717,7 @@ struct hb_job_s
 #define HB_COLR_TRA_SMPTE428     17
 #define HB_COLR_TRA_ARIB_STD_B67 18 //known as "Hybrid log-gamma"
 // 0, 3-6, 8-15, 17-65535: reserved/not implemented
+#define HB_COLR_MAT_UNSET       -1
 #define HB_COLR_MAT_RGB          0
 #define HB_COLR_MAT_BT709        1
 #define HB_COLR_MAT_UNDEF        2
@@ -702,6 +736,11 @@ struct hb_job_s
 
     hb_mastering_display_metadata_t mastering;
     hb_content_light_metadata_t coll;
+    hb_ambient_viewing_environment_metadata_t ambient;
+    hb_dovi_conf_t dovi;
+
+    enum {NONE = 0x0, ALL = 0x3, DOVI = 0x1, HDR_10_PLUS = 0x2} passthru_dynamic_hdr_metadata;
+
 
     hb_list_t     * list_chapter;
 
@@ -755,7 +794,7 @@ struct hb_job_s
                                         // faithful reproduction of the source
                                         // stream and may have blank frames
                                         // added or initial frames dropped.
-    int             mp4_optimize;
+    int             optimize;
     int             ipod_atom;
 
     int                     indepth_scan;
@@ -783,16 +822,6 @@ struct hb_job_s
 #if HB_PROJECT_FEATURE_QSV
         hb_qsv_context *ctx;
 #endif
-        // shared encoding parameters
-        // initialized by the QSV encoder, then used upstream (e.g. by filters)
-        // to configure their output so that it matches what the encoder expects
-        struct
-        {
-            int pic_struct;
-            int align_width;
-            int align_height;
-            int is_init_done;
-        } enc_info;
     } qsv;
 
     int hw_decode;
@@ -821,12 +850,9 @@ struct hb_job_s
     int64_t         reader_pts_offset; // Reader can discard some video.
                                        // Other pipeline stages need to know
                                        // this.  E.g. sync and decsrtsub
-#endif
-#if HB_PROJECT_FEATURE_NVENC
-    struct
-    {
-        void *hw_device_ctx;
-    } nv_hw_ctx;
+
+    void           *hw_device_ctx;
+    int             hw_pix_fmt;
 #endif
 };
 
@@ -1165,8 +1191,13 @@ struct hb_title_s
     int             color_matrix;
     int             color_range;
     int             chroma_location;
+
     hb_mastering_display_metadata_t mastering;
     hb_content_light_metadata_t     coll;
+    hb_ambient_viewing_environment_metadata_t ambient;
+    hb_dovi_conf_t  dovi;
+    int             hdr_10_plus;
+
     hb_rational_t   vrate;
     int             crop[4];
     int             loose_crop[4];
@@ -1179,6 +1210,7 @@ struct hb_title_s
     uint32_t        video_stream_type;      /* stream type from source stream */
     int             video_codec_param;      /* codec specific config */
     char          * video_codec_name;
+    int             video_codec_profile;
     int             video_bitrate;
     hb_rational_t   video_timebase;
     char          * container_name;
@@ -1186,9 +1218,13 @@ struct hb_title_s
 
     // additional supported video decoders (e.g. HW-accelerated implementations)
     int           video_decode_support;
-#define HB_DECODE_SUPPORT_SW    0x01 // software (libavcodec or mpeg2dec)
-#define HB_DECODE_SUPPORT_QSV   0x02 // Intel Quick Sync Video
-#define HB_DECODE_SUPPORT_NVDEC 0x04 // Nvidia Nvdec
+#define HB_DECODE_SUPPORT_SW             0x01 // software (libavcodec or mpeg2dec)
+#define HB_DECODE_SUPPORT_QSV            0x02 // Intel Quick Sync Video
+#define HB_DECODE_SUPPORT_NVDEC          0x04
+#define HB_DECODE_SUPPORT_VIDEOTOOLBOX   0x08
+
+#define HB_DECODE_SUPPORT_HWACCEL        (HB_DECODE_SUPPORT_NVDEC | HB_DECODE_SUPPORT_VIDEOTOOLBOX)
+#define HB_DECODE_SUPPORT_FORCE_HW       0x80000000
 
     hb_metadata_t * metadata;
 
@@ -1235,8 +1271,8 @@ struct hb_state_s
             /* HB_STATE_WORKING || HB_STATE_SEARCHING || HB_STATE_WORKDONE */
 #define HB_PASS_SUBTITLE    -1
 #define HB_PASS_ENCODE      0
-#define HB_PASS_ENCODE_1ST  1   // Some code depends on these values being
-#define HB_PASS_ENCODE_2ND  2   // 1 and 2.  Do not change.
+#define HB_PASS_ENCODE_ANALYSIS  1   // Some code depends on these values being
+#define HB_PASS_ENCODE_FINAL     2   // 1 and 2.  Do not change.
             int           pass_id;
             int           pass;
             int           pass_count;
@@ -1335,6 +1371,7 @@ struct hb_work_object_s
     int                 status;
     int                 frame_count;
     int                 codec_param;
+    void              * hw_device_ctx;
     hb_title_t        * title;
 
     hb_work_object_t  * next;
@@ -1358,6 +1395,7 @@ extern hb_work_object_t hb_encvt;
 extern hb_work_object_t hb_encx264;
 extern hb_work_object_t hb_enctheora;
 extern hb_work_object_t hb_encx265;
+extern hb_work_object_t hb_encsvtav1;
 extern hb_work_object_t hb_decavcodeca;
 extern hb_work_object_t hb_decavcodecv;
 extern hb_work_object_t hb_declpcm;
@@ -1378,6 +1416,7 @@ typedef struct hb_filter_init_s
 {
     hb_job_t      * job;
     int             pix_fmt;
+    int             hw_pix_fmt;
     int             color_prim;
     int             color_transfer;
     int             color_matrix;
@@ -1389,12 +1428,7 @@ typedef struct hb_filter_init_s
     int             cfr;
     int             grayscale;
     hb_rational_t   time_base;
-#if HB_PROJECT_FEATURE_NVENC
-    struct
-    {
-        void *hw_frames_ctx;
-    } nv_hw_ctx;
-#endif
+    void          * hw_frames_ctx;
 } hb_filter_init_t;
 
 typedef struct hb_filter_info_s
@@ -1449,16 +1483,17 @@ struct hb_filter_object_s
 enum
 {
     HB_FILTER_INVALID = 0,
-    // for QSV - important to have before other filters
     HB_FILTER_FIRST = 1,
-    HB_FILTER_QSV_PRE = 1,
 
+    HB_FILTER_PRE_VT,
     // First, filters that may change the framerate (drop or dup frames)
     HB_FILTER_DETELECINE,
     HB_FILTER_COMB_DETECT,
     HB_FILTER_DECOMB,
     HB_FILTER_YADIF,
+    HB_FILTER_YADIF_VT,
     HB_FILTER_BWDIF,
+    HB_FILTER_BWDIF_VT,
     HB_FILTER_VFR,
     // Filters that must operate on the original source image are next
     HB_FILTER_DEBLOCK,
@@ -1466,25 +1501,29 @@ enum
     HB_FILTER_HQDN3D = HB_FILTER_DENOISE,
     HB_FILTER_NLMEANS,
     HB_FILTER_CHROMA_SMOOTH,
+    HB_FILTER_CHROMA_SMOOTH_VT,
     HB_FILTER_ROTATE,
+    HB_FILTER_ROTATE_VT,
     HB_FILTER_RENDER_SUB,
     HB_FILTER_CROP_SCALE,
+    HB_FILTER_CROP_SCALE_VT,
     HB_FILTER_LAPSHARP,
+    HB_FILTER_LAPSHARP_VT,
     HB_FILTER_UNSHARP,
+    HB_FILTER_UNSHARP_VT,
     HB_FILTER_GRAYSCALE,
+    HB_FILTER_GRAYSCALE_VT,
     HB_FILTER_PAD,
+    HB_FILTER_PAD_VT,
     HB_FILTER_COLORSPACE,
     HB_FILTER_FORMAT,
+    HB_FILTER_RPU,
 
     // Finally filters that don't care what order they are in,
     // except that they must be after the above filters
     HB_FILTER_AVFILTER,
 
-    // for QSV - important to have as a last one
-    HB_FILTER_QSV_POST,
-    // default MSDK VPP filter
-    HB_FILTER_QSV,
-    HB_FILTER_LAST = HB_FILTER_QSV,
+    HB_FILTER_LAST,
     // wrapper filter for frame based multi-threading of simple filters
     HB_FILTER_MT_FRAME
 };
@@ -1509,6 +1548,9 @@ typedef void hb_error_handler_t( const char *errmsg );
 
 extern void hb_register_error_handler( hb_error_handler_t * handler );
 
+void hb_str_to_locale(char *buffer);
+void hb_str_from_locale(char *buffer);
+
 char * hb_strdup_vaprintf( const char * fmt, va_list args );
 char * hb_strdup_printf(const char *fmt, ...) HB_WPRINTF(1, 2);
 char * hb_strncat_dup( const char * s1, const char * s2, size_t n );
@@ -1523,6 +1565,7 @@ char ** hb_str_vsplit( const char * str, char delem );
 int hb_yuv2rgb(int yuv);
 int hb_rgb2yuv(int rgb);
 int hb_rgb2yuv_bt709(int rgb);
+int hb_rgb2yuv_bt2020(int rgb);
 
 const char * hb_subsource_name( int source );
 
@@ -1547,6 +1590,7 @@ int hb_output_color_matrix(hb_job_t * job);
 int hb_get_bit_depth(int format);
 int hb_get_chroma_sub_sample(int format, int *h_shift, int *v_shift);
 int hb_get_best_pix_fmt(hb_job_t * job);
+int hb_get_best_hw_pix_fmt(hb_job_t * job);
 
 #define HB_NEG_FLOAT_REG "(([-])?(([0-9]+([.,][0-9]+)?)|([.,][0-9]+))"
 #define HB_FLOAT_REG     "(([0-9]+([.,][0-9]+)?)|([.,][0-9]+))"
