@@ -1,6 +1,6 @@
 /* h265_common.h
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -26,10 +26,6 @@ static const char * const hb_h265_profile_names_10bit[]     = {
     "auto", "main10", "main10-intra", NULL, };
 static const char * const hb_x265_profile_names_10bit[]     = {
     "auto", "main10", "main10-intra", "main422-10", "main422-10-intra", "main444-10", "main444-10-intra", NULL, };
-#if HB_PROJECT_FEATURE_QSV
-static const char * const hb_h265_qsv_profile_names_10bit[] = {
-    "auto", "main10", NULL, };
-#endif
 static const char * const hb_h265_profile_names_12bit[]     = {
     "auto", "main12", "main12-intra", NULL, };
 static const char * const hb_x265_profile_names_12bit[]     = {
