@@ -1,6 +1,6 @@
 /* handbrake.h
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -47,12 +47,15 @@ void          hb_dvd_set_dvdnav( int enable );
    Scan the specified path. Can be a DVD device, a VIDEO_TS folder or
    a VOB file. If title_index is 0, scan all titles. */
 void          hb_scan( hb_handle_t *, const char * path,
-                       int title_index, int preview_count,
-                       int store_previews, uint64_t min_duration );
-void          hb_scan2( hb_handle_t *, const char * path,
                        int title_index, int preview_count,
                        int store_previews, uint64_t min_duration,
-                       int crop_auto_switch_threshold, int crop_median_threshold );
+                       int crop_auto_switch_threshold, int crop_median_threshold,
+                       hb_list_t * exclude_extensions, int hw_decode);
+                       
+void          hb_scan_list( hb_handle_t * h, hb_list_t * paths, int title_index,
+                      int preview_count, int store_previews, uint64_t min_duration,
+                      int crop_threshold_frames, int crop_threshold_pixels,
+                      hb_list_t * exclude_extensions, int hw_decode);
 
 void          hb_scan_stop( hb_handle_t * );
 void          hb_force_rescan( hb_handle_t * );
@@ -126,9 +129,8 @@ typedef struct hb_interjob_s
 
     hb_subtitle_t *select_subtitle; /* foreign language scan subtitle */
 
-#ifdef __APPLE__
-     void *vt_context;
- #endif
+    void *context;
+    int   context_size;
 } hb_interjob_t;
 
 hb_interjob_t * hb_interjob_get( hb_handle_t * );
