@@ -1,6 +1,6 @@
 /* hb.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -80,7 +80,7 @@ void hb_avcodec_init()
 {
 }
 
-int hb_avcodec_open(AVCodecContext *avctx, AVCodec *codec,
+int hb_avcodec_open(AVCodecContext *avctx, const AVCodec *codec,
                     AVDictionary **av_opts, int thread_count)
 {
     int ret;
@@ -117,6 +117,11 @@ int hb_picture_fill(uint8_t *data[], int stride[], hb_buffer_t *buf)
 {
     int ret, ii;
 
+    if (buf->f.max_plane < 0)
+    {
+        return -1;
+    }
+
     for (ii = 0; ii <= buf->f.max_plane; ii++)
         stride[ii] = buf->plane[ii].stride;
     for (; ii < 4; ii++)
@@ -354,6 +359,20 @@ void hb_remove_previews( hb_handle_t * h )
     closedir( dir );
 }
 
+void hb_scan( hb_handle_t * h, const char * path, int title_index,
+              int preview_count, int store_previews, uint64_t min_duration,
+              int crop_threshold_frames, int crop_threshold_pixels,
+              hb_list_t * exclude_extensions, int hw_decode)
+{
+    // TODO: Compatibility later for the other UI's.  Remove when they are updated.
+    hb_list_t *file_paths = hb_list_init();
+    hb_list_add(file_paths, (char *)path);
+
+    hb_scan_list(h, file_paths, title_index, preview_count, store_previews, min_duration, crop_threshold_frames, crop_threshold_pixels, exclude_extensions, hw_decode);
+
+    hb_list_close(&file_paths);
+}
+
 /**
  * Initializes a scan of the by calling hb_scan_init
  * @param h Handle to hb_handle_t
@@ -361,16 +380,31 @@ void hb_remove_previews( hb_handle_t * h )
  * @param title_index Desired title to scan.  0 for all titles.
  * @param preview_count Number of preview images to generate.
  * @param store_previews Whether or not to write previews to disk.
+ * @param min_duration Ignore titles below a given threshold
+ * @param crop_threshold_frames The number of frames to trigger smart crop
+ * @param crop_threshold_pixels The variance in pixels detected that are allowed for.
+ * @param exclude_extensions A list of extensions to exclude for this scan.
+ * @param hw_decode  The preferred hardware decoder to use..
  */
-void hb_scan( hb_handle_t * h, const char * path, int title_index,
-              int preview_count, int store_previews, uint64_t min_duration )
+void hb_scan_list( hb_handle_t * h, hb_list_t * paths, int title_index,
+              int preview_count, int store_previews, uint64_t min_duration,
+              int crop_threshold_frames, int crop_threshold_pixels,
+              hb_list_t * exclude_extensions, int hw_decode)
 {
     hb_title_t * title;
 
-    // Check if scanning is necessary.
-    if (h->title_set.path != NULL && !strcmp(h->title_set.path, path))
+    char *single_path = NULL;
+    int path_count = hb_list_count(paths);
+
+    if (path_count == 1)
     {
-        // Current title_set path matches requested path.
+        single_path = hb_list_item(paths, 0);
+    }
+    
+    // Check if scanning is necessary. Only works on Single Path.
+    if (single_path != NULL && h->title_set.path != NULL && !strcmp(h->title_set.path, single_path))
+    {
+        // Current title_set path matches requested single_path.
         // Check if the requested title has already been scanned.
         int ii;
         for (ii = 0; ii < hb_list_count(h->title_set.list_title); ii++)
@@ -432,10 +466,18 @@ void hb_scan( hb_handle_t * h, const char * path, int title_index,
     }
 #endif
 
-    hb_log( "hb_scan: path=%s, title_index=%d", path, title_index );
-    h->scan_thread = hb_scan_init( h, &h->scan_die, path, title_index,
+    char *path_info = single_path;
+    if (path_count > 1)
+    {
+        path_info = "(multiple)";
+    }
+
+    hb_log( "hb_scan: path=%s, title_index=%d", path_info, title_index );
+    h->scan_thread = hb_scan_init( h, &h->scan_die, paths, title_index,
                                    &h->title_set, preview_count,
-                                   store_previews, min_duration );
+                                   store_previews, min_duration,
+                                   crop_threshold_frames, crop_threshold_pixels,
+                                   exclude_extensions, hw_decode);
 }
 
 void hb_force_rescan( hb_handle_t * h )
@@ -885,6 +927,7 @@ hb_image_t * hb_get_preview3(hb_handle_t * h, int picture,
     init.time_base.den = 90000;
     init.job = job;
     init.pix_fmt = AV_PIX_FMT_YUV420P;
+    init.hw_pix_fmt = AV_PIX_FMT_NONE;
     init.color_range = AVCOL_RANGE_MPEG;
 
     init.color_prim = title->color_prim;
@@ -911,13 +954,12 @@ hb_image_t * hb_get_preview3(hb_handle_t * h, int picture,
             case HB_FILTER_COLORSPACE:
             case HB_FILTER_DECOMB:
             case HB_FILTER_DETELECINE:
-            case HB_FILTER_DEINTERLACE:
+            case HB_FILTER_YADIF:
             case HB_FILTER_GRAYSCALE:
                 break;
 
             case HB_FILTER_VFR:
             case HB_FILTER_RENDER_SUB:
-            case HB_FILTER_QSV:
             case HB_FILTER_NLMEANS:
             case HB_FILTER_CHROMA_SMOOTH:
             case HB_FILTER_LAPSHARP:
@@ -925,6 +967,7 @@ hb_image_t * hb_get_preview3(hb_handle_t * h, int picture,
             case HB_FILTER_DEBLOCK:
             case HB_FILTER_COMB_DETECT:
             case HB_FILTER_HQDN3D:
+            case HB_FILTER_BWDIF:
                 // Not implemented, N/A, or requires multiple frame input
                 hb_list_rem(list_filter, filter);
                 hb_filter_close(&filter);
@@ -1118,7 +1161,7 @@ fail:
  */
 int hb_detect_comb( hb_buffer_t * buf, int color_equal, int color_diff, int threshold, int prog_equal, int prog_diff, int prog_threshold )
 {
-    int j, k, n, off, cc_1, cc_2, cc[3];
+    int j, k, n, off, cc_1, cc_2, cc[3] = {0};
 	// int flag[3] ; // debugging flag
     uint16_t s1, s2, s3, s4;
     cc_1 = 0; cc_2 = 0;
@@ -1176,7 +1219,7 @@ int hb_detect_comb( hb_buffer_t * buf, int color_equal, int color_diff, int thre
     }
 
 
-    /* HandBrake is all yuv420, so weight the average percentage of all 3 planes accordingly.*/
+    /* HandBrake previews are all yuv420, so weight the average percentage of all 3 planes accordingly. */
     int average_cc = ( 2 * cc[0] + ( cc[1] / 2 ) + ( cc[2] / 2 ) ) / 3;
 
     /* Now see if that average percentage of combed pixels surpasses the threshold percentage given by the user.*/
@@ -1708,6 +1751,11 @@ void hb_add_filter2( hb_value_array_t * list, hb_dict_t * filter_dict )
 void hb_add_filter_dict( hb_job_t * job, hb_filter_object_t * filter,
                          const hb_dict_t * settings_in )
 {
+    if (filter == NULL)
+    {
+        hb_log("hb_add_filter_dict: filter is null");
+        return;
+    }
     hb_dict_t * settings;
 
     // Always set filter->settings to a valid hb_dict_t
@@ -1757,6 +1805,11 @@ void hb_add_filter_dict( hb_job_t * job, hb_filter_object_t * filter,
 void hb_add_filter( hb_job_t * job, hb_filter_object_t * filter,
                     const char * settings_in )
 {
+    if (filter == NULL)
+    {
+        hb_log("hb_add_filter: filter is null");
+        return;
+    }
     hb_dict_t * settings = hb_parse_filter_settings(settings_in);
     if (settings_in != NULL && settings == NULL)
     {
@@ -1963,7 +2016,7 @@ void hb_job_setup_passes(hb_handle_t * h, hb_job_t * job, hb_list_t * list_pass)
 {
     if (job->vquality > HB_INVALID_VIDEO_QUALITY)
     {
-        job->twopass = 0;
+        job->multipass = 0;
     }
     if (job->indepth_scan)
     {
@@ -1972,12 +2025,16 @@ void hb_job_setup_passes(hb_handle_t * h, hb_job_t * job, hb_list_t * list_pass)
         hb_add_internal(h, job, list_pass);
         job->indepth_scan = 0;
     }
-    if (job->twopass)
+    if (job->multipass)
     {
-        hb_deep_log(2, "Adding two-pass encode");
-        job->pass_id = HB_PASS_ENCODE_1ST;
-        hb_add_internal(h, job, list_pass);
-        job->pass_id = HB_PASS_ENCODE_2ND;
+        hb_deep_log(2, "Adding multi-pass encode");
+        int analysis_pass_count = hb_video_encoder_get_count_of_analysis_passes(job->vcodec);
+        for (int i = 0; i < analysis_pass_count; i++)
+        {
+            job->pass_id = HB_PASS_ENCODE_ANALYSIS;
+            hb_add_internal(h, job, list_pass);
+        }
+        job->pass_id = HB_PASS_ENCODE_FINAL;
         hb_add_internal(h, job, list_pass);
     }
     else
@@ -2177,10 +2234,6 @@ int hb_global_init()
         return -1;
     }
 
-#if HB_PROJECT_FEATURE_QSV
-    hb_param_configure_qsv();
-#endif
-
     /* libavcodec */
     hb_avcodec_init();
 
@@ -2210,6 +2263,7 @@ int hb_global_init()
 #if HB_PROJECT_FEATURE_X265
     hb_register(&hb_encx265);
 #endif
+    hb_register(&hb_encsvtav1);
 #if HB_PROJECT_FEATURE_QSV
     if (!disable_hardware)
     {
@@ -2436,6 +2490,7 @@ hb_interjob_t * hb_interjob_get( hb_handle_t * h )
     return h->interjob;
 }
 
-int is_hardware_disabled(void){
+int is_hardware_disabled(void)
+{
     return disable_hardware;
 }
