@@ -1,6 +1,6 @@
 /* json.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -225,6 +225,17 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
     if (title == NULL)
         return NULL;
 
+    int h_shift, v_shift;
+    int chroma_available = hb_get_chroma_sub_sample(title->pix_fmt, &h_shift, &v_shift);
+    char *chroma_subsampling = NULL;
+
+    if (chroma_available == 0)
+    {
+        int h_value = 4 >> h_shift;
+        int v_value = v_shift ? 0 : h_value;
+        chroma_subsampling = hb_strdup_printf("4:%d:%d", h_value, v_value);
+    }
+
     dict = json_pack_ex(&error, 0,
     "{"
         // Type, Path, Name, Index, Playlist, AngleCount
@@ -237,8 +248,8 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
         "s:[oooo],"
         // LooseCrop[Top, Bottom, Left, Right]}
         "s:[oooo],"
-        // Color {Format, Range, Primary, Transfer, Matrix, ChromaLocation}
-        "s:{s:o, s:o, s:o, s:o, s:o, s:o},"
+        // Color {Format, Range, Primary, Transfer, Matrix, ChromaLocation, ChromaSubsampling, BitDepth}
+        "s:{s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o},"
         // FrameRate {Num, Den}
         "s:{s:o, s:o},"
         // InterlaceDetected, VideoCodec
@@ -278,6 +289,8 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
         "Transfer",         hb_value_int(title->color_transfer),
         "Matrix",           hb_value_int(title->color_matrix),
         "ChromaLocation",   hb_value_int(title->chroma_location),
+        "ChromaSubsampling", hb_value_string(chroma_subsampling ? chroma_subsampling : "unknown"),
+        "BitDepth",          hb_value_int(hb_get_bit_depth(title->pix_fmt)),
     "FrameRate",
         "Num",              hb_value_int(title->vrate.num),
         "Den",              hb_value_int(title->vrate.den),
@@ -288,9 +301,83 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
     if (dict == NULL)
     {
         hb_error("hb_title_to_dict_internal, json pack failure: %s", error.text);
+        free(chroma_subsampling);
         return NULL;
     }
 
+    free(chroma_subsampling);
+
+    // Mastering Display Color Volume metadata
+    hb_dict_t *mastering_dict;
+    if (title->mastering.has_primaries || title->mastering.has_luminance)
+    {
+        mastering_dict = json_pack_ex(&error, 0,
+        "{"
+        // DisplayPrimaries[3][2]
+        "s:[[[ii],[ii]],[[ii],[ii]],[[ii],[ii]]],"
+        // WhitePoint[2],
+        "s:[[i,i],[i,i]],"
+        // MinLuminance, MaxLuminance, HasPrimaries, HasLuminance
+        "s:[i,i],s:[i,i],s:b,s:b"
+        "}",
+            "DisplayPrimaries", title->mastering.display_primaries[0][0].num,
+                                title->mastering.display_primaries[0][0].den,
+                                title->mastering.display_primaries[0][1].num,
+                                title->mastering.display_primaries[0][1].den,
+                                title->mastering.display_primaries[1][0].num,
+                                title->mastering.display_primaries[1][0].den,
+                                title->mastering.display_primaries[1][1].num,
+                                title->mastering.display_primaries[1][1].den,
+                                title->mastering.display_primaries[2][0].num,
+                                title->mastering.display_primaries[2][0].den,
+                                title->mastering.display_primaries[2][1].num,
+                                title->mastering.display_primaries[2][1].den,
+            "WhitePoint", title->mastering.white_point[0].num,
+                          title->mastering.white_point[0].den,
+                          title->mastering.white_point[1].num,
+                          title->mastering.white_point[1].den,
+            "MinLuminance", title->mastering.min_luminance.num,
+                            title->mastering.min_luminance.den,
+            "MaxLuminance", title->mastering.max_luminance.num,
+                            title->mastering.max_luminance.den,
+            "HasPrimaries", title->mastering.has_primaries,
+            "HasLuminance", title->mastering.has_luminance
+        );
+        hb_dict_set(dict, "MasteringDisplayColorVolume", mastering_dict);
+    }
+
+    // Content Light Level metadata
+    hb_dict_t *coll_dict;
+    if (title->coll.max_cll && title->coll.max_fall)
+    {
+        coll_dict = json_pack_ex(&error, 0, "{s:i, s:i}",
+            "MaxCLL",  title->coll.max_cll,
+            "MaxFALL", title->coll.max_fall);
+        hb_dict_set(dict, "ContentLightLevel", coll_dict);
+    }
+
+    // Dolby Vision Configuration Record
+    hb_dict_t *dovi_dict;
+    if (title->dovi.dv_profile)
+    {
+        dovi_dict = json_pack_ex(&error, 0, "{s:i, s:i, s:i, s:i, s:i, s:i, s:i, s:i}",
+            "DVVersionMajor",          title->dovi.dv_version_major,
+            "DVVersionMinor",          title->dovi.dv_version_minor,
+            "DVProfile",               title->dovi.dv_profile,
+            "DVLevel",                 title->dovi.dv_level,
+            "RPUPresentFlag",          title->dovi.rpu_present_flag,
+            "ELPresentFlag",           title->dovi.el_present_flag,
+            "BLPresentFlag",           title->dovi.bl_present_flag,
+            "BLSignalCompatibilityId", title->dovi.dv_bl_signal_compatibility_id);
+        hb_dict_set(dict, "DolbyVisionConfigurationRecord", dovi_dict);
+    }
+
+    // HDR10+ Flag
+    if (title->hdr_10_plus)
+    {
+        hb_dict_set(dict, "HDR10+", hb_value_int(title->hdr_10_plus));
+    }
+
     if (title->container_name != NULL)
     {
         hb_dict_set(dict, "Container", hb_value_string(title->container_name));
@@ -507,8 +594,8 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
     // Destination {Mux, InlineParameterSets, AlignAVStart,
     //              ChapterMarkers, ChapterList}
     "s:{s:o, s:o, s:o, s:o, s:[]},"
-    // Source {Path, Title, Angle}
-    "s:{s:o, s:o, s:o,},"
+    // Source {Path, Title, Angle, HWDecode}
+    "s:{s:o, s:o, s:o, s:o},"
     // PAR {Num, Den}
     "s:{s:o, s:o},"
     // Video {Encoder, HardwareDecode, QSV {Decode, AsyncDepth, AdapterIndex}}
@@ -533,6 +620,7 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
             "Path",             hb_value_string(job->title->path),
             "Title",            hb_value_int(job->title->index),
             "Angle",            hb_value_int(job->angle),
+            "HWDecode",         hb_value_int(job->hw_decode),
         "PAR",
             "Num",              hb_value_int(job->par.num),
             "Den",              hb_value_int(job->par.den),
@@ -568,13 +656,13 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
     {
         hb_dict_set(dest_dict, "File", hb_value_string(job->file));
     }
-    if (job->mux & HB_MUX_MASK_MP4)
+    if (job->mux)
     {
-        hb_dict_t *mp4_dict;
-        mp4_dict = json_pack_ex(&error, 0, "{s:o, s:o}",
-            "Mp4Optimize",      hb_value_bool(job->mp4_optimize),
+        hb_dict_t *options_dict;
+        options_dict = json_pack_ex(&error, 0, "{s:o, s:o}",
+            "Optimize",         hb_value_bool(job->optimize),
             "IpodAtom",         hb_value_bool(job->ipod_atom));
-        hb_dict_set(dest_dict, "Mp4Options", mp4_dict);
+        hb_dict_set(dest_dict, "Options", options_dict);
     }
     hb_dict_t *source_dict = hb_dict_get(dict, "Source");
     hb_dict_t *range_dict;
@@ -639,23 +727,23 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
                 hb_value_int(job->color_matrix));
     hb_dict_set(video_dict, "ChromaLocation",
                 hb_value_int(job->chroma_location));
-    if (job->color_prim_override != HB_COLR_PRI_UNDEF)
+    if (job->color_prim_override != HB_COLR_PRI_UNSET)
     {
         hb_dict_set(video_dict, "ColorPrimariesOverride",
                     hb_value_int(job->color_prim_override));
     }
-    if (job->color_transfer_override != HB_COLR_TRA_UNDEF)
+    if (job->color_transfer_override != HB_COLR_TRA_UNSET)
     {
         hb_dict_set(video_dict, "ColorTransferOverride",
                     hb_value_int(job->color_transfer_override));
     }
-    if (job->color_matrix_override != HB_COLR_MAT_UNDEF)
+    if (job->color_matrix_override != HB_COLR_MAT_UNSET)
     {
         hb_dict_set(video_dict, "ColorMatrixOverride",
                     hb_value_int(job->color_matrix_override));
     }
 
-    // Mastering metadata
+    // Mastering Display Color Volume metadata
     hb_dict_t *mastering_dict;
     if (job->mastering.has_primaries || job->mastering.has_luminance)
     {
@@ -691,7 +779,7 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
             "HasPrimaries", job->mastering.has_primaries,
             "HasLuminance", job->mastering.has_luminance
         );
-        hb_dict_set(video_dict, "Mastering", mastering_dict);
+        hb_dict_set(video_dict, "MasteringDisplayColorVolume", mastering_dict);
     }
 
     // Content Light Level metadata
@@ -704,6 +792,22 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
         hb_dict_set(video_dict, "ContentLightLevel", coll_dict);
     }
 
+    // Dolby Vision Configuration Record
+    hb_dict_t *dovi_dict;
+    if (job->dovi.dv_profile)
+    {
+        dovi_dict = json_pack_ex(&error, 0, "{s:i, s:i, s:i, s:i, s:i, s:i, s:i, s:i}",
+            "DVVersionMajor",          job->dovi.dv_version_major,
+            "DVVersionMinor",          job->dovi.dv_version_minor,
+            "DVProfile",               job->dovi.dv_profile,
+            "DVLevel",                 job->dovi.dv_level,
+            "RPUPresentFlag",          job->dovi.rpu_present_flag,
+            "ELPresentFlag",           job->dovi.el_present_flag,
+            "BLPresentFlag",           job->dovi.bl_present_flag,
+            "BLSignalCompatibilityId", job->dovi.dv_bl_signal_compatibility_id);
+        hb_dict_set(video_dict, "DolbyVisionConfigurationRecord", dovi_dict);
+    }
+
     if (job->vquality > HB_INVALID_VIDEO_QUALITY)
     {
         hb_dict_set(video_dict, "Quality", hb_value_double(job->vquality));
@@ -711,9 +815,9 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
     else
     {
         hb_dict_set(video_dict, "Bitrate", hb_value_int(job->vbitrate));
-        hb_dict_set(video_dict, "TwoPass", hb_value_bool(job->twopass));
+        hb_dict_set(video_dict, "MultiPass", hb_value_bool(job->multipass));
         hb_dict_set(video_dict, "Turbo",
-                            hb_value_bool(job->fastfirstpass));
+                            hb_value_bool(job->fastanalysispass));
     }
     if (job->encoder_preset != NULL)
     {
@@ -910,13 +1014,14 @@ void hb_json_job_scan( hb_handle_t * h, const char * json_job )
 
     dict = hb_value_json(json_job);
 
-    int title_index;
+    int title_index, hw_decode;
     const char *path = NULL;
 
-    result = json_unpack_ex(dict, &error, 0, "{s:{s:s, s:i}}",
+    result = json_unpack_ex(dict, &error, 0, "{s:{s:s, s:i, s?i}}",
                             "Source",
                                 "Path",     unpack_s(&path),
-                                "Title",    unpack_i(&title_index)
+                                "Title",    unpack_i(&title_index),
+                                "HWDecode", unpack_i(&hw_decode)
                            );
     if (result < 0)
     {
@@ -926,8 +1031,8 @@ void hb_json_job_scan( hb_handle_t * h, const char * json_job )
     }
 
     // If the job wants to use Hardware decode, it must also be
-    // enabled during scan.  So enable it here.
-    hb_scan(h, path, title_index, -1, 0, 0);
+    // enabled during scan.  So enable it here.                      
+    hb_scan(h, path, title_index, -1, 0, 0, 0, 0, NULL, hw_decode);
 
     // Wait for scan to complete
     hb_state_t state;
@@ -997,6 +1102,7 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
     hb_value_t       * mux = NULL, * vcodec = NULL;
     hb_dict_t        * mastering_dict = NULL;
     hb_dict_t        * coll_dict = NULL;
+    hb_dict_t        * dovi_dict = NULL;
     hb_value_t       * acodec_copy_mask = NULL, * acodec_fallback = NULL;
     const char       * destfile = NULL;
     const char       * range_type = NULL;
@@ -1016,18 +1122,19 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
     "s:i,"
     // Destination {File, Mux, InlineParameterSets, AlignAVStart,
     //              ChapterMarkers, ChapterList,
-    //              Mp4Options {Mp4Optimize, IpodAtom}}
+    //              Options {Optimize, IpodAtom}}
     "s:{s?s, s:o, s?b, s?b, s:b, s?o s?{s?b, s?b}},"
     // Source {Angle, Range {Type, Start, End, SeekPoints}}
     "s:{s?i, s?{s:s, s?I, s?I, s?I}},"
     // PAR {Num, Den}
     "s?{s:i, s:i},"
     // Video {Codec, Quality, Bitrate, Preset, Tune, Profile, Level, Options
-    //       TwoPass, Turbo,
+    //       MultiPass, Turbo,
     //       ColorInputFormat, ColorOutputFormat, ColorRange,
     //       ColorPrimaries, ColorTransfer, ColorMatrix, ChromaLocation,
-    //       Mastering,
+    //       MasteringDisplayColorVolume,
     //       ContentLightLevel,
+    //       DolbyVisionConfigurationRecord
     //       ColorPrimariesOverride, ColorTransferOverride, ColorMatrixOverride,
     //       HardwareDecode
     //       QSV {Decode, AsyncDepth, AdapterIndex}}
@@ -1037,6 +1144,7 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
     "   s?i, s?i, s?i, s?i,"
     "   s?o,"
     "   s?o,"
+    "   s?o,"
     "   s?i, s?i, s?i,"
     "   s?i,"
     "   s?{s?b, s?i, s?i}},"
@@ -1057,8 +1165,8 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
             "AlignAVStart",         unpack_b(&job->align_av_start),
             "ChapterMarkers",       unpack_b(&job->chapter_markers),
             "ChapterList",          unpack_o(&chapter_list),
-            "Mp4Options",
-                "Mp4Optimize",      unpack_b(&job->mp4_optimize),
+            "Options",
+                "Optimize",         unpack_b(&job->optimize),
                 "IpodAtom",         unpack_b(&job->ipod_atom),
         "Source",
             "Angle",                unpack_i(&job->angle),
@@ -1079,8 +1187,8 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
             "Profile",              unpack_s(&video_profile),
             "Level",                unpack_s(&video_level),
             "Options",              unpack_s(&video_options),
-            "TwoPass",              unpack_b(&job->twopass),
-            "Turbo",                unpack_b(&job->fastfirstpass),
+            "MultiPass",            unpack_b(&job->multipass),
+            "Turbo",                unpack_b(&job->fastanalysispass),
             "ColorInputFormat",     unpack_i(&job->input_pix_fmt),
             "ColorOutputFormat",    unpack_i(&job->output_pix_fmt),
             "ColorRange",           unpack_i(&job->color_range),
@@ -1088,8 +1196,9 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
             "ColorTransfer",        unpack_i(&job->color_transfer),
             "ColorMatrix",          unpack_i(&job->color_matrix),
             "ChromaLocation",       unpack_i(&job->chroma_location),
-            "Mastering",            unpack_o(&mastering_dict),
+            "MasteringDisplayColorVolume", unpack_o(&mastering_dict),
             "ContentLightLevel",    unpack_o(&coll_dict),
+            "DolbyVisionConfigurationRecord", unpack_o(&dovi_dict),
             "ColorPrimariesOverride", unpack_i(&job->color_prim_override),
             "ColorTransferOverride",  unpack_i(&job->color_transfer_override),
             "ColorMatrixOverride",    unpack_i(&job->color_matrix_override),
@@ -1273,6 +1382,26 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
         }
     }
 
+    if (dovi_dict != NULL)
+    {
+        result = json_unpack_ex(dovi_dict, &error, 0,
+        "{s:i, s:i, s:i, s:i, s:i, s:i, s:i, s:i}",
+            "DVVersionMajor",          unpack_u(&job->dovi.dv_version_major),
+            "DVVersionMinor",          unpack_u(&job->dovi.dv_version_minor),
+            "DVProfile",               unpack_u(&job->dovi.dv_profile),
+            "DVLevel",                 unpack_u(&job->dovi.dv_level),
+            "RPUPresentFlag",          unpack_u(&job->dovi.rpu_present_flag),
+            "ELPresentFlag",           unpack_u(&job->dovi.el_present_flag),
+            "BLPresentFlag",           unpack_u(&job->dovi.bl_present_flag),
+            "BLSignalCompatibilityId", unpack_u(&job->dovi.dv_bl_signal_compatibility_id)
+        );
+        if (result < 0)
+        {
+            hb_error("hb_dict_to_job: failed to parse dovi_dict: %s", error.text);
+            goto fail;
+        }
+    }
+
     // process chapter list
     if (chapter_list != NULL &&
         hb_value_type(chapter_list) == HB_VALUE_TYPE_ARRAY)
