@@ -1,6 +1,6 @@
 /* lang.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -206,7 +206,7 @@ static const iso639_lang_t languages[] =
 
 static const int lang_count = sizeof(languages) / sizeof(languages[0]);
 
-const int lang_lookup_index( const char * str )
+int lang_lookup_index( const char * str )
 {
     int             ii = 0;
     const iso639_lang_t * lang;
@@ -314,8 +314,10 @@ int lang_to_code(const iso639_lang_t *lang)
 {
     int code = 0;
 
-    if (lang)
+    if (lang && strlen(lang->iso639_1) >= 2)
+    {
         code = (lang->iso639_1[0] << 8) | lang->iso639_1[1];
+    }
 
     return code;
 }
