@@ -1,12 +1,13 @@
 /* muxavformat.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
 
+#include <time.h>
 #include <ogg/ogg.h>
 #include "libavcodec/bsf.h"
 #include "libavformat/avformat.h"
@@ -184,7 +185,8 @@ static int avformatInit( hb_mux_object_t * m )
             meta_mux = META_MUX_MP4;
 
             av_dict_set(&av_opts, "brand", "mp42", 0);
-            if (job->mp4_optimize)
+            av_dict_set(&av_opts, "strict", "experimental", 0);
+            if (job->optimize)
                 av_dict_set(&av_opts, "movflags", "faststart+disable_chpl+write_colr", 0);
             else
                 av_dict_set(&av_opts, "movflags", "+disable_chpl+write_colr", 0);
@@ -226,15 +228,17 @@ static int avformatInit( hb_mux_object_t * m )
 
     ret = avio_open2(&m->oc->pb, job->file, AVIO_FLAG_WRITE,
                      &m->oc->interrupt_callback, NULL);
-    if( ret < 0 )
+    if (ret < 0)
     {
-      if( ret == -2 ) 
-      {
-        hb_error( "avio_open2 failed, errno -2: Could not write to indicated output file. Please check destination path and file permissions" );
-      }
-      else
-        hb_error( "avio_open2 failed, errno %d", ret);
-      goto error;
+        if (ret == -2)
+        {
+            hb_error("avio_open2 failed, errno -2: Could not write to indicated output file. Please check destination path and file permissions");
+        }
+        else
+        {
+            hb_error("avio_open2 failed, errno %d", ret);
+        }
+        goto error;
     }
 
     /* Video track */
@@ -380,8 +384,11 @@ static int avformatInit( hb_mux_object_t * m )
             priv_size                  = 0;
             break;
 
-        case HB_VCODEC_FFMPEG_SVT_AV1:
-        case HB_VCODEC_FFMPEG_SVT_AV1_10BIT:
+        case HB_VCODEC_SVT_AV1:
+        case HB_VCODEC_SVT_AV1_10BIT:
+        case HB_VCODEC_FFMPEG_NVENC_AV1:
+        case HB_VCODEC_FFMPEG_NVENC_AV1_10BIT:
+        case HB_VCODEC_FFMPEG_VCE_AV1:
             track->st->codecpar->codec_id = AV_CODEC_ID_AV1;
 
             if (job->config.extradata.length > 0)
@@ -433,7 +440,6 @@ static int avformatInit( hb_mux_object_t * m )
                 }
             }
         } break;
-
         case HB_VCODEC_THEORA:
         {
             track->st->codecpar->codec_id = AV_CODEC_ID_THEORA;
@@ -556,10 +562,11 @@ static int avformatInit( hb_mux_object_t * m )
             uint8_t *mastering_data = av_malloc(sizeof(AVMasteringDisplayMetadata));
             memcpy(mastering_data, &mastering, sizeof(AVMasteringDisplayMetadata));
 
-            av_stream_add_side_data(track->st,
+            av_packet_side_data_add(&track->st->codecpar->coded_side_data,
+                                    &track->st->codecpar->nb_coded_side_data,
                                     AV_PKT_DATA_MASTERING_DISPLAY_METADATA,
                                     mastering_data,
-                                    sizeof(AVMasteringDisplayMetadata));
+                                    sizeof(AVMasteringDisplayMetadata), 0);
         }
 
         if (job->coll.max_cll && job->coll.max_fall)
@@ -571,13 +578,51 @@ static int avformatInit( hb_mux_object_t * m )
             uint8_t *coll_data = av_malloc(sizeof(AVContentLightMetadata));
             memcpy(coll_data, &coll, sizeof(AVContentLightMetadata));
 
-            av_stream_add_side_data(track->st,
+            av_packet_side_data_add(&track->st->codecpar->coded_side_data,
+                                    &track->st->codecpar->nb_coded_side_data,
                                     AV_PKT_DATA_CONTENT_LIGHT_LEVEL,
                                     coll_data,
-                                    sizeof(AVContentLightMetadata));
+                                    sizeof(AVContentLightMetadata), 0);
         }
     }
 
+    if (job->ambient.ambient_illuminance.num && job->ambient.ambient_illuminance.den)
+    {
+        AVAmbientViewingEnvironment ambient = hb_ambient_hb_to_ff(job->ambient);
+
+        uint8_t *ambient_data = av_malloc(sizeof(AVAmbientViewingEnvironment));
+        memcpy(ambient_data, &ambient, sizeof(AVAmbientViewingEnvironment));
+
+        av_packet_side_data_add(&track->st->codecpar->coded_side_data,
+                                &track->st->codecpar->nb_coded_side_data,
+                                AV_PKT_DATA_AMBIENT_VIEWING_ENVIRONMENT,
+                                ambient_data,
+                                sizeof(AVAmbientViewingEnvironment), 0);
+    }
+
+    if (job->passthru_dynamic_hdr_metadata & DOVI)
+    {
+        if (job->dovi.dv_profile == 5 && job->mux == HB_MUX_AV_MP4)
+        {
+            if (track->st->codecpar->codec_id == AV_CODEC_ID_HEVC)
+            {
+                track->st->codecpar->codec_tag = MKTAG('d','v','h','1');
+            }
+        }
+        AVDOVIDecoderConfigurationRecord dovi = hb_dovi_hb_to_ff(job->dovi);
+
+        uint8_t *dovi_data = av_malloc(sizeof(AVDOVIDecoderConfigurationRecord));
+        memcpy(dovi_data, &dovi, sizeof(AVDOVIDecoderConfigurationRecord));
+
+        av_packet_side_data_add(&track->st->codecpar->coded_side_data,
+                                &track->st->codecpar->nb_coded_side_data,
+                                AV_PKT_DATA_DOVI_CONF,
+                                dovi_data,
+                                sizeof(AVDOVIDecoderConfigurationRecord), 0);
+
+        m->oc->strict_std_compliance = FF_COMPLIANCE_UNOFFICIAL;
+    }
+
     hb_rational_t vrate = job->vrate;
 
     // If the vrate is the internal clock rate, there's a good chance
@@ -880,16 +925,19 @@ static int avformatInit( hb_mux_object_t * m )
                          codec == HB_ACODEC_FDK_HAAC))
                     {
                         hb_mux_data_t * fallback_track;
-                        int           * sd;
+                        AVPacketSideData *sd;
 
                         track = audio->priv.mux_data;
                         fallback_track = fallback->priv.mux_data;
-                        sd = (int*)av_stream_new_side_data(track->st,
+                        sd = av_packet_side_data_new(&track->st->codecpar->coded_side_data,
+                                                     &track->st->codecpar->nb_coded_side_data,
                                                      AV_PKT_DATA_FALLBACK_TRACK,
-                                                     sizeof(int));
+                                                     sizeof(int), 0);
+
                         if (sd != NULL)
                         {
-                            *sd = fallback_track->st->index;
+                            int *data = (int *)sd->data;
+                            *data = fallback_track->st->index;
                         }
                     }
                 }
@@ -1215,7 +1263,6 @@ static int avformatInit( hb_mux_object_t * m )
     ret = avformat_write_header(m->oc, &av_opts);
     if( ret < 0 )
     {
-        av_dict_free( &av_opts );
         hb_error( "muxavformat: avformat_write_header failed!");
         goto error;
     }
@@ -1230,6 +1277,7 @@ static int avformatInit( hb_mux_object_t * m )
     return 0;
 
 error:
+    av_dict_free(&av_opts);
     free(job->mux_data);
     job->mux_data = NULL;
     avformat_free_context(m->oc);
@@ -1395,12 +1443,10 @@ static int avformatMux(hb_mux_object_t *m, hb_mux_data_t *track, hb_buffer_t *bu
         {
             m->pkt->flags |= AV_PKT_FLAG_KEY;
         }
-#ifdef AV_PKT_FLAG_DISPOSABLE
         if (!(buf->s.flags & HB_FLAG_FRAMETYPE_REF))
         {
             m->pkt->flags |= AV_PKT_FLAG_DISPOSABLE;
         }
-#endif
     }
     else if (buf->s.frametype & HB_FRAME_MASK_KEY)
     {
