@@ -1,6 +1,6 @@
 /* param.c
  *
- * Copyright (c) 2003-2022 HandBrake Team
+ * Copyright (c) 2003-2023 HandBrake Team
  * This file is part of the HandBrake source code
  * Homepage: <http://handbrake.fr/>.
  * It may be used under the terms of the GNU General Public License v2.
@@ -13,9 +13,6 @@
 #include "handbrake/param.h"
 #include "handbrake/common.h"
 #include "handbrake/colormap.h"
-#if HB_PROJECT_FEATURE_QSV
-#include "handbrake/qsv_common.h"
-#endif
 #include <regex.h>
 
 static hb_filter_param_t nlmeans_presets[] =
@@ -217,14 +214,10 @@ static hb_filter_param_t yadif_presets[] =
     { 3, "Default",            "default",      "mode=3"         },
     { 2, "Skip Spatial Check", "skip-spatial", "mode=1"         },
     { 5, "Bob",                "bob",          "mode=7"         },
-#if HB_PROJECT_FEATURE_QSV
-    { 6, "QSV",                "qsv",          "mode=11"        },
-#endif
     { 0,  NULL,                NULL,           NULL             },
     { 2, "Fast",               "fast",         "mode=1"         },
     { 3, "Slow",               "slow",         "mode=1"         },
     { 4, "Slower",             "slower",       "mode=3"         },
-    { 7, "QSV",                "qsv",          "mode=3"         }
 };
 
 static hb_filter_param_t bwdif_presets[] =
@@ -290,16 +283,6 @@ static filter_param_map_t param_map[] =
     { HB_FILTER_INVALID,     NULL,                NULL,     0, 0, },
 };
 
-void hb_param_configure_qsv(void)
-{
-#if HB_PROJECT_FEATURE_QSV
-    if (!hb_qsv_available())
-    {
-        memset(&yadif_presets[4], 0, sizeof(hb_filter_param_t));
-    }
-#endif
-}
-
 /* NL-means presets and tunes
  *
  * Presets adjust strength:
@@ -1264,7 +1247,6 @@ hb_generate_filter_settings(int filter_id, const char *preset, const char *tune,
         case HB_FILTER_VFR:
         case HB_FILTER_RENDER_SUB:
         case HB_FILTER_GRAYSCALE:
-        case HB_FILTER_QSV:
             settings = hb_parse_filter_settings(custom);
             break;
         case HB_FILTER_NLMEANS:
