@@ -1,6 +1,6 @@
 /* encvt.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -13,6 +13,9 @@
 #include "libavutil/avutil.h"
 
 #include "handbrake/handbrake.h"
+#include "handbrake/hdr10plus.h"
+#include "handbrake/hbffmpeg.h"
+#include "cv_utils.h"
 
 int  encvt_init(hb_work_object_t *, hb_job_t *);
 int  encvt_work(hb_work_object_t *, hb_buffer_t **, hb_buffer_t **);
@@ -76,6 +79,7 @@ struct hb_work_private_s
         CFBooleanRef allowFrameReordering;
         CFBooleanRef allowTemporalCompression;
         CFBooleanRef prioritizeEncodingSpeedOverQuality;
+        CFBooleanRef preserveDynamicHDRMetadata;
         struct
         {
             int maxrate;
@@ -90,6 +94,7 @@ struct hb_work_private_s
             int chromaLocation;
             CFDataRef masteringDisplay;
             CFDataRef contentLightLevel;
+            CFDataRef ambientViewingEnviroment;
         }
         color;
         SInt32 width;
@@ -126,6 +131,7 @@ void hb_vt_param_default(struct hb_vt_param *param)
     param->allowFrameReordering     = kCFBooleanTrue;
     param->allowTemporalCompression = kCFBooleanTrue;
     param->prioritizeEncodingSpeedOverQuality = kCFBooleanFalse;
+    param->preserveDynamicHDRMetadata         = kCFBooleanFalse;
     param->fieldDetail              = HB_VT_FIELDORDER_PROGRESSIVE;
 }
 
@@ -210,133 +216,42 @@ static void compute_dts_offset(hb_work_private_t *pv, hb_buffer_t *buf)
     }
 }
 
-static CFStringRef hb_vt_colr_pri_xlat(int color_prim)
+static void hb_vt_add_dynamic_hdr_metadata(CVPixelBufferRef pxbuffer, hb_job_t *job, hb_buffer_t *buf)
 {
-    switch (color_prim)
+    for (int i = 0; i < buf->nb_side_data; i++)
     {
-        case HB_COLR_PRI_BT2020:
-            return kCMFormatDescriptionColorPrimaries_ITU_R_2020;
-        case HB_COLR_PRI_BT709:
-            return kCMFormatDescriptionColorPrimaries_ITU_R_709_2;
-        case HB_COLR_PRI_EBUTECH:
-            return kCMFormatDescriptionColorPrimaries_EBU_3213;
-        case HB_COLR_PRI_SMPTEC:
-            return kCMFormatDescriptionColorPrimaries_SMPTE_C;
-        default:
-            return NULL;
-    }
-}
+        const AVFrameSideData *side_data = buf->side_data[i];
+        if (job->passthru_dynamic_hdr_metadata & HDR_10_PLUS &&
+            side_data->type == AV_FRAME_DATA_DYNAMIC_HDR_PLUS)
+        {
+#ifdef MAC_OS_VERSION_13_0
+            if (__builtin_available(macOS 13, *))
+            {
+                uint8_t *payload = NULL;
+                uint32_t playload_size = 0;
 
-static CFStringRef hb_vt_colr_tra_xlat(int color_transfer)
-{
-    switch (color_transfer)
-    {
-        case HB_COLR_TRA_BT709:
-            return kCMFormatDescriptionTransferFunction_ITU_R_709_2;
-        case HB_COLR_TRA_SMPTE240M:
-            return kCMFormatDescriptionTransferFunction_SMPTE_240M_1995;
-        case HB_COLR_TRA_SMPTEST2084:
-            return kCVImageBufferTransferFunction_SMPTE_ST_2084_PQ;
-        case HB_COLR_TRA_LINEAR:
-            if (__builtin_available(macOS 10.14, *)) { return kCVImageBufferTransferFunction_Linear; }
-        case HB_COLR_TRA_IEC61966_2_1:
-            return kCVImageBufferTransferFunction_sRGB;
-        case HB_COLR_TRA_ARIB_STD_B67:
-            return kCVImageBufferTransferFunction_ITU_R_2100_HLG;
-        case HB_COLR_TRA_GAMMA22:
-            return kCVImageBufferTransferFunction_UseGamma;
-        case HB_COLR_TRA_GAMMA28:
-            return kCVImageBufferTransferFunction_UseGamma;
-        case HB_COLR_TRA_BT2020_10:
-        case HB_COLR_TRA_BT2020_12:
-            return kCVImageBufferTransferFunction_ITU_R_2020;
-        default:
-            return NULL;
-    }
-}
+                hb_dynamic_hdr10_plus_to_itu_t_t35((AVDynamicHDRPlus *)side_data->data, &payload, &playload_size);
+                if (!playload_size)
+                {
+                    continue;
+                }
 
-static CFNumberRef hb_vt_colr_gamma_xlat(int color_transfer)
-{
-    Float32 gamma = 0;
-    switch (color_transfer)
-    {
-        case HB_COLR_TRA_GAMMA22:
-            gamma = 2.2;
-        case HB_COLR_TRA_GAMMA28:
-            gamma = 2.8;
+                CFDataRef data = CFDataCreate(kCFAllocatorDefault, payload, playload_size);
+                if (data)
+                {
+                    CVBufferSetAttachment(pxbuffer, kCMSampleAttachmentKey_HDR10PlusPerFrameData, data, kCVAttachmentMode_ShouldPropagate);
+                    CFRelease(data);
+                }
+            }
+#endif
+        }
+        if (job->passthru_dynamic_hdr_metadata & DOVI &&
+            side_data->type == AV_FRAME_DATA_DOVI_RPU_BUFFER)
+        {
+           // Not supported yet
+        }
     }
 
-    return gamma > 0 ? CFNumberCreate(NULL, kCFNumberFloat32Type, &gamma) : NULL;
-}
-
-static CFStringRef hb_vt_colr_mat_xlat(int color_matrix)
-{
-    switch (color_matrix)
-    {
-        case HB_COLR_MAT_BT2020_NCL:
-            return kCMFormatDescriptionYCbCrMatrix_ITU_R_2020;
-        case HB_COLR_MAT_BT709:
-            return kCMFormatDescriptionYCbCrMatrix_ITU_R_709_2;
-        case HB_COLR_MAT_SMPTE170M:
-            return kCMFormatDescriptionYCbCrMatrix_ITU_R_601_4;
-        case HB_COLR_MAT_SMPTE240M:
-            return kCMFormatDescriptionYCbCrMatrix_SMPTE_240M_1995;
-        default:
-            return NULL;
-    }
-}
-
-static CFStringRef hb_vt_chroma_loc_xlat(int chroma_location)
-{
-    switch (chroma_location)
-    {
-        case AVCHROMA_LOC_LEFT:
-            return kCVImageBufferChromaLocation_Left;
-        case AVCHROMA_LOC_CENTER:
-            return kCVImageBufferChromaLocation_Center;
-        case AVCHROMA_LOC_TOPLEFT:
-            return kCVImageBufferChromaLocation_TopLeft;
-        case AVCHROMA_LOC_TOP:
-            return kCVImageBufferChromaLocation_Top;
-        case AVCHROMA_LOC_BOTTOMLEFT:
-            return kCVImageBufferChromaLocation_BottomLeft;
-        case AVCHROMA_LOC_BOTTOM:
-            return kCVImageBufferChromaLocation_Bottom;
-        default:
-            return NULL;
-    }
-}
-
-static void hb_vt_add_color_tag(CVPixelBufferRef pxbuffer, hb_job_t *job)
-{
-    CFStringRef prim       = hb_vt_colr_pri_xlat(job->color_prim);
-    CFStringRef transfer   = hb_vt_colr_tra_xlat(job->color_transfer);
-    CFNumberRef gamma      = hb_vt_colr_gamma_xlat(job->color_transfer);
-    CFStringRef matrix     = hb_vt_colr_mat_xlat(job->color_matrix);
-    CFStringRef chroma_loc = hb_vt_chroma_loc_xlat(job->chroma_location);
-
-    if (prim)
-    {
-        CVBufferSetAttachment(pxbuffer, kCVImageBufferColorPrimariesKey, prim, kCVAttachmentMode_ShouldPropagate);
-    }
-    if (transfer)
-    {
-        CVBufferSetAttachment(pxbuffer, kCVImageBufferTransferFunctionKey, transfer, kCVAttachmentMode_ShouldPropagate);
-    }
-    if (gamma)
-    {
-        CVBufferSetAttachment(pxbuffer, kCVImageBufferGammaLevelKey, gamma, kCVAttachmentMode_ShouldPropagate);
-        CFRelease(gamma);
-    }
-    if (matrix)
-    {
-        CVBufferSetAttachment(pxbuffer, kCVImageBufferYCbCrMatrixKey, matrix, kCVAttachmentMode_ShouldPropagate);
-    }
-    if (chroma_loc)
-    {
-        CVBufferSetAttachment(pxbuffer, kCVImageBufferChromaLocationTopFieldKey, chroma_loc, kCVAttachmentMode_ShouldPropagate);
-        CVBufferSetAttachment(pxbuffer, kCVImageBufferChromaLocationBottomFieldKey, chroma_loc, kCVAttachmentMode_ShouldPropagate);
-    }
 }
 
 static inline int64_t rescale(hb_rational_t q, int b)
@@ -393,62 +308,19 @@ static CFDataRef hb_vt_content_light_level_xlat(hb_content_light_metadata_t coll
     return data;
 }
 
-static OSType hb_vt_get_cv_pixel_format(hb_job_t* job)
+static CFDataRef hb_vt_ambient_viewing_enviroment_xlat(hb_ambient_viewing_environment_metadata_t ambient)
 {
-    if (job->output_pix_fmt == AV_PIX_FMT_NV12)
-    {
-        return job->color_range == AVCOL_RANGE_JPEG ?
-                                        kCVPixelFormatType_420YpCbCr8BiPlanarFullRange :
-                                        kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange;
-    }
-    else if (job->output_pix_fmt == AV_PIX_FMT_YUV420P)
-    {
-        return job->color_range == AVCOL_RANGE_JPEG ?
-                                        kCVPixelFormatType_420YpCbCr8PlanarFullRange :
-                                        kCVPixelFormatType_420YpCbCr8Planar;
-    }
-    else if (job->output_pix_fmt == AV_PIX_FMT_BGRA)
-    {
-        return kCVPixelFormatType_32BGRA;
-    }
-    else if (job->output_pix_fmt == AV_PIX_FMT_P010LE)
-    {
-        return job->color_range == AVCOL_RANGE_JPEG ?
-                                        kCVPixelFormatType_420YpCbCr10BiPlanarFullRange :
-                                        kCVPixelFormatType_420YpCbCr10BiPlanarVideoRange;
-    }
-    else if (job->output_pix_fmt == AV_PIX_FMT_NV16)
-    {
-        return job->color_range == AVCOL_RANGE_JPEG ?
-                                        kCVPixelFormatType_422YpCbCr8BiPlanarFullRange :
-                                        kCVPixelFormatType_422YpCbCr8BiPlanarVideoRange;
-    }
-    else if (job->output_pix_fmt == AV_PIX_FMT_P210)
-    {
-        return job->color_range == AVCOL_RANGE_JPEG ?
-                                        kCVPixelFormatType_422YpCbCr10BiPlanarFullRange :
-                                        kCVPixelFormatType_422YpCbCr10BiPlanarVideoRange;
-    }
-    else if (job->output_pix_fmt == AV_PIX_FMT_NV24)
-    {
-        return job->color_range == AVCOL_RANGE_JPEG ?
-                                        kCVPixelFormatType_444YpCbCr8BiPlanarFullRange :
-                                        kCVPixelFormatType_444YpCbCr8BiPlanarVideoRange;
-    }
-    else if (job->output_pix_fmt == AV_PIX_FMT_P410)
-    {
-        return job->color_range == AVCOL_RANGE_JPEG ?
-                                        kCVPixelFormatType_444YpCbCr10BiPlanarFullRange :
-                                        kCVPixelFormatType_444YpCbCr10BiPlanarVideoRange;
-    }
-    else if (job->output_pix_fmt == AV_PIX_FMT_P416)
-    {
-        return kCVPixelFormatType_444YpCbCr16BiPlanarVideoRange;
-    }
-    else
-    {
-        return 0;
-    }
+    CFMutableDataRef data = CFDataCreateMutable(kCFAllocatorDefault, 8);
+
+    uint32_t ambient_illuminance = CFSwapInt32HostToBig(rescale(ambient.ambient_illuminance, 10000));
+    uint16_t ambient_light_x =  CFSwapInt16HostToBig(rescale(ambient.ambient_light_x, 50000));
+    uint16_t ambient_light_y =  CFSwapInt16HostToBig(rescale(ambient.ambient_light_y, 50000));
+
+    CFDataAppendBytes(data, (UInt8 *)&ambient_illuminance, 4);
+    CFDataAppendBytes(data, (UInt8 *)&ambient_light_x, 2);
+    CFDataAppendBytes(data, (UInt8 *)&ambient_light_y, 2);
+
+    return data;
 }
 
 static int hb_vt_settings_xlat(hb_work_private_t *pv, hb_job_t *job)
@@ -457,13 +329,13 @@ static int hb_vt_settings_xlat(hb_work_private_t *pv, hb_job_t *job)
     hb_vt_param_default(&pv->settings);
 
     pv->settings.codec       = job->vcodec == HB_VCODEC_VT_H264 ? kCMVideoCodecType_H264 : kCMVideoCodecType_HEVC;
-    pv->settings.pixelFormat = hb_vt_get_cv_pixel_format(job);
+    pv->settings.pixelFormat = hb_cv_get_pixel_format(job->output_pix_fmt, job->color_range);
     pv->settings.timescale = 90000;
 
     // set the preset
     if (job->encoder_preset != NULL && *job->encoder_preset != '\0')
     {
-        if (!strcasecmp(job->encoder_profile, "speed"))
+        if (!strcasecmp(job->encoder_preset, "speed"))
         {
             pv->settings.prioritizeEncodingSpeedOverQuality = kCFBooleanTrue;
         }
@@ -619,6 +491,16 @@ static int hb_vt_settings_xlat(hb_work_private_t *pv, hb_job_t *job)
         }
     }
 
+    if (job->ambient.ambient_illuminance.num && job->ambient.ambient_illuminance.den)
+    {
+        pv->settings.color.ambientViewingEnviroment = hb_vt_ambient_viewing_enviroment_xlat(job->ambient);
+    }
+
+    if (job->passthru_dynamic_hdr_metadata & HDR_10_PLUS)
+    {
+        pv->settings.preserveDynamicHDRMetadata = kCFBooleanTrue;
+    }
+
     return 0;
 }
 
@@ -723,46 +605,61 @@ static int hb_vt_parse_options(hb_work_private_t *pv, hb_job_t *job)
     return 0;
 }
 
-void pixelBufferReleasePlanarBytesCallback(
-                                           void *releaseRefCon,
-                                           const void *dataPtr,
-                                           size_t dataSize,
-                                           size_t numberOfPlanes,
-                                           const void *planeAddresses[])
-{
-    hb_buffer_t *buf = (hb_buffer_t *)releaseRefCon;
-    hb_buffer_close(&buf);
-}
-
 static OSStatus wrap_buf(hb_work_private_t *pv, hb_buffer_t *buf, CVPixelBufferRef *pix_buf)
 {
-    OSStatus err;
-    int numberOfPlanes = pv->settings.pixelFormat == kCVPixelFormatType_420YpCbCr8Planar ||
-                         pv->settings.pixelFormat == kCVPixelFormatType_420YpCbCr8PlanarFullRange ? 3 : 2;
+    OSStatus err = 0;
 
-    void *planeBaseAddress[3] = {buf->plane[0].data, buf->plane[1].data, buf->plane[2].data};
-    size_t planeWidth[3] = {buf->plane[0].width, buf->plane[1].width, buf->plane[2].width};
-    size_t planeHeight[3] = {buf->plane[0].height, buf->plane[1].height, buf->plane[2].height};
-    size_t planeBytesPerRow[3] = {buf->plane[0].stride, buf->plane[1].stride, buf->plane[2].stride};
+    if (pv->job->hw_pix_fmt == AV_PIX_FMT_VIDEOTOOLBOX)
+    {
+        if (buf->storage_type == AVFRAME)
+        {
+            *pix_buf = (CVPixelBufferRef)((AVFrame *)buf->storage)->data[3];
+            CVPixelBufferRetain(*pix_buf);
+        }
+        else if (buf->storage_type == COREMEDIA)
+        {
+            *pix_buf = (CVPixelBufferRef)buf->storage;
+            CVPixelBufferRetain(*pix_buf);
+        }
+        else
+        {
+            err = 1;
+        }
+    }
+    else
+    {
+        int numberOfPlanes = pv->settings.pixelFormat == kCVPixelFormatType_420YpCbCr8Planar ||
+        pv->settings.pixelFormat == kCVPixelFormatType_420YpCbCr8PlanarFullRange ? 3 : 2;
 
-    err = CVPixelBufferCreateWithPlanarBytes(
-                                             kCFAllocatorDefault,
-                                             buf->f.width,
-                                             buf->f.height,
-                                             pv->settings.pixelFormat,
-                                             buf->data,
-                                             0,
-                                             numberOfPlanes,
-                                             planeBaseAddress,
-                                             planeWidth,
-                                             planeHeight,
-                                             planeBytesPerRow,
-                                             &pixelBufferReleasePlanarBytesCallback,
-                                             buf,
-                                             NULL,
-                                             pix_buf);
+        void *planeBaseAddress[3] = {buf->plane[0].data, buf->plane[1].data, buf->plane[2].data};
+        size_t planeWidth[3] = {buf->plane[0].width, buf->plane[1].width, buf->plane[2].width};
+        size_t planeHeight[3] = {buf->plane[0].height, buf->plane[1].height, buf->plane[2].height};
+        size_t planeBytesPerRow[3] = {buf->plane[0].stride, buf->plane[1].stride, buf->plane[2].stride};
 
-    hb_vt_add_color_tag(*pix_buf, pv->job);
+        err = CVPixelBufferCreateWithPlanarBytes(
+                                                 kCFAllocatorDefault,
+                                                 buf->f.width,
+                                                 buf->f.height,
+                                                 pv->settings.pixelFormat,
+                                                 buf->data,
+                                                 0,
+                                                 numberOfPlanes,
+                                                 planeBaseAddress,
+                                                 planeWidth,
+                                                 planeHeight,
+                                                 planeBytesPerRow,
+                                                 NULL,
+                                                 buf,
+                                                 NULL,
+                                                 pix_buf);
+    }
+
+    if (*pix_buf)
+    {
+        hb_cv_add_color_tag(*pix_buf, pv->job->color_prim, pv->job->color_transfer,
+                            pv->job->color_matrix, pv->job->chroma_location);
+        hb_vt_add_dynamic_hdr_metadata(*pix_buf, pv->job, buf);
+    }
 
     return err;
 }
@@ -778,15 +675,15 @@ void hb_vt_compression_output_callback(
 
     if (sourceFrameRefCon)
     {
-        CVPixelBufferRef pixelbuffer = sourceFrameRefCon;
-        CVPixelBufferRelease(pixelbuffer);
+        hb_buffer_t *buf = (hb_buffer_t *)sourceFrameRefCon;
+        hb_buffer_close(&buf);
     }
 
     if (status != noErr)
     {
         hb_log("VTCompressionSession: hb_vt_compression_output_callback called error");
     }
-    else
+    else if (sampleBuffer)
     {
         CFRetain(sampleBuffer);
         CMSimpleQueueRef queue = outputCallbackRefCon;
@@ -796,6 +693,10 @@ void hb_vt_compression_output_callback(
             hb_log("VTCompressionSession: hb_vt_compression_output_callback queue full");
         }
     }
+    else
+    {
+        hb_log("VTCompressionSession: hb_vt_compression_output_callback sample buffer is NULL");
+    }
 }
 
 static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_private_t *pv, int cookieOnly)
@@ -1004,42 +905,48 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
 
     err = VTSessionSetProperty(pv->session,
                                kVTCompressionPropertyKey_ColorPrimaries,
-                               hb_vt_colr_pri_xlat(pv->settings.color.prim));
+                               hb_cv_colr_pri_xlat(pv->settings.color.prim));
     if (err != noErr)
     {
         hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_ColorPrimaries failed");
     }
     err = VTSessionSetProperty(pv->session,
                                kVTCompressionPropertyKey_TransferFunction,
-                               hb_vt_colr_tra_xlat(pv->settings.color.transfer));
+                               hb_cv_colr_tra_xlat(pv->settings.color.transfer));
     if (err != noErr)
     {
         hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_TransferFunction failed");
     }
+    CFNumberRef gamma = hb_cv_colr_gamma_xlat(pv->settings.color.transfer);
     err = VTSessionSetProperty(pv->session,
                                CFSTR("GammaLevel"),
-                               hb_vt_colr_gamma_xlat(pv->settings.color.transfer));
+                               gamma);
+    if (gamma)
+    {
+        CFRelease(gamma);
+    }
+
     if (err != noErr)
     {
         hb_log("VTSessionSetProperty: GammaLevel failed");
     }
     err = VTSessionSetProperty(pv->session,
                                kVTCompressionPropertyKey_YCbCrMatrix,
-                               hb_vt_colr_mat_xlat(pv->settings.color.matrix));
+                               hb_cv_colr_mat_xlat(pv->settings.color.matrix));
     if (err != noErr)
     {
         hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_YCbCrMatrix failed");
     }
     err = VTSessionSetProperty(pv->session,
                                CFSTR("ChromaLocationTopField"),
-                               hb_vt_chroma_loc_xlat(pv->settings.color.chromaLocation));
+                               hb_cv_chroma_loc_xlat(pv->settings.color.chromaLocation));
     if (err != noErr)
     {
         hb_log("VTSessionSetProperty: ChromaLocationTopField failed");
     }
     err = VTSessionSetProperty(pv->session,
                                    CFSTR("ChromaLocationBottomField"),
-                                   hb_vt_chroma_loc_xlat(pv->settings.color.chromaLocation));
+                                   hb_cv_chroma_loc_xlat(pv->settings.color.chromaLocation));
     if (err != noErr)
     {
         hb_log("VTSessionSetProperty: ChromaLocationBottomField failed");
@@ -1057,7 +964,7 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
         }
     }
 
-    if (CFDictionaryContainsKey(supportedProps, kVTCompressionPropertyKey_MasteringDisplayColorVolume) &&
+    if (CFDictionaryContainsKey(supportedProps, kVTCompressionPropertyKey_ContentLightLevelInfo) &&
         pv->settings.color.contentLightLevel != NULL)
     {
         err = VTSessionSetProperty(pv->session,
@@ -1069,6 +976,18 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
         }
     }
 
+    if (CFDictionaryContainsKey(supportedProps, CFSTR("AmbientViewingEnvironment")) &&
+        pv->settings.color.ambientViewingEnviroment != NULL)
+    {
+        err = VTSessionSetProperty(pv->session,
+                                   CFSTR("AmbientViewingEnvironment"),
+                                   pv->settings.color.ambientViewingEnviroment);
+        if (err != noErr)
+        {
+            hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_AmbientViewingEnvironment failed");
+        }
+    }
+
     if (__builtin_available(macOS 11.0, *))
     {
         if (CFDictionaryContainsKey(supportedProps, kVTCompressionPropertyKey_HDRMetadataInsertionMode) &&
@@ -1084,6 +1003,20 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
         }
     }
 
+    if (__builtin_available(macOS 13.0, *))
+    {
+        if (CFDictionaryContainsKey(supportedProps, kVTCompressionPropertyKey_PreserveDynamicHDRMetadata))
+        {
+            err = VTSessionSetProperty(pv->session,
+                                       kVTCompressionPropertyKey_PreserveDynamicHDRMetadata,
+                                       pv->settings.preserveDynamicHDRMetadata);
+            if (err != noErr)
+            {
+                hb_log("VTSessionSetProperty: kVTCompressionPropertyKey_PreserveDynamicHDRMetadata failed");
+            }
+        }
+    }
+
     CFNumberRef parNum = CFNumberCreate(kCFAllocatorDefault,
                                         kCFNumberSInt32Type,
                                         &pv->settings.par.num);
@@ -1190,9 +1123,9 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
     }
 
     // Multi-pass
-    if (job->pass_id == HB_PASS_ENCODE_1ST && cookieOnly == 0)
+    if (job->pass_id == HB_PASS_ENCODE_ANALYSIS && cookieOnly == 0)
     {
-        const char *filename = hb_get_temporary_filename("videotoolbox.log");
+        char *filename = hb_get_temporary_filename("videotoolbox.log");
 
         CFStringRef path = CFStringCreateWithCString(kCFAllocatorDefault, filename, kCFStringEncodingUTF8);
         CFURLRef url = CFURLCreateWithFileSystemPath(NULL, path, kCFURLPOSIXPathStyle, FALSE);
@@ -1219,8 +1152,9 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
             }
         }
 
-        CFRelease(path);
         CFRelease(url);
+        CFRelease(path);
+        free(filename);
     }
 
     err = VTCompressionSessionPrepareToEncodeFrames(pv->session);
@@ -1243,7 +1177,9 @@ static OSStatus init_vtsession(hb_work_object_t *w, hb_job_t *job, hb_work_priva
     {
         if (CFBooleanGetValue(allowFrameReordering))
         {
-            job->areBframes = job->vcodec == HB_VCODEC_VT_H265 || job->vcodec == HB_VCODEC_VT_H265_10BIT ? 2 : 1;
+            // There is no way to know if b-pyramid will be
+            // used or not, to be safe always assume it's enabled
+            job->areBframes = 2;
         }
         CFRelease(allowFrameReordering);
     }
@@ -1333,6 +1269,7 @@ static OSStatus create_cookie(hb_work_object_t *w, hb_job_t *job, hb_work_privat
     if (pool == NULL)
     {
         hb_log("VTCompressionSession: VTCompressionSessionGetPixelBufferPool error");
+        return -1;
     }
 
     err = CVPixelBufferPoolCreatePixelBuffer(NULL, pool, &pix_buf);
@@ -1408,7 +1345,7 @@ static OSStatus reuse_vtsession(hb_work_object_t *w, hb_job_t * job, hb_work_pri
     OSStatus err = noErr;
 
     hb_interjob_t *interjob = hb_interjob_get(job->h);
-    vt_interjob_t *context = interjob->vt_context;
+    vt_interjob_t *context = interjob->context;
 
     if (job->vcodec == HB_VCODEC_VT_H264)
     {
@@ -1419,9 +1356,10 @@ static OSStatus reuse_vtsession(hb_work_object_t *w, hb_job_t * job, hb_work_pri
         set_h265_cookie(w, context->format);
     }
 
-    pv->queue = context->queue;
     pv->session = context->session;
     pv->passStorage = context->passStorage;
+    pv->queue = context->queue;
+    pv->format = context->format;
 
     if (err != noErr)
     {
@@ -1467,8 +1405,8 @@ static OSStatus reuse_vtsession(hb_work_object_t *w, hb_job_t * job, hb_work_pri
         CFRelease(allowFrameReordering);
     }
 
-    interjob->vt_context = NULL;
     free(context);
+    interjob->context = NULL;
 
     return err;
 }
@@ -1496,9 +1434,9 @@ int encvt_init(hb_work_object_t *w, hb_job_t *job)
         return -1;
     }
 
-    pv->remainingPasses = job->pass_id == HB_PASS_ENCODE_1ST ? 1 : 0;
+    pv->remainingPasses = job->pass_id == HB_PASS_ENCODE_ANALYSIS ? 1 : 0;
 
-    if (job->pass_id != HB_PASS_ENCODE_2ND)
+    if (job->pass_id != HB_PASS_ENCODE_FINAL)
     {
         err = create_cookie(w, job, pv);
         if (err != noErr)
@@ -1542,6 +1480,24 @@ void encvt_close(hb_work_object_t * w)
 
     hb_chapter_queue_close(&pv->chapter_queue);
 
+    // A cancelled encode doesn't send an EOF,
+    // do some additional cleanups here
+    if (*pv->job->die)
+    {
+        if (pv->session)
+        {
+            VTCompressionSessionCompleteFrames(pv->session, kCMTimeIndefinite);
+        }
+        if (pv->queue)
+        {
+            CMSampleBufferRef sampleBuffer;
+            while ((sampleBuffer = (CMSampleBufferRef)CMSimpleQueueDequeue(pv->queue)))
+            {
+                CFRelease(sampleBuffer);
+            }
+        }
+    }
+
     if (pv->remainingPasses == 0 || *pv->job->die)
     {
         if (pv->session)
@@ -1572,6 +1528,10 @@ void encvt_close(hb_work_object_t * w)
     {
         CFRelease(pv->settings.color.contentLightLevel);
     }
+    if (pv->settings.color.ambientViewingEnviroment)
+    {
+        CFRelease(pv->settings.color.ambientViewingEnviroment);
+    }
 
     free(pv);
     w->private_data = NULL;
@@ -1593,13 +1553,33 @@ static hb_buffer_t * extract_buf(CMSampleBufferRef sampleBuffer, hb_work_object_
     if (buffer)
     {
         size_t sampleSize = CMBlockBufferGetDataLength(buffer);
-        buf = hb_buffer_init(sampleSize);
+        Boolean isContiguous = CMBlockBufferIsRangeContiguous(buffer, 0, sampleSize);
 
-        err = CMBlockBufferCopyDataBytes(buffer, 0, sampleSize, buf->data);
-
-        if (err != kCMBlockBufferNoErr)
+        if (isContiguous)
         {
-            hb_log("VTCompressionSession: CMBlockBufferCopyDataBytes error");
+            size_t lengthAtOffsetOut, totalLengthOut;
+            char * _Nullable dataPointerOut;
+
+            buf = hb_buffer_wrapper_init();
+            err = CMBlockBufferGetDataPointer(buffer, 0, &lengthAtOffsetOut, &totalLengthOut, &dataPointerOut);
+            if (err != kCMBlockBufferNoErr)
+            {
+                hb_log("VTCompressionSession: CMBlockBufferGetDataPointer error");
+            }
+            buf->data = (uint8_t *)dataPointerOut;
+            buf->size = totalLengthOut;
+            buf->storage = sampleBuffer;
+            buf->storage_type = COREMEDIA;
+            CFRetain(sampleBuffer);
+        }
+        else
+        {
+            buf = hb_buffer_init(sampleSize);
+            err = CMBlockBufferCopyDataBytes(buffer, 0, sampleSize, buf->data);
+            if (err != kCMBlockBufferNoErr)
+            {
+                hb_log("VTCompressionSession: CMBlockBufferCopyDataBytes error");
+            }
         }
 
         buf->s.frametype = HB_FRAME_IDR;
@@ -1688,7 +1668,10 @@ static hb_buffer_t *vt_encode(hb_work_object_t *w, hb_buffer_t *in)
     else
     {
         CFDictionaryRef frameProperties = NULL;
-        if (in->s.new_chap && job->chapter_markers)
+        // macOS Sonoma has got an unfixed bug that makes the whole
+        // system crash and restart on M* Ultra if we force a keyframe
+        // on the first frame. So avoid that.
+        if (in->s.new_chap && job->chapter_markers && pv->frameno_in)
         {
             // chapters have to start with an IDR frame
             const void *keys[1] = { kVTEncodeFrameOptionKey_ForceKeyFrame };
@@ -1712,8 +1695,9 @@ static hb_buffer_t *vt_encode(hb_work_object_t *w, hb_buffer_t *in)
                                               CMTimeMake(in->s.start, pv->settings.timescale),
                                               CMTimeMake(in->s.duration, pv->settings.timescale),
                                               frameProperties,
-                                              pix_buffer,
+                                              in,
                                               NULL);
+        CVPixelBufferRelease(pix_buffer);
 
         if (err)
         {
@@ -1783,7 +1767,7 @@ int encvt_work(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
 
         hb_job_t *job = pv->job;
 
-        if (job->pass_id == HB_PASS_ENCODE_1ST)
+        if (job->pass_id == HB_PASS_ENCODE_ANALYSIS)
         {
             OSStatus err = noErr;
             Boolean furtherPassesRequestedOut;
@@ -1794,6 +1778,10 @@ int encvt_work(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
             {
                 hb_log("VTCompressionSessionEndPass error");
             }
+            if (furtherPassesRequestedOut == false)
+            {
+                hb_log("VTCompressionSessionEndPass: no additional pass requested");
+            }
 
             // Save the sessions and the related context
             // for the next pass.
@@ -1804,13 +1792,11 @@ int encvt_work(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
             context->format = pv->format;
 
             hb_interjob_t *interjob = hb_interjob_get(job->h);
-            interjob->vt_context = context;
+            interjob->context = context;
         }
-        else if (job->pass_id == HB_PASS_ENCODE_2ND)
+        else if (job->pass_id == HB_PASS_ENCODE_FINAL)
         {
-            VTCompressionSessionEndPass(pv->session,
-                                        NULL,
-                                        0);
+            VTCompressionSessionEndPass(pv->session, NULL, 0);
         }
 
         return HB_WORK_DONE;
