@@ -1,6 +1,6 @@
 /* ports.c
 
-   Copyright (c) 2003-2022 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -13,17 +13,11 @@
 #define _WIN32_WINNT 0x600
 #endif
 
-#ifdef USE_PTHREAD
 #ifdef SYS_LINUX
 #define _GNU_SOURCE
 #include <sched.h>
 #endif
 #include <pthread.h>
-#endif
-
-#ifdef SYS_BEOS
-#include <kernel/OS.h>
-#endif
 
 #if defined(SYS_DARWIN) || defined(SYS_FREEBSD) || defined(SYS_NETBSD) || defined(SYS_OPENBSD)
 #include <sys/types.h>
@@ -109,35 +103,7 @@ int gettimeofday( struct timeval * tv, struct timezone * tz )
 #endif
 */
 
-// Convert utf8 string to current code page.
-// The internal string representation in hb is utf8. But some
-// libraries (libmkv, and mp4v2) expect filenames in the current
-// code page.  So we must convert.
-char * hb_utf8_to_cp(const char *src)
-{
-    char *dst = NULL;
-
-#if defined( SYS_MINGW )
-    int num_chars = MultiByteToWideChar(CP_UTF8, 0, src, -1, NULL, 0);
-    if (num_chars <= 0)
-        return NULL;
-    wchar_t * tmp = calloc(num_chars, sizeof(wchar_t));
-    MultiByteToWideChar(CP_UTF8, 0, src, -1, tmp, num_chars);
-    int len = WideCharToMultiByte(GetACP(), 0, tmp, num_chars, NULL, 0, NULL, NULL);
-    if (len <= 0)
-        return NULL;
-    dst = calloc(len, sizeof(char));
-    WideCharToMultiByte(GetACP(), 0, tmp, num_chars, dst, len, NULL, NULL);
-    free(tmp);
-#else
-    // Other platforms don't have code pages
-    dst = strdup(src);
-#endif
-
-    return dst;
-}
-
-int hb_dvd_region(char *device, int *region_mask)
+int hb_dvd_region(const char *device, int *region_mask)
 {
 #if defined( DVD_LU_SEND_RPC_STATE ) && defined( DVD_AUTH )
     struct stat  st;
@@ -210,13 +176,10 @@ void hb_snooze( int delay )
     {
         return;
     }
-#if defined( SYS_BEOS )
-    snooze( 1000 * delay );
-#elif defined( SYS_DARWIN ) || defined( SYS_LINUX ) || defined( SYS_FREEBSD) || \
-      defined(SYS_NETBSD) || defined(SYS_OPENBSD) || defined( SYS_SunOS )
-    usleep( 1000 * delay );
-#elif defined( SYS_CYGWIN ) || defined( SYS_MINGW )
+#if defined( SYS_CYGWIN ) || defined( SYS_MINGW )
     Sleep( delay );
+#else
+    usleep( 1000 * delay );
 #endif
 }
 
@@ -239,21 +202,25 @@ struct
 
 int hb_get_cpu_count()
 {
+    init_cpu_info();
     return hb_cpu_info.count;
 }
 
 int hb_get_cpu_platform()
 {
+    init_cpu_info();
     return hb_cpu_info.platform;
 }
 
 const char* hb_get_cpu_name()
 {
+    init_cpu_info();
     return hb_cpu_info.name;
 }
 
 const char* hb_get_cpu_platform_name()
 {
+    init_cpu_info();
     switch (hb_cpu_info.platform)
     {
         case HB_CPU_PLATFORM_INTEL_BNL:
@@ -307,13 +274,16 @@ const char* hb_get_cpu_platform_name()
 
 static void init_cpu_info()
 {
-    hb_cpu_info.name     = NULL;
+    if (hb_cpu_info.count != 0)
+        return;
+
+    hb_cpu_info.name     = "Unknown";
     hb_cpu_info.count    = init_cpu_count();
     hb_cpu_info.platform = HB_CPU_PLATFORM_UNSPECIFIED;
 
+#if ARCH_X86_64 || ARCH_X86_32
     if (av_get_cpu_flags() & AV_CPU_FLAG_SSE)
     {
-#if ARCH_X86_64 || ARCH_X86_32
         int eax, ebx, ecx, edx, family, model;
 
         cpuid(1, &eax, &ebx, &ecx, &edx);
@@ -420,7 +390,14 @@ static void init_cpu_info()
                   &hb_cpu_info.buf4[11]);
 
             hb_cpu_info.name    = hb_cpu_info.buf;
-            hb_cpu_info.buf[47] = '\0'; // just in case
+
+            // ensure string is null-terminated and trim trailing whitespace
+            int ii = sizeof(hb_cpu_info.buf) - 1;
+            do {
+                hb_cpu_info.buf[ii] = '\0';
+                ii -= 1;
+            }
+            while (ii > 0 && isspace(hb_cpu_info.buf[ii]));
 
             while (isspace(*hb_cpu_info.name))
             {
@@ -428,8 +405,8 @@ static void init_cpu_info()
                 hb_cpu_info.name++;
             }
         }
-#endif // ARCH_X86_64 || ARCH_X86_32
     }
+#endif // ARCH_X86_64 || ARCH_X86_32
 }
 
 /*
@@ -453,11 +430,6 @@ static int init_cpu_count()
     for( cpu_count = 0, bit = 0; bit < sizeof(p_aff); bit++ )
          cpu_count += (((uint8_t *)&p_aff)[bit / 8] >> (bit % 8)) & 1;
 
-#elif defined(SYS_BEOS)
-    system_info info;
-    get_system_info( &info );
-    cpu_count = info.cpu_count;
-
 #elif defined(SYS_DARWIN) || defined(SYS_FREEBSD) || defined(SYS_NETBSD) || defined(SYS_OPENBSD)
     size_t length = sizeof( cpu_count );
 #if defined SYS_OPENBSD || defined(SYS_NETBSD)
@@ -828,14 +800,7 @@ struct hb_thread_s
 
     hb_lock_t     * lock;
     int             exited;
-
-#if defined( SYS_BEOS )
-    thread_id       thread;
-#elif USE_PTHREAD
     pthread_t       thread;
-//#elif defined( SYS_CYGWIN )
-//    HANDLE          thread;
-#endif
 };
 
 /* Get a unique identifier to thread and represent as 64-bit unsigned.
@@ -844,22 +809,18 @@ struct hb_thread_s
  */
 static uint64_t hb_thread_to_integer( const hb_thread_t* t )
 {
-#if defined( USE_PTHREAD )
-    #if defined( SYS_CYGWIN )
-        return (uint64_t)t->thread;
-    #elif defined( _WIN32 ) || defined( __MINGW32__ )
-    #if defined(PTW32_VERSION)
-        return (uint64_t)(ptrdiff_t)t->thread.p;
-    #else
-        return (uint64_t)t->thread;
-    #endif
-    #elif defined( SYS_DARWIN )
-        return (unsigned long)t->thread;
-    #else
-        return (uint64_t)t->thread;
-    #endif
+#if defined( SYS_CYGWIN )
+    return (uint64_t)t->thread;
+#elif defined( _WIN32 ) || defined( __MINGW32__ )
+# if defined(PTW32_VERSION)
+    return (uint64_t)(ptrdiff_t)t->thread.p;
+# else
+    return (uint64_t)t->thread;
+# endif
+#elif defined( SYS_DARWIN )
+    return (unsigned long)t->thread;
 #else
-    return 0;
+    return (uint64_t)t->thread;
 #endif
 }
 
@@ -878,8 +839,7 @@ static void attribute_align_thread hb_thread_func( void * _t )
     hb_thread_t * t = (hb_thread_t *) _t;
 
 #if (defined( SYS_DARWIN ) && !defined(__aarch64__)) || defined( SYS_FREEBSD ) || \
-    defined ( __FreeBSD__ ) || defined(SYS_NETBSD) || defined( SYS_OPENBSD ) || \
-    defined ( __OpenBSD__ )
+    defined( SYS_NETBSD ) || defined( SYS_OPENBSD )
     /* Set the thread priority
        Do not change priority on Darwin arm systems */
     struct sched_param param;
@@ -892,10 +852,6 @@ static void attribute_align_thread hb_thread_func( void * _t )
     pthread_setname_np( t->name );
 #endif
 
-#if defined( SYS_BEOS )
-    signal( SIGINT, SIG_IGN );
-#endif
-
     /* Start the actual routine */
     t->function( t->arg );
 
@@ -927,24 +883,9 @@ hb_thread_t * hb_thread_init( const char * name, void (* function)(void *),
     t->lock     = hb_lock_init();
 
     /* Create and start the thread */
-#if defined( SYS_BEOS )
-    t->thread = spawn_thread( (thread_func) hb_thread_func,
-                              name, priority, t );
-    resume_thread( t->thread );
-
-#elif USE_PTHREAD
     pthread_create( &t->thread, NULL,
                     (void * (*)( void * )) hb_thread_func, t );
 
-//#elif defined( SYS_CYGWIN )
-//    t->thread = CreateThread( NULL, 0,
-//        (LPTHREAD_START_ROUTINE) hb_thread_func, t, 0, NULL );
-//
-//    /* Maybe use THREAD_PRIORITY_LOWEST instead */
-//    if( priority == HB_LOW_PRIORITY )
-//        SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL );
-#endif
-
     hb_deep_log( 2, "thread %"PRIx64" started (\"%s\")", hb_thread_to_integer( t ), t->name );
     return t;
 }
@@ -959,17 +900,8 @@ void hb_thread_close( hb_thread_t ** _t )
     hb_thread_t * t = *_t;
 
     /* Join the thread */
-#if defined( SYS_BEOS )
-    long exit_value;
-    wait_for_thread( t->thread, &exit_value );
-
-#elif USE_PTHREAD
     pthread_join( t->thread, NULL );
 
-//#elif defined( SYS_CYGWIN )
-//    WaitForSingleObject( t->thread, INFINITE );
-#endif
-
     hb_deep_log( 2, "thread %"PRIx64" joined (\"%s\")", hb_thread_to_integer( t ), t->name );
 
     hb_lock_close( &t->lock );
@@ -999,13 +931,7 @@ int hb_thread_has_exited( hb_thread_t * t )
  ***********************************************************************/
 struct hb_lock_s
 {
-#if defined( SYS_BEOS )
-    sem_id          sem;
-#elif USE_PTHREAD
     pthread_mutex_t mutex;
-//#elif defined( SYS_CYGWIN )
-//    HANDLE          mutex;
-#endif
 };
 
 /************************************************************************
@@ -1020,22 +946,16 @@ hb_lock_t * hb_lock_init()
 {
     hb_lock_t * l = calloc( sizeof( hb_lock_t ), 1 );
 
-#if defined( SYS_BEOS )
-    l->sem = create_sem( 1, "sem" );
-#elif USE_PTHREAD
     pthread_mutexattr_t mta;
 
     pthread_mutexattr_init(&mta);
 
-#if defined( SYS_CYGWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || \
-    defined(SYS_NETBSD) || defined( SYS_OPENBSD ) || defined ( __OpenBSD__ )
+#if defined( SYS_CYGWIN ) || defined( SYS_FREEBSD ) || defined( SYS_NETBSD ) || \
+    defined( SYS_OPENBSD )
     pthread_mutexattr_settype(&mta, PTHREAD_MUTEX_NORMAL);
 #endif
 
     pthread_mutex_init( &l->mutex, &mta );
-//#elif defined( SYS_CYGWIN )
-//    l->mutex = CreateMutex( 0, FALSE, 0 );
-#endif
 
     return l;
 }
@@ -1048,13 +968,7 @@ void hb_lock_close( hb_lock_t ** _l )
     {
         return;
     }
-#if defined( SYS_BEOS )
-    delete_sem( l->sem );
-#elif USE_PTHREAD
     pthread_mutex_destroy( &l->mutex );
-//#elif defined( SYS_CYGWIN )
-//    CloseHandle( l->mutex );
-#endif
     free( l );
 
     *_l = NULL;
@@ -1062,24 +976,12 @@ void hb_lock_close( hb_lock_t ** _l )
 
 void hb_lock( hb_lock_t * l )
 {
-#if defined( SYS_BEOS )
-    acquire_sem( l->sem );
-#elif USE_PTHREAD
     pthread_mutex_lock( &l->mutex );
-//#elif defined( SYS_CYGWIN )
-//    WaitForSingleObject( l->mutex, INFINITE );
-#endif
 }
 
 void hb_unlock( hb_lock_t * l )
 {
-#if defined( SYS_BEOS )
-    release_sem( l->sem );
-#elif USE_PTHREAD
     pthread_mutex_unlock( &l->mutex );
-//#elif defined( SYS_CYGWIN )
-//    ReleaseMutex( l->mutex );
-#endif
 }
 
 /************************************************************************
@@ -1087,13 +989,7 @@ void hb_unlock( hb_lock_t * l )
  ***********************************************************************/
 struct hb_cond_s
 {
-#if defined( SYS_BEOS )
-    int                 thread;
-#elif USE_PTHREAD
     pthread_cond_t      cond;
-//#elif defined( SYS_CYGWIN )
-//    HANDLE              event;
-#endif
 };
 
 /************************************************************************
@@ -1112,14 +1008,7 @@ hb_cond_t * hb_cond_init()
     if( c == NULL )
         return NULL;
 
-#if defined( SYS_BEOS )
-    c->thread = -1;
-#elif USE_PTHREAD
     pthread_cond_init( &c->cond, NULL );
-//#elif defined( SYS_CYGWIN )
-//    c->event = CreateEvent( NULL, FALSE, FALSE, NULL );
-#endif
-
     return c;
 }
 
@@ -1131,12 +1020,7 @@ void hb_cond_close( hb_cond_t ** _c )
     {
         return;
     }
-#if defined( SYS_BEOS )
-#elif USE_PTHREAD
     pthread_cond_destroy( &c->cond );
-//#elif defined( SYS_CYGWIN )
-//    CloseHandle( c->event );
-#endif
     free( c );
 
     *_c = NULL;
@@ -1144,18 +1028,7 @@ void hb_cond_close( hb_cond_t ** _c )
 
 void hb_cond_wait( hb_cond_t * c, hb_lock_t * lock )
 {
-#if defined( SYS_BEOS )
-    c->thread = find_thread( NULL );
-    release_sem( lock->sem );
-    suspend_thread( c->thread );
-    acquire_sem( lock->sem );
-    c->thread = -1;
-#elif USE_PTHREAD
     pthread_cond_wait( &c->cond, &lock->mutex );
-//#elif defined( SYS_CYGWIN )
-//    SignalObjectAndWait( lock->mutex, c->event, INFINITE, FALSE );
-//    WaitForSingleObject( lock->mutex, INFINITE );
-#endif
 }
 
 void hb_clock_gettime( struct timespec *tp )
@@ -1174,51 +1047,22 @@ void hb_yield(void)
 
 void hb_cond_timedwait( hb_cond_t * c, hb_lock_t * lock, int msec )
 {
-#if defined( SYS_BEOS )
-    c->thread = find_thread( NULL );
-    release_sem( lock->sem );
-    suspend_thread( c->thread );
-    acquire_sem( lock->sem );
-    c->thread = -1;
-#elif USE_PTHREAD
     struct timespec ts;
     hb_clock_gettime(&ts);
     ts.tv_nsec += (msec % 1000) * 1000000;
     ts.tv_sec += msec / 1000 + (ts.tv_nsec / 1000000000);
     ts.tv_nsec %= 1000000000;
     pthread_cond_timedwait( &c->cond, &lock->mutex, &ts );
-#endif
 }
 
 void hb_cond_signal( hb_cond_t * c )
 {
-#if defined( SYS_BEOS )
-    while( c->thread != -1 )
-    {
-        thread_info info;
-        get_thread_info( c->thread, &info );
-        if( info.state == B_THREAD_SUSPENDED )
-        {
-            resume_thread( c->thread );
-            break;
-        }
-        /* Looks like we have been called between hb_cond_wait's
-           release_sem() and suspend_thread() lines. Wait until the
-           thread is actually suspended before we resume it */
-        snooze( 5000 );
-    }
-#elif USE_PTHREAD
     pthread_cond_signal( &c->cond );
-//#elif defined( SYS_CYGWIN )
-//    PulseEvent( c->event );
-#endif
 }
 
 void hb_cond_broadcast( hb_cond_t * c )
 {
-#if USE_PTHREAD
     pthread_cond_broadcast( &c->cond );
-#endif
 }
 
 /************************************************************************
@@ -1587,10 +1431,20 @@ static int try_adapter(const char * name, const char * dir,
     return -1;
 }
 
-static int open_adapter(const char * name)
+static int open_adapter(const char * name, const uint dri_render_node)
 {
-    int fd = try_adapter(name, DRI_PATH, DRI_NODE_RENDER,
-                         DRI_RENDER_NODE_START, DRI_RENDER_NODE_LAST);
+    int fd;
+    // If dri_render_node is unknown enumerate across the predifined range of renders
+    if (dri_render_node == 0)
+    {
+        fd = try_adapter(name, DRI_PATH, DRI_NODE_RENDER,
+                            DRI_RENDER_NODE_START, DRI_RENDER_NODE_LAST);
+    }
+    else
+    {
+        fd = try_adapter(name, DRI_PATH, DRI_NODE_RENDER,
+                            dri_render_node, dri_render_node);
+    }
     if (fd < 0)
     {
         fd = try_adapter(name, DRI_PATH, DRI_NODE_CARD,
@@ -1626,7 +1480,8 @@ static int try_va_interface(hb_display_t * hbDisplay,
     return 0;
 }
 
-hb_display_t * hb_display_init(const char         *  driver_name,
+hb_display_t * hb_display_init(const char         * driver_name,
+                               const uint32_t       dri_render_node,
                                const char * const * interface_names)
 {
     hb_display_t * hbDisplay = calloc(sizeof(hb_display_t), 1);
@@ -1634,7 +1489,7 @@ hb_display_t * hb_display_init(const char         *  driver_name,
     int            ii;
 
     hbDisplay->vaDisplay = NULL;
-    hbDisplay->vaFd      = open_adapter(driver_name);
+    hbDisplay->vaFd      = open_adapter(driver_name, dri_render_node);
     if (hbDisplay->vaFd < 0)
     {
         hb_deep_log( 3, "hb_va_display_init: no display found" );
@@ -1701,7 +1556,8 @@ void hb_display_close(hb_display_t ** _d)
 
 #else // !SYS_LINUX && !SYS_FREEBSD
 
-hb_display_t * hb_display_init(const char         *  driver_name,
+hb_display_t * hb_display_init(const char         * driver_name,
+                               const uint32_t       dri_render_node,
                                const char * const * interface_names)
 {
     return NULL;
@@ -1715,7 +1571,8 @@ void hb_display_close(hb_display_t ** _d)
 #endif // SYS_LINUX || SYS_FREEBSD
 #else // !HB_PROJECT_FEATURE_QSV
 
-hb_display_t * hb_display_init(const char         *  driver_name,
+hb_display_t * hb_display_init(const char         * driver_name,
+                               const uint32_t       dri_render_node,
                                const char * const * interface_names)
 {
     return NULL;
